@@ -5,59 +5,37 @@
  */
 
 #include "fd.h"
+#include "device.h"
+#include "parse.h"
 #include "func.h"
 #include "funcno.h"
 #include "kanji.h"
 
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 #include "termemu.h"
 #endif
 
+#define	CL_REG			0
+#define	CL_BACK			1
+#define	CL_DIR			2
+#define	CL_RONLY		3
+#define	CL_HIDDEN		4
+#define	CL_LINK			5
+#define	CL_SOCK			6
+#define	CL_FIFO			7
+#define	CL_BLOCK		8
+#define	CL_CHAR			9
+#define	CL_EXE			10
+#define	ANSI_FG			8
+#define	ANSI_BG			9
+
 #if	MSDOS
 extern int setcurdrv __P_((int, int));
 #endif
 
-#ifdef	USEMKDEVH
-#include <sys/mkdev.h>
-#else
-# ifdef	USEMKNODH
-# include <sys/mknod.h>
-# else
-#  ifdef	SVR4
-#  include <sys/sysmacros.h>
-#   ifndef	major
-#   define	major(n)	((((u_long)(n)) >> 18) & 0x3fff)
-#   endif
-#   ifndef	minor
-#   define	minor(n)	(((u_long)(n)) & 0x3ffff)
-#   endif
-#  else
-#   ifndef	major
-#   define	major(n)	((((u_int)(n)) >> 8) & 0xff)
-#   endif
-#   ifndef	minor
-#   define	minor(n)	(((u_int)(n)) & 0xff)
-#   endif
-#  endif
-# endif
-#endif
-
-#define	CL_REG		0
-#define	CL_BACK		1
-#define	CL_DIR		2
-#define	CL_RONLY	3
-#define	CL_HIDDEN	4
-#define	CL_LINK		5
-#define	CL_SOCK		6
-#define	CL_FIFO		7
-#define	CL_BLOCK	8
-#define	CL_CHAR		9
-#define	CL_EXE		10
-#define	ANSI_FG		8
-#define	ANSI_BG		9
-
-extern bindtable bindlist[];
-extern functable funclist[];
+extern bindlist_t bindlist;
+extern int maxbind;
+extern CONST functable funclist[];
 #ifndef	_NOWRITEFS
 extern int writefs;
 #endif
@@ -74,10 +52,6 @@ extern int custno;
 extern int internal_status;
 extern int hideclock;
 extern int fd_restricted;
-#ifndef	_NOPTY
-extern int ptymode;
-extern int parentfd;
-#endif
 
 #ifndef	_NOCOLOR
 static int NEAR getcolorid __P_((namelist *));
@@ -85,6 +59,7 @@ static int NEAR biascolor __P_((int));
 static int NEAR getcolor __P_((int));
 #endif
 static VOID NEAR pathbar __P_((VOID_A));
+static CONST char *NEAR skipstr __P_((CONST char *, int));
 static VOID NEAR statusbar __P_((VOID_A));
 static VOID NEAR stackbar __P_((VOID_A));
 static VOID NEAR cputbytes __P_((off_t, off_t, int));
@@ -94,19 +69,19 @@ static int NEAR putsize2 __P_((char *, namelist *, int));
 static int NEAR putfilename __P_((char *, namelist *, int));
 static VOID NEAR infobar __P_((VOID_A));
 static int NEAR calclocate __P_((int));
-static int NEAR calcfilepos __P_((namelist *, int, char *));
-static int NEAR listupmyself __P_((char *));
+static int NEAR calcfilepos __P_((namelist *, int, CONST char *));
+static int NEAR listupmyself __P_((CONST char *));
 #ifndef	_NOSPLITWIN
-static int NEAR listupwin __P_((char *));
+static int NEAR listupwin __P_((CONST char *));
 #endif
 static int NEAR searchmove __P_((int, char *));
 #ifndef	_NOPRECEDE
 static int readstatus __P_((VOID_A));
 #endif
-static int NEAR readfilelist __P_((reg_t *, char *));
+static int NEAR readfilelist __P_((CONST reg_t *, CONST char *));
 static VOID NEAR getfilelist __P_((VOID_A));
-static int NEAR browsedir __P_((char *, char *));
-static char *NEAR initcwd __P_((char *, char *, int));
+static int NEAR browsedir __P_((char *, CONST char *));
+static char *NEAR initcwd __P_((CONST char *, char *, int));
 
 int curcolumns = 0;
 int defcolumns = 0;
@@ -127,13 +102,24 @@ off_t totalsize = (off_t)0;
 off_t blocksize = (off_t)0;
 namelist filestack[MAXSTACK];
 char fullpath[MAXPATHLEN] = "";
-char *macrolist[MAXMACROTABLE];
 int maxmacro = 0;
 int isearch = 0;
+int autoupdate = 0;
 #if	FD >= 2
 int helplayout = 0;
 #endif
-char *helpindex[MAXHELPINDEX] = {
+#ifdef	DEP_DYNAMICLIST
+macrolist_t macrolist = NULL;
+helpindex_t helpindex = NULL;
+orighelpindex_t orighelpindex
+#else	/* !DEP_DYNAMICLIST */
+macrolist_t macrolist = { NULL };
+# ifndef	_NOCUSTOMIZE
+orighelpindex_t orighelpindex = NULL;
+# endif
+helpindex_t helpindex
+#endif	/* !DEP_DYNAMICLIST */
+= {
 #ifdef	_NOTREE
 	"help",
 #else
@@ -173,8 +159,8 @@ char *helpindex[MAXHELPINDEX] = {
 #endif	/* FD >= 2 */
 };
 #ifdef	HAVEFLAGS
-char fflagsymlist[] = "ANacuacu";
-u_long fflaglist[] = {
+CONST char fflagsymlist[] = "ANacuacu";
+CONST u_long fflaglist[] = {
 	SF_ARCHIVED, UF_NODUMP,
 	SF_APPEND, SF_IMMUTABLE, SF_NOUNLINK,
 	UF_APPEND, UF_IMMUTABLE, UF_NOUNLINK
@@ -191,24 +177,20 @@ char *precedepath = NULL;
 #ifndef	_NOSPLITWIN
 int windows = 1;
 #endif
-#if	!defined (_NOSPLITWIN) || !defined (_NOPTY)
+#if	!defined (_NOSPLITWIN) || defined (DEP_PTY)
 int win = 0;
 #endif
 int calc_x = -1;
 int calc_y = -1;
 
-static CONST char typesymlist[] = "dbclsp";
-static CONST u_short typelist[] = {
-	S_IFDIR, S_IFBLK, S_IFCHR, S_IFLNK, S_IFSOCK, S_IFIFO
-};
 static CONST u_short modelist[] = {
 	S_IFDIR, S_IFLNK, S_IFSOCK, S_IFIFO, S_IFBLK, S_IFCHR
 };
-#define	MAXMODELIST	arraysize(modelist)
+#define	MODELISTSIZ		arraysize(modelist)
 static CONST char suffixlist[] = {
 	'/', '@', '=', '|'
 };
-#define	MAXSUFFIXLIST	arraysize(suffixlist)
+#define	SUFFIXLISTSIZ		arraysize(suffixlist)
 #ifndef	_NOCOLOR
 static CONST u_char colorlist[] = {
 	CL_DIR, CL_LINK, CL_SOCK, CL_FIFO, CL_BLOCK, CL_CHAR
@@ -243,7 +225,7 @@ namelist *namep;
 
 	if (!isread(namep)) return(CL_HIDDEN);
 	if (!iswrite(namep)) return(CL_RONLY);
-	for (i = 0; i < MAXMODELIST; i++)
+	for (i = 0; i < MODELISTSIZ; i++)
 		if ((namep -> st_mode & S_IFMT) == modelist[i])
 			return(colorlist[i]);
 	if (isexec(namep)) return(CL_EXE);
@@ -382,9 +364,18 @@ VOID helpbar(VOID_A)
 	Xtflush();
 }
 
+static CONST char *NEAR skipstr(s, n)
+CONST char *s;
+int n;
+{
+	while (n-- > 0 && *s) s++;
+
+	return(s);
+}
+
 static VOID NEAR statusbar(VOID_A)
 {
-	char *str[6];
+	CONST char *str[6];
 
 #ifndef	_NOTRADLAYOUT
 	if (istradlayout()) {
@@ -434,18 +425,18 @@ static VOID NEAR statusbar(VOID_A)
 		if (haste) Xkanjiputs(OMIT_K);
 		else
 #endif
-		if (!(sorton & 7)) Xkanjiputs(ORAW_K + 3);
+		if (!(sorton & 7)) Xkanjiputs(skipstr(ORAW_K, 3));
 		else {
 			str[0] = ONAME_K;
 			str[1] = OEXT_K;
 			str[2] = OSIZE_K;
 			str[3] = ODATE_K;
 			str[4] = OLEN_K;
-			Xkanjiputs(&(str[(sorton & 7) - 1][3]));
+			Xkanjiputs(skipstr(str[(sorton & 7) - 1], 3));
 
 			str[0] = OINC_K;
 			str[1] = ODEC_K;
-			Xcprintf2("(%k)", &(str[sorton / 8][3]));
+			Xcprintf2("(%k)", skipstr(str[sorton / 8], 3));
 		}
 	}
 
@@ -591,30 +582,17 @@ static VOID NEAR sizebar(VOID_A)
 	Xtflush();
 }
 
-u_int getfmode(c)
-int c;
-{
-	int i;
-
-	c = tolower2(c);
-	for (i = 0; typesymlist[i]; i++)
-		if (c == typesymlist[i]) return(typelist[i]);
-
-	return((u_int)-1);
-}
-
 int putmode(buf, mode, notype)
 char *buf;
 u_int mode;
 int notype;
 {
-	int i, j;
+	int i, c;
 
 	i = 0;
 	if (!notype) {
-		for (j = 0; j < arraysize(typelist); j++)
-			if ((mode & S_IFMT) == typelist[j]) break;
-		buf[i++] = (j < arraysize(typelist)) ? typesymlist[j] : '-';
+		if (!(c = getfsymbol(mode))) c = '-';
+		buf[i++] = c;
 	}
 
 	buf[i++] = (mode & S_IRUSR) ? 'r' : '-';
@@ -698,7 +676,7 @@ int width, max;
 	snprintf2(tmp, sizeof(tmp), "%<*qd", max, n);
 	for (i = max - width; i > 0; i--) if (tmp[i - 1] == ' ') break;
 	len = max - i;
-	strncpy(buf, &(tmp[i]), len);
+	strncpy2(buf, &(tmp[i]), len);
 
 	return(len);
 }
@@ -714,7 +692,7 @@ int width;
 	else if (s_isfifo(namep))
 		snprintf2(buf, width + 1, "%*.*s", width, width, "<VOL>");
 #else	/* !MSDOS */
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 	else if (dospath2(nullstr) && s_isfifo(namep))
 		snprintf2(buf, width + 1, "%*.*s", width, width, "<VOL>");
 # endif
@@ -738,7 +716,7 @@ int width;
 #ifdef	NOSYMLINK
 	strncpy3(buf, namep -> name, &width, fnameofs);
 #else	/* !NOSYMLINK */
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 	char path[MAXPATHLEN];
 # endif
 	char *tmp;
@@ -772,7 +750,7 @@ int width;
 			tmp[i] = '\0';
 			strncpy3(buf, tmp, &w, len);
 		}
-		free(tmp);
+		free2(tmp);
 	}
 #endif	/* !NOSYMLINK */
 
@@ -786,7 +764,7 @@ static VOID NEAR infobar(VOID_A)
 	int len;
 
 	if (!filelist || filepos < 0 || maxfile < 0) return;
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	if (parentfd >= 0) return;
 #endif
 
@@ -840,7 +818,7 @@ static VOID NEAR infobar(VOID_A)
 			? (S_IFLNK | 0777) : filelist[filepos].st_mode, 1);
 
 		Xkanjiputs(buf);
-		free(buf);
+		free2(buf);
 		Xtflush();
 		return;
 	}
@@ -927,7 +905,7 @@ static VOID NEAR infobar(VOID_A)
 
 	putfilename(&(buf[len]), &(filelist[filepos]), n_lastcolumn - len);
 	Xkanjiputs(buf);
-	free(buf);
+	free2(buf);
 	Xtflush();
 }
 
@@ -1053,16 +1031,16 @@ int no, isstandout;
 	if (!havestat(&(list[no]))) {
 		if (isstandout > 0) Xputterm(T_STANDOUT);
 		Xkanjiputs(buf);
-		free(buf);
+		free2(buf);
 		if (isstandout > 0) Xputterm(END_STANDOUT);
 		return(wid);
 	}
 #endif
 
 	if (isdisptyp(dispmode) && i < width) {
-		for (j = 0; j < MAXMODELIST; j++)
+		for (j = 0; j < MODELISTSIZ; j++)
 			if ((list[no].st_mode & S_IFMT) == modelist[j]) break;
-		if (j < MAXSUFFIXLIST) buf[i] = suffixlist[j];
+		if (j < SUFFIXLISTSIZ) buf[i] = suffixlist[j];
 		else if (s_isreg(&(list[no]))
 		&& (list[no].st_mode & S_IEXEC_ALL))
 			buf[i] = '*';
@@ -1118,7 +1096,7 @@ int no, isstandout;
 #endif
 	if (isstandout > 0) Xputterm(T_STANDOUT);
 	Xkanjiputs(buf);
-	free(buf);
+	free2(buf);
 #ifndef	_NOCOLOR
 	if ((bgcolor = getcolor(CL_BACK)) >= 0) {
 		Xchgcolor(bgcolor, 1);
@@ -1135,7 +1113,7 @@ int no, isstandout;
 static int NEAR calcfilepos(list, max, def)
 namelist *list;
 int max;
-char *def;
+CONST char *def;
 {
 	int i;
 
@@ -1151,10 +1129,10 @@ char *def;
 int listupfile(list, max, def, isstandout)
 namelist *list;
 int max;
-char *def;
+CONST char *def;
 int isstandout;
 {
-	char *cp;
+	CONST char *cp;
 	int i, n, pos;
 
 	for (i = 0; i < FILEPERROW; i++) {
@@ -1210,7 +1188,7 @@ int isstandout;
 }
 
 static int NEAR listupmyself(def)
-char *def;
+CONST char *def;
 {
 #ifndef	_NOTREE
 	if (treepath) rewritetree();
@@ -1220,7 +1198,7 @@ char *def;
 	if (custno >= 0) rewritecust();
 	else
 #endif
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	if (ptylist[win].pid && ptylist[win].status < 0) /*EMPTY*/;
 	else
 #endif
@@ -1232,7 +1210,7 @@ char *def;
 
 #ifndef	_NOSPLITWIN
 static int NEAR listupwin(def)
-char *def;
+CONST char *def;
 {
 	int i, x, y, n, dupwin;
 
@@ -1260,7 +1238,7 @@ char *def;
 			x = win_x;
 			y = win_y;
 		}
-# ifndef	_NOPTY
+# ifdef	DEP_PTY
 		else if (ptylist[win].pid && ptylist[win].status < 0) continue;
 # endif
 		else if (filelist
@@ -1293,27 +1271,20 @@ int n;
 		} while (browselist);
 #  endif
 	}
-	if (winvar[win].v_archivedir) {
-		free(winvar[win].v_archivedir);
-		winvar[win].v_archivedir = NULL;
-	}
+	free2(winvar[win].v_archivedir);
+	winvar[win].v_archivedir = NULL;
 # endif	/* !_NOARCHIVE */
-	if (winvar[win].v_fullpath) {
-		free(winvar[win].v_fullpath);
-		winvar[win].v_fullpath = NULL;
-	}
+	free2(winvar[win].v_fullpath);
+	winvar[win].v_fullpath = NULL;
 	if (filelist) {
-		for (i = 0; i < maxfile; i++)
-			if (filelist[i].name) free(filelist[i].name);
-		free(filelist);
+		for (i = 0; i < maxfile; i++) free2(filelist[i].name);
+		free2(filelist);
 		filelist = NULL;
 	}
 	maxfile = maxent = filepos = sorton = dispmode = 0;
-	if (findpattern) {
-		free(findpattern);
-		findpattern = NULL;
-	}
-# ifndef	_NOPTY
+	free2(findpattern);
+	findpattern = NULL;
+# ifdef	DEP_PTY
 	killpty(win, NULL);
 # endif
 	win = dupwin;
@@ -1366,7 +1337,7 @@ int all;
 	int x, y;
 
 	if (!filelist || filepos < 0 || maxfile < 0) return;
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	if (parentfd >= 0) return;
 #endif
 
@@ -1405,8 +1376,8 @@ static int NEAR searchmove(ch, buf)
 int ch;
 char *buf;
 {
-	char *str[4];
-	int i, n, s, pos, len;
+	CONST char *str[4];
+	int i, n, s, pos, len, func;
 
 	if (isearch > 0) {
 		n = isearch;
@@ -1419,18 +1390,18 @@ char *buf;
 	len = strlen(buf);
 	pos = filepos;
 
-	for (i = 0; i < MAXBINDTABLE && bindlist[i].key >= 0; i++)
-		if (ch == (int)(bindlist[i].key)) break;
+	for (i = 0; i < maxbind; i++) if (ch == (int)(bindlist[i].key)) break;
+	func = (i < maxbind) ? ffunc(i) : -1;
 	if (ch == K_BS) {
 		if (!len) isearch = 0;
 		else buf[--len] = '\0';
 	}
-	else if (len && bindlist[i].f_func == SEARCH_FORW) {
+	else if (len && func == SEARCH_FORW) {
 		if (n > 2) pos++;
 		else if (n > 1) pos = 0;
 		s = 1;
 	}
-	else if (len && bindlist[i].f_func == SEARCH_BACK) {
+	else if (len && func == SEARCH_BACK) {
 		if (n > 2) pos--;
 		else if (n > 1) pos = maxfile - 1;
 		s = -1;
@@ -1484,13 +1455,6 @@ char *buf;
 	return(i);
 }
 
-VOID addlist(VOID_A)
-{
-	if (maxfile < maxent) return;
-	maxent = ((maxfile + 1) / BUFUNIT + 1) * BUFUNIT;
-	filelist = (namelist *)realloc2(filelist, maxent * sizeof(namelist));
-}
-
 #ifndef	_NOPRECEDE
 static int readstatus(VOID_A)
 {
@@ -1519,8 +1483,8 @@ static int readstatus(VOID_A)
 #endif	/* !_NOPRECEDE */
 
 static int NEAR readfilelist(re, arcre)
-reg_t *re;
-char *arcre;
+CONST reg_t *re;
+CONST char *arcre;
 {
 #ifndef	_NOPRECEDE
 	char cwd[MAXPATHLEN];
@@ -1547,12 +1511,12 @@ char *arcre;
 		if (ishidedot(dispmode) && *(dp -> d_name) == '.'
 		&& !isdotdir(dp -> d_name))
 			continue;
-		strcpy(buf, dp -> d_name);
+		strcpy2(buf, dp -> d_name);
 		for (i = 0; i < stackdepth; i++)
 			if (!strcmp(buf, filestack[i].name)) break;
 		if (i < stackdepth) continue;
 
-		addlist();
+		filelist = addlist(filelist, maxfile, &maxent);
 		filelist[maxfile].name = strdup2(buf);
 		filelist[maxfile].ent = maxfile;
 		filelist[maxfile].tmpflags = 0;
@@ -1566,7 +1530,7 @@ char *arcre;
 		else
 #endif
 		if (getstatus(&(filelist[maxfile])) < 0) {
-			free(filelist[maxfile].name);
+			free2(filelist[maxfile].name);
 			continue;
 		}
 
@@ -1586,7 +1550,7 @@ static VOID NEAR getfilelist(VOID_A)
 	arcre = NULL;
 	if (findpattern) {
 #ifndef	_NOARCHIVE
-		if (*findpattern == '/') arcre = findpattern + 1;
+		if (*findpattern == '/') arcre = &(findpattern[1]);
 		else
 #endif
 		re = regexp_init(findpattern, -1);
@@ -1614,8 +1578,9 @@ static VOID NEAR getfilelist(VOID_A)
 	regexp_free(re);
 
 	if (!maxfile) {
-		addlist();
-		filelist[0].name = NOFIL_K;
+		filelist = addlist(filelist, maxfile, &maxent);
+		filelist[0].name = (char *)NOFIL_K;
+		filelist[0].tmpflags = 0;
 		filelist[0].st_nlink = -1;
 	}
 
@@ -1626,12 +1591,17 @@ static VOID NEAR getfilelist(VOID_A)
 }
 
 static int NEAR browsedir(file, def)
-char *file, *def;
+char *file;
+CONST char *def;
 {
+#ifndef	_NOARCHIVE
+	CONST char *tmp;
+#endif
 #ifndef	_NOPRECEDE
 	int dupsorton;
 #endif
-	char *cp, buf[MAXNAMLEN + 1];
+	CONST char *cp;
+	char buf[MAXNAMLEN + 1];
 	int ch, i, no, old, funcstat;
 
 #ifndef	_NOPRECEDE
@@ -1651,7 +1621,9 @@ char *file, *def;
 	waitmes();
 
 #ifndef	_NOARCHIVE
-	if (archivefile) def = (*file) ? file : nullstr;
+	if (!archivefile) /*EMPTY*/;
+	else if (*file) def = file;
+	else def = nullstr;
 #endif
 	getfilelist();
 
@@ -1686,34 +1658,42 @@ char *file, *def;
 #ifndef	_NOPRECEDE
 		if (haste) keywaitfunc = readstatus;
 #endif
-		Xgetkey(-1, 0);
-		ch = Xgetkey(1, 0);
-		Xgetkey(-1, 0);
+		Xgetkey(-1, 0, 0);
+		ch = Xgetkey(1, 0, autoupdate);
+		Xgetkey(-1, 0, 0);
 #ifndef	_NOPRECEDE
 		keywaitfunc = NULL;
 #endif
 
 		old = filepos;
 		if (isearch) {
+#if	FD >= 3
+			if (ch == K_TIMEOUT) {
+				no = FNC_NONE;
+				continue;
+			}
+#endif
 			no = searchmove(ch, buf);
 			if (no >= 0) {
 				fnameofs = 0;
 				continue;
 			}
 		}
-		for (i = 0; i < MAXBINDTABLE && bindlist[i].key >= 0; i++)
+		for (i = 0; i < maxbind; i++)
 			if (ch == (int)(bindlist[i].key)) break;
-#ifndef	_NOPRECEDE
-		if (haste && !havestat(&(filelist[filepos]))
-		&& (bindlist[i].d_func < 255
-		|| (funclist[bindlist[i].f_func].status & FN_NEEDSTATUS))
-		&& getstatus(&(filelist[filepos])) < 0)
-			no = WARNING_BELL;
+#if	FD >= 3
+		if (ch == K_TIMEOUT) return(REREAD_DIR);
 		else
 #endif
-		no = (bindlist[i].d_func < 255 && isdir(&(filelist[filepos])))
-			? (int)(bindlist[i].d_func)
-			: (int)(bindlist[i].f_func);
+		if (i >= maxbind) no = NO_OPERATION;
+#ifndef	_NOPRECEDE
+		else if (haste && !havestat(&(filelist[filepos]))
+		&& (hasdfunc(i) || (funclist[ffunc(i)].status & FN_NEEDSTATUS))
+		&& getstatus(&(filelist[filepos])) < 0)
+			no = WARNING_BELL;
+#endif
+		else no = (hasdfunc(i) && isdir(&(filelist[filepos])))
+			? dfunc(i) : ffunc(i);
 		if (no < FUNCLISTSIZ) funcstat = funclist[no].status;
 #ifndef	_NOARCHIVE
 		else if (archivefile) continue;
@@ -1769,22 +1749,22 @@ char *file, *def;
 			}
 		}
 		else {
-			no = ptyusercomm(macrolist[no - FUNCLISTSIZ],
+			no = ptyusercomm(getmacro(no),
 				filelist[filepos].name, 0);
 			no = evalstatus(no);
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 			if (ptymode && isearch) no = FNC_NONE;
 #endif
 		}
 
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 		while (ptylist[win].pid && ptylist[win].status < 0) {
 			VOID_C frontend();
 			no = FNC_UPDATE;
 			funcstat &= ~FN_REWRITEMODE;
 			funcstat |= FN_REWIN;
 		}
-#endif	/* !_NOPTY */
+#endif	/* DEP_PTY */
 
 #ifndef	_NOPRECEDE
 		if (sorton) haste = 0;
@@ -1815,26 +1795,24 @@ char *file, *def;
 				escapearch();
 			} while (browselist);
 # endif
-			strcpy(file, filelist[filepos].name);
+			strcpy2(file, filelist[filepos].name);
 		}
 		else if (no == FNC_CHDIR) {
-			char *tmp;
-
 			tmp = (filepos >= 0) ? filelist[filepos].name : NULL;
 			if (!(cp = archchdir(tmp))) {
 				if (!tmp) tmp = parentpath;
 				warning(-1, tmp);
-				strcpy(file, tmp);
+				strcpy2(file, tmp);
 			}
-			else if (cp != (char *)-1) strcpy(file, cp);
+			else if (cp != (char *)-1) strcpy2(file, cp);
 			else {
 				escapearch();
-				strcpy(file, filelist[filepos].name);
+				strcpy2(file, filelist[filepos].name);
 			}
 		}
 		else if (no == FNC_EFFECT) {
 			if (filepos < 0) *file = '\0';
-			else strcpy(file, filelist[filepos].name);
+			else strcpy2(file, filelist[filepos].name);
 		}
 		no = FNC_NONE;
 	}
@@ -1844,15 +1822,16 @@ char *file, *def;
 		no -= FNC_EFFECT;
 		if (!maxfile && !ischgdir(&(filelist[filepos]))) cp = curpath;
 		else cp = filelist[filepos].name;
-		strcpy(file, cp);
+		strcpy2(file, cp);
 	}
 
-	i = (maxfile || !ischgdir(&(filelist[0]))) ? maxfile : 1;
 #ifndef	_NOARCHIVE
-	if (!archivefile)
+	if (archivefile) i = 0;
+	else
 #endif
+	i = (maxfile || !ischgdir(&(filelist[0]))) ? maxfile : 1;
 	while (i-- > 0) {
-		free(filelist[i].name);
+		free2(filelist[i].name);
 		filelist[i].name = NULL;
 	}
 	maxfile = filepos = 0;
@@ -1864,7 +1843,8 @@ char *file, *def;
 }
 
 static char *NEAR initcwd(path, buf, evaled)
-char *path, *buf;
+CONST char *path;
+char *buf;
 int evaled;
 {
 	char *cp, *file;
@@ -1886,7 +1866,7 @@ int evaled;
 	if (chdir2(cp) >= 0) file = NULL;
 	else {
 		file = strrdelim(cp, 0);
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 		if (!file && dospath2(cp)) file = &(cp[2]);
 #endif
 
@@ -1901,31 +1881,31 @@ int evaled;
 				hideclock = 2;
 				warning(-1, cp);
 #if	MSDOS
-				strcpy(fullpath, origpath);
+				strcpy2(fullpath, origpath);
 #endif
-				free(cp);
+				free2(cp);
 				return(NULL);
 			}
 			if (i == _SC_) file++;
 			else *file = i;
 		}
-		strcpy(buf, file);
+		strcpy2(buf, file);
 		file = buf;
 	}
-	free(cp);
+	free2(cp);
 
 	return(file);
 }
 
 VOID main_fd(pathlist, evaled)
-char **pathlist;
+char *CONST *pathlist;
 int evaled;
 {
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	char buf[MAXPATHLEN];
 #endif
 	char *def, *cwd, file[MAXNAMLEN + 1], prev[MAXNAMLEN + 1];
-	int i, argc, ischgdir;
+	int n, argc, ischgdir;
 
 	if (!pathlist) argc = 0;
 	else for (argc = 0; argc < MAXINVOKEARGS; argc++)
@@ -1933,19 +1913,19 @@ int evaled;
 
 	cwd = getwd2();
 	def = NULL;
-	for (i = MAXWINDOWS - 1; i >= 0; i--) {
+	for (n = MAXWINDOWS - 1; n >= 0; n--) {
 #ifndef	_NOSPLITWIN
-		win = i;
-		winvar[i].v_fullpath = NULL;
+		win = n;
+		winvar[n].v_fullpath = NULL;
 # ifndef	_NOARCHIVE
-		winvar[i].v_archivedir = NULL;
+		winvar[n].v_archivedir = NULL;
 # endif
 #endif	/* !_NOSPLITWIN */
-#ifndef	_NOPTY
-		ptylist[i].pid = (p_id_t)0;
-		ptylist[i].path = NULL;
-		ptylist[i].fd = ptylist[i].pipe = -1;
-		ptylist[i].status = 0;
+#ifdef	DEP_PTY
+		ptylist[n].pid = (p_id_t)0;
+		ptylist[n].path = NULL;
+		ptylist[n].fd = ptylist[n].pipe = -1;
+		ptylist[n].status = 0;
 #endif
 
 #ifndef	_NOARCHIVE
@@ -1955,7 +1935,7 @@ int evaled;
 		launchp = NULL;
 		arcflist = NULL;
 		maxarcf = 0;
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 		archdrive = 0;
 # endif
 # ifndef	_NOBROWSE
@@ -1975,14 +1955,14 @@ int evaled;
 		curcolumns = defcolumns;
 		FILEPERROW = 0;
 
-		if (i >= argc) continue;
+		if (n >= argc) continue;
 
 		chdir2(cwd);
-		def = initcwd(pathlist[i], prev, evaled);
+		def = initcwd(pathlist[n], prev, evaled);
 
 #ifndef	_NOSPLITWIN
-		winvar[i].v_fullpath = strdup2(fullpath);
-		if (i) {
+		winvar[n].v_fullpath = strdup2(fullpath);
+		if (n) {
 			getfilelist();
 			sorton = sorttype % 100;
 			filepos = calcfilepos(filelist, maxfile, def);
@@ -1997,7 +1977,7 @@ int evaled;
 		if (fileperrow(windows + 1) < WFILEMIN) {
 			hideclock = 2;
 			warning(0, NOROW_K);
-			for (i = windows; i < argc; i++) shutwin(i);
+			for (n = windows; n < argc; n++) shutwin(n);
 			break;
 		}
 		windows++;
@@ -2010,7 +1990,7 @@ int evaled;
 
 	for (;;) {
 		if (!def && isdotdir(file) == 1) {
-			strcpy(prev, getbasename(fullpath));
+			strcpy2(prev, getbasename(fullpath));
 			if (*prev) def = prev;
 			else copycurpath(file);
 		}
@@ -2019,7 +1999,7 @@ int evaled;
 		&& chdir3(nodospath(buf, file), 1) < 0) {
 			hideclock = 2;
 			warning(-1, file);
-			strcpy(prev, file);
+			strcpy2(prev, file);
 			def = prev;
 		}
 
@@ -2030,29 +2010,30 @@ int evaled;
 			ischgdir = browsedir(file, def);
 		} while (archivefile);
 #endif
+
 		if (ischgdir < 0) {
 			if (ischgdir > -2) chdir2(cwd);
 			break;
 		}
 		if (ischgdir) def = NULL;
 		else {
-			strcpy(prev, file);
+			strcpy2(prev, file);
 			copycurpath(file);
 			def = prev;
 		}
 	}
 
-	free(cwd);
+	free2(cwd);
 #ifdef	_NOSPLITWIN
-	if (filelist) free(filelist);
-	if (findpattern) free(findpattern);
+	free2(filelist);
+	free2(findpattern);
 #else
-	for (i = 0; i < MAXWINDOWS; i++) shutwin(i);
+	for (n = 0; n < MAXWINDOWS; n++) shutwin(n);
 	windows = 1;
 	win = 0;
 	calcwin();
 #endif
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	killallpty();
 #endif
 	Xlocate(0, n_line - 1);
