@@ -7,43 +7,43 @@
 #ifndef	__KCTYPE_H_
 #define	__KCTYPE_H_
 
+#include "depend.h"
 #include <ctype.h>
 
-#ifndef	__SYS_TYPES_STAT_H_
-#define	__SYS_TYPES_STAT_H_
-#include <sys/types.h>
-#include <sys/stat.h>
-#endif
-
 #ifdef	K_EXTERN
 #define	K_INTERN
-#define	K_INIT(x)	=x
+#define	K_INIT(x)		=x
 #else
-#define	K_EXTERN	extern
+#define	K_EXTERN		extern
 #define	K_INIT(x)
 #endif
 
-#define	C_BS		'\010'
-#define	C_DEL		'\177'
-#define	C_EKANA		0x8e
+#define	C_BS			'\010'
+#define	C_DEL			'\177'
+#define	C_EKANA			0x8e
 
 #ifdef	LSI_C
 #include <jctype.h>
-#define	toupper2(c)	toupper((u_char)(c))
-#define	tolower2(c)	tolower((u_char)(c))
-#define	isalnum2(c)	isalnum((u_char)(c))
-#define	isalpha2(c)	isalpha((u_char)(c))
-#define	iscntrl2(c)	iscntrl((u_char)(c))
-#define	isdigit2(c)	isdigit((u_char)(c))
-#define	isupper2(c)	isupper((u_char)(c))
-#define	islower2(c)	islower((u_char)(c))
-#define	isxdigit2(c)	isxdigit((u_char)(c))
-#define	issjis1(c)	iskanji((u_char)(c))
-#define	issjis2(c)	iskanji2((u_char)(c))
-#define	iskana2(c)	iskana((u_char)(c))
-#define	isspace2(c)	isspace((u_char)(c))
-#define	isprint2(c)	isprint((u_char)(c))
-#define	isblank2(c)	((c) == ' ' || (c) == '\t')
+#define	toupper2(c)		toupper((u_char)(c))
+#define	tolower2(c)		tolower((u_char)(c))
+#define	isalnum2(c)		isalnum((u_char)(c))
+#define	isalpha2(c)		isalpha((u_char)(c))
+#define	iscntrl2(c)		iscntrl((u_char)(c))
+#define	isdigit2(c)		isdigit((u_char)(c))
+#define	isupper2(c)		isupper((u_char)(c))
+#define	islower2(c)		islower((u_char)(c))
+#define	isxdigit2(c)		isxdigit((u_char)(c))
+#define	issjis1(c)		iskanji((u_char)(c))
+#define	issjis2(c)		iskanji2((u_char)(c))
+#define	iskana2(c)		iskana((u_char)(c))
+#define	isspace2(c)		isspace((u_char)(c))
+#define	isprint2(c)		isprint((u_char)(c))
+#define	isblank2(c)		((c) == ' ' || (c) == '\t')
+#define	isutf2(c1, c2)		((((u_char)(c1) & 0xe0) == 0xc0) \
+				&& ((u_char)(c2) & 0xc0) == 0x80)
+#define	isutf3(c1, c2, c3)	(((u_char)(c1) & 0xf0) == 0xe0 \
+				&& ((u_char)(c2) & 0xc0) == 0x80 \
+				&& ((u_char)(c3) & 0xc0) == 0x80)
 #else	/* !LSI_C */
 K_EXTERN CONST u_char uppercase[256]
 #ifdef	K_INTERN
@@ -156,130 +156,136 @@ K_EXTERN CONST u_char kctypetable[256]
 = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 0x00 */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 0x10 */
-	   0, 0060, 0060, 0060, 0060, 0060, 0060, 0060,	/* 0x20 */
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,	/* 0x30 */
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,	/* 0x40 */
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,	/* 0x50 */
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,
+	   0, 0020, 0020, 0020, 0020, 0020, 0020, 0020,	/* 0x20 */
+	0020, 0020, 0020, 0020, 0020, 0020, 0020, 0020,
+	0020, 0020, 0020, 0020, 0020, 0020, 0020, 0020,	/* 0x30 */
+	0020, 0020, 0020, 0020, 0020, 0020, 0020, 0020,
+	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x40 */
+	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,
+	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x50 */
+	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,
 	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x60 */
 	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,
 	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x70 */
 	0022, 0022, 0022, 0022, 0022, 0022, 0022,    0,
-	0002, 0003, 0003, 0003, 0003, 0003, 0003, 0003,	/* 0x80 */
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,	/* 0x90 */
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,
-	0002, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xa0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xb0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xc0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xd0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,	/* 0xe0 */
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,
+	0042, 0043, 0043, 0043, 0043, 0043, 0043, 0043,	/* 0x80 */
+	0043, 0043, 0043, 0043, 0043, 0043, 0043, 0043,
+	0043, 0043, 0043, 0043, 0043, 0043, 0043, 0043,	/* 0x90 */
+	0043, 0043, 0043, 0043, 0043, 0043, 0043, 0043,
+	0042, 0056, 0056, 0056, 0056, 0056, 0056, 0056,	/* 0xa0 */
+	0056, 0056, 0056, 0056, 0056, 0056, 0056, 0056,
+	0056, 0056, 0056, 0056, 0056, 0056, 0056, 0056,	/* 0xb0 */
+	0056, 0056, 0056, 0056, 0056, 0056, 0056, 0056,
+	0116, 0116, 0116, 0116, 0116, 0116, 0116, 0116,	/* 0xc0 */
+	0116, 0116, 0116, 0116, 0116, 0116, 0116, 0116,
+	0116, 0116, 0116, 0116, 0116, 0116, 0116, 0116,	/* 0xd0 */
+	0116, 0116, 0116, 0116, 0116, 0116, 0116, 0116,
+	0213, 0213, 0213, 0213, 0213, 0213, 0213, 0213,	/* 0xe0 */
+	0213, 0213, 0213, 0213, 0213, 0213, 0213, 0213,
 	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,	/* 0xf0 */
 	0013, 0013, 0013, 0013, 0013, 0010, 0010,    0
 }
 #endif	/* K_INTERN */
 ;
-#define	toupper2(c)	(uppercase[(u_char)(c)])
-#define	tolower2(c)	(lowercase[(u_char)(c)])
-#define	C_DIGIT		0001
-#define	C_UPPER		0002
-#define	C_LOWER		0004
-#define	C_HEX		0010
-#define	C_CNTRL		0020
-#define	C_SPACE		0040
-#define	C_PRINT		0100
-#define	C_BLANK		0200
-#define	isalnum2(c)	(ctypetable[(u_char)(c)] \
-			& (C_DIGIT | C_UPPER | C_LOWER))
-#define	isalpha2(c)	(ctypetable[(u_char)(c)] & (C_UPPER | C_LOWER))
-#define	iscntrl2(c)	(ctypetable[(u_char)(c)] & C_CNTRL)
-#define	isdigit2(c)	(ctypetable[(u_char)(c)] & C_DIGIT)
-#define	isupper2(c)	(ctypetable[(u_char)(c)] & C_UPPER)
-#define	islower2(c)	(ctypetable[(u_char)(c)] & C_LOWER)
-#define	isxdigit2(c)	(ctypetable[(u_char)(c)] & (C_DIGIT | C_HEX))
-#define	isspace2(c)	(ctypetable[(u_char)(c)] & C_SPACE)
-#define	isprint2(c)	(ctypetable[(u_char)(c)] & C_PRINT)
-#define	isblank2(c)	(ctypetable[(u_char)(c)] & C_BLANK)
-#define	KC_SJIS1	0001
-#define	KC_SJIS2	0002
-#define	KC_KANA		0004
-#define	KC_EUCJP	0010
-#define	KC_JIS		0020
-#define	KC_JKANA	0040
-#define	iskana2(c)	(kctypetable[(u_char)(c)] & KC_KANA)
-
-# ifndef	issjis1
-# define	issjis1(c)	(kctypetable[(u_char)(c)] & KC_SJIS1)
-# endif
-# ifndef	issjis2
-# define	issjis2(c)	(kctypetable[(u_char)(c)] & KC_SJIS2)
-# endif
-
-# ifndef	iseuc
-# define	iseuc(c)	(kctypetable[(u_char)(c)] & KC_EUCJP)
-# endif
-
-# ifndef	isjis
-# define	isjis(c)	(kctypetable[(u_char)(c)] & KC_JIS)
-# endif
+#define	toupper2(c)		(uppercase[(u_char)(c)])
+#define	tolower2(c)		(lowercase[(u_char)(c)])
+#define	C_DIGIT			0001
+#define	C_UPPER			0002
+#define	C_LOWER			0004
+#define	C_HEX			0010
+#define	C_CNTRL			0020
+#define	C_SPACE			0040
+#define	C_PRINT			0100
+#define	C_BLANK			0200
+#define	isalnum2(c)		(ctypetable[(u_char)(c)] \
+				& (C_DIGIT | C_UPPER | C_LOWER))
+#define	isalpha2(c)		(ctypetable[(u_char)(c)] & (C_UPPER | C_LOWER))
+#define	iscntrl2(c)		(ctypetable[(u_char)(c)] & C_CNTRL)
+#define	isdigit2(c)		(ctypetable[(u_char)(c)] & C_DIGIT)
+#define	isupper2(c)		(ctypetable[(u_char)(c)] & C_UPPER)
+#define	islower2(c)		(ctypetable[(u_char)(c)] & C_LOWER)
+#define	isxdigit2(c)		(ctypetable[(u_char)(c)] & (C_DIGIT | C_HEX))
+#define	isspace2(c)		(ctypetable[(u_char)(c)] & C_SPACE)
+#define	isprint2(c)		(ctypetable[(u_char)(c)] & C_PRINT)
+#define	isblank2(c)		(ctypetable[(u_char)(c)] & C_BLANK)
+#define	KC_SJIS1		0001
+#define	KC_SJIS2		0002
+#define	KC_KANA			0004
+#define	KC_EUCJP		0010
+#define	KC_JIS			0020
+#define	KC_UTFN			0040
+#define	KC_UTF2			0100
+#define	KC_UTF3			0200
+#define	iskana2(c)		(kctypetable[(u_char)(c)] & KC_KANA)
+#ifndef	issjis1
+#define	issjis1(c)		(kctypetable[(u_char)(c)] & KC_SJIS1)
+#endif
+#ifndef	issjis2
+#define	issjis2(c)		(kctypetable[(u_char)(c)] & KC_SJIS2)
+#endif
+#ifndef	iseuc
+#define	iseuc(c)		(kctypetable[(u_char)(c)] & KC_EUCJP)
+#endif
+#ifndef	isjis
+#define	isjis(c)		(kctypetable[(u_char)(c)] & KC_JIS)
+#endif
+#define	isutf2(c1, c2)		((kctypetable[(u_char)(c1)] & KC_UTF2) \
+				&& (kctypetable[(u_char)(c2)] & KC_UTFN))
+#define	isutf3(c1, c2, c3)	((kctypetable[(u_char)(c1)] & KC_UTF3) \
+				&& (kctypetable[(u_char)(c2)] & KC_UTFN) \
+				&& (kctypetable[(u_char)(c3)] & KC_UTFN))
 #endif	/* !LSI_C */
-#define	ismsb(c)	(((u_char)(c)) & 0x80)
 
-#define	isekana(s, i)	((u_char)((s)[i]) == C_EKANA && iskana2((s)[(i) + 1]))
-#define	isskana(s, i)	iskana2((s)[i])
-#define	isjkana(s, i)	(kctypetable[(u_char)((s)[i])] & KC_JKANA)
+#define	ismsb(c)		(((u_char)(c)) & 0x80)
+#define	isekana(s, i)		((u_char)((s)[i]) == C_EKANA \
+				&& iskana2((s)[(i) + 1]))
+#define	isskana(s, i)		iskana2((s)[i])
+#define	isjkana(s, i)		iskana2(((s)[i]) ^ 0x80)
+#define	iswucs2(u)		(((u) & 0xff00) \
+				&& ((u) < 0xff61 || (u) > 0xff9f))
 
-#define	isutf2(c1, c2)	((((u_char)(c1) & 0xe0) == 0xc0) \
-			&& ((u_char)(c2) & 0xc0) == 0x80)
-#define	isutf3(c1, c2, c3) \
-			(((u_char)(c1) & 0xf0) == 0xe0 \
-			&& ((u_char)(c2) & 0xc0) == 0x80 \
-			&& ((u_char)(c3) & 0xc0) == 0x80)
+#ifndef	MINIMUMSHELL
+K_EXTERN CONST u_char b2htable[16]
+#ifdef	K_INTERN
+= {
+	'0', '1', '2', '3', '4', '5', '6', '7',
+	'8', '9', 'a', 'b', 'c', 'd', 'e', 'f',
+}
+#endif
+;
+#define	tohexa(c)		b2htable[(u_char)(c)]
+#endif	/* !MINIMUMSHELL */
 
-#define	NOCNV	0
-#define	ENG	1
-#define	SJIS	2
-#define	EUC	3
-#define	JIS7	4
-#define	O_JIS7	5
-#define	JIS8	6
-#define	O_JIS8	7
-#define	JUNET	8
-#define	O_JUNET	9
-#define	HEX	10
-#define	CAP	11
-#define	UTF8	12
-#define	M_UTF8	13
-#define	I_UTF8	14
+#define	NOCNV			0
+#define	ENG			1
+#define	SJIS			2
+#define	EUC			3
+#define	JIS7			4
+#define	O_JIS7			5
+#define	JIS8			6
+#define	O_JIS8			7
+#define	JUNET			8
+#define	O_JUNET			9
+#define	HEX			10
+#define	CAP			11
+#define	UTF8			12
+#define	M_UTF8			13
+#define	I_UTF8			14
 
 #ifdef	CODEEUC
-#define	DEFCODE	EUC
-#define	SECCODE	SJIS
-#define	KANAWID	2
+#define	DEFCODE			EUC
+#define	SECCODE			SJIS
+#define	KANAWID			2
 #else
-#define	DEFCODE	SJIS
-#define	SECCODE	EUC
-#define	KANAWID	1
+#define	DEFCODE			SJIS
+#define	SECCODE			EUC
+#define	KANAWID			1
 #endif
 
-#define	MAXKANJIBUF	(3 + 2 + 3)
-
-#ifdef	NOMULTIKANJI
-#define	_NOKANJICONV
-#define	_NOKANJIFCONV
-#endif
-
-#if	defined (_NOKANJICONV) && !defined (_NOKANJIFCONV)
-#define	_NOKANJIFCONV
-#endif
+#define	MAXKANJIBUF		(3 + 2 + 3)
+#define	MAXKLEN			2
+#define	MAXUTF8LEN		3
+#define	MAXNFLEN		4
 
 #if	!defined (_NOKANJICONV) || (defined (FD) && !defined (_NODOSDRIVE))
 K_EXTERN int kanjierrno K_INIT(0);
@@ -287,14 +293,16 @@ K_EXTERN int kanjierrno K_INIT(0);
 #ifndef	_NOKANJIFCONV
 K_EXTERN int defaultkcode K_INIT(NOCNV);
 #endif
-#ifndef	_NOKANJICONV
+#ifdef	_NOKANJICONV
+#define	inputkcode		NOCNV
+#else
 K_EXTERN int inputkcode K_INIT(NOCNV);
 #endif
 #if	!defined (_NOKANJICONV) \
 || (!defined (_NOENGMES) && !defined (_NOJPNMES))
 K_EXTERN int outputkcode K_INIT(NOCNV);
 #endif
-#if	!defined (_NOKANJICONV) && !defined (_NOPTY)
+#if	!defined (_NOKANJICONV) && defined (DEP_PTY)
 K_EXTERN int ptyinkcode K_INIT(NOCNV);
 K_EXTERN int ptyoutkcode K_INIT(NOCNV);
 #endif
@@ -305,11 +313,11 @@ K_EXTERN int fnamekcode K_INIT(NOCNV);
 K_EXTERN int messagelang K_INIT(NOCNV);
 #endif
 
-#define	L_INPUT		0001
-#define	L_OUTPUT	0002
-#define	L_FNAME		0004
-#define	L_TERMINAL	0010
-#define	L_MESSAGE	0020
+#define	L_INPUT			0001
+#define	L_OUTPUT		0002
+#define	L_FNAME			0004
+#define	L_TERMINAL		0010
+#define	L_MESSAGE		0020
 
 K_EXTERN CONST char kanjiiomode[]
 # ifdef	K_INTERN
@@ -333,10 +341,10 @@ K_EXTERN CONST char kanjiiomode[]
 # endif	/* K_INTERN */
 ;
 
-K_EXTERN int iskanji1 __P_((char *, int));
+K_EXTERN int iskanji1 __P_((CONST char *, int));
 #ifdef	K_INTERN
 int iskanji1(s, i)
-char *s;
+CONST char *s;
 int i;
 {
 # ifdef	CODEEUC
@@ -347,10 +355,10 @@ int i;
 }
 #endif	/* K_INTERN */
 
-K_EXTERN int iskana1 __P_((char *, int *));
+K_EXTERN int iskana1 __P_((CONST char *, int *));
 #ifdef	K_INTERN
 int iskana1(s, ip)
-char *s;
+CONST char *s;
 int *ip;
 {
 # ifdef	CODEEUC
@@ -373,10 +381,10 @@ int *ip;
 # define	isinkanji2(c,k)	issjis2(c)
 # endif
 #else	/* !_NOKANJICONV */
-#define	isinkanji1(c,k)	(((k) == EUC) ? iseuc(c) : \
-			(((k) == SJIS) ? issjis1(c) : 0))
-#define	isinkanji2(c,k)	(((k) == EUC) ? iseuc(c) : \
-			(((k) == SJIS) ? issjis2(c) : 0))
+#define	isinkanji1(c,k)		(((k) == EUC) ? iseuc(c) : \
+				(((k) == SJIS) ? issjis1(c) : 0))
+#define	isinkanji2(c,k)		(((k) == EUC) ? iseuc(c) : \
+				(((k) == SJIS) ? issjis2(c) : 0))
 #endif	/* !_NOKANJICONV */
 
 #endif	/* !__KCTYPE_H_ */
