@@ -5,39 +5,58 @@
  */
 
 #include "fd.h"
+#include "time.h"
+#include "log.h"
+#include "termio.h"
+#include "realpath.h"
+#include "parse.h"
+#include "kconv.h"
 #include "func.h"
 #include "kanji.h"
 #include "version.h"
 
+#ifdef	DEP_ORIGSHELL
+#include "system.h"
+#endif
+#ifdef	DEP_PTY
+#include "termemu.h"
+#endif
+#ifdef	DEP_IME
+#include "roman.h"
+#endif
+#ifdef	DEP_URLPATH
+#include "urldisk.h"
+#endif
+
 #if	MSDOS
 #include <process.h>
 # ifndef	DJGPP
 # include <dos.h>
-#  ifdef	__TURBOC__
-extern u_int _stklen = 0x5800;
-#  define	harderr_t	void
-#  else
-#  define	harderr_t	int
-#  endif
 # endif
 #endif	/* !MSDOS */
 
-#ifdef	_NOORIGSHELL
-#include "termio.h"
-#include "wait.h"
-#define	Xexit2		exit2
-#define	isorgpid()	(1)
-#else
-#include "system.h"
-#define	isorgpid()	(mypid == orgpid)
+#define	CLOCKUPDATE		10	/* sec */
+#ifndef	BINDIR
+#define	BINDIR			"/usr/local/bin"
 #endif
-
-#if	!defined (_NOIME) && defined (DEBUG)
-#include "roman.h"
-#endif
-
 #ifndef	DATADIR
-#define	DATADIR		progpath
+#define	DATADIR			progpath
+#endif
+#ifndef	DEFPATH
+#define	DEFPATH			":/bin:/usr/bin"
+#endif
+
+#ifdef	DEP_ORIGSHELL
+#define	isorgpid()		(mypid == orgpid)
+#else
+#define	isorgpid()		(1)
+#endif
+
+#ifdef	__TURBOC__
+extern u_int _stklen = 0x5800;
+#define	harderr_t		void
+#else
+#define	harderr_t		int
 #endif
 
 #if	MSDOS
@@ -49,25 +68,39 @@ extern char *adjustfname __P_((char *));
 # endif
 #endif	/* MSDOS */
 
-#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 extern VOID killjob __P_((VOID_A));
 #endif
 
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 extern char **environ;
 extern char **environ2;
 #endif
-extern bindtable bindlist[];
+extern bindlist_t bindlist;
+extern int maxbind;
+#if	defined (DEP_DYNAMICLIST) || !defined (_NOCUSTOMIZE)
+extern origbindlist_t origbindlist;
+extern int origmaxbind;
+#endif
 #ifndef	_NOARCHIVE
-extern launchtable launchlist[];
+extern launchlist_t launchlist;
 extern int maxlaunch;
-extern archivetable archivelist[];
+extern archivelist_t archivelist;
 extern int maxarchive;
+# if	defined (DEP_DYNAMICLIST) || !defined (_NOCUSTOMIZE)
+extern origlaunchlist_t origlaunchlist;
+extern int origmaxlaunch;
+extern origarchivelist_t origarchivelist;
+extern int origmaxarchive;
+# endif
+#endif	/* !_NOARCHIVE */
+extern helpindex_t helpindex;
+#if	defined (DEP_DYNAMICLIST) || !defined (_NOCUSTOMIZE)
+extern orighelpindex_t orighelpindex;
 #endif
 extern int wheader;
 extern char fullpath[];
 extern short savehist[];
-extern char *helpindex[];
 extern int subwindow;
 extern int win_x;
 extern int win_y;
@@ -75,23 +108,16 @@ extern int win_y;
 extern int custno;
 #endif
 extern char *deftmpdir;
-#ifdef	_USEUNICODE
-extern char *unitblpath;
-#endif
-#ifndef	_NOIME
+#ifdef	DEP_IME
 extern char *dicttblpath;
 #endif
-#ifndef	_NOPTY
-extern int ptymode;
-extern int ptyinternal;
-extern int parentfd;
-#endif
-
-#define	CLOCKUPDATE	10	/* sec */
 
 #if	MSDOS && !defined (PROTECTED_MODE)
 static harderr_t far criticalerror __P_((u_int, u_int, u_short far *));
 #endif
+#ifndef	DEP_ORIGSHELL
+static VOID NEAR Xexit2 __P_((int));
+#endif
 static VOID NEAR signalexit __P_((int));
 #ifdef	SIGALRM
 static int ignore_alrm __P_((VOID_A));
@@ -158,20 +184,26 @@ static VOID NEAR printtime __P_((int));
 #ifdef	SIGALRM
 static int trapalrm __P_((VOID_A));
 #endif
-static char *NEAR getversion __P_((int *));
-#ifndef	_NOLOGGING
-static VOID NEAR startlog __P_((char **));
+#ifdef	DEP_DOSDRIVE
+static int dos_intrkey __P_((VOID_A));
+#endif
+static int read_intrkey __P_((VOID_A));
+#ifdef	DEP_LOGGING
+static VOID NEAR startlog __P_((char *CONST *));
 static VOID NEAR endlog __P_((int));
 #endif
-#ifdef	_NOORIGSHELL
-static int NEAR execruncomline __P_((char *, char *, int, char *));
+#ifndef	DEP_ORIGSHELL
+static int NEAR execruncomline __P_((char *, CONST char *, int, CONST char *));
 #endif
-static int NEAR initoption __P_((int, char *[], char *[]));
-static int NEAR evaloption __P_((char *[]));
-static char *NEAR searchenv __P_((char *, char *[]));
-static VOID NEAR setexecname __P_((char *));
-static VOID NEAR setexecpath __P_((char *, char *[]));
-int main __P_((int, char *[], char *[]));
+static int NEAR initoption __P_((int, char *CONST []));
+static int NEAR evaloption __P_((char *CONST []));
+static char *NEAR searchenv __P_((CONST char *, char *CONST []));
+#if	!MSDOS
+static char *NEAR searchexecname __P_((CONST char *, char *CONST []));
+#endif
+static VOID NEAR setexecname __P_((CONST char *));
+static VOID NEAR setexecpath __P_((CONST char *, char *CONST []));
+int main __P_((int, char *CONST [], char *CONST []));
 
 char *origpath = NULL;
 char *progpath = NULL;
@@ -189,24 +221,11 @@ int hideclock = 0;
 int noalrm = 1;
 #endif
 int fd_restricted = 0;
-#ifndef	_NOCUSTOMIZE
-char **orighelpindex = NULL;
-bindtable *origbindlist = NULL;
-# ifndef	_NOKEYMAP
+#if	!defined (_NOCUSTOMIZE) && !defined (_NOKEYMAP)
 keyseq_t *origkeymaplist = NULL;
-# endif
-# ifndef	_NOARCHIVE
-launchtable *origlaunchlist = NULL;
-int origmaxlaunch = 0;
-archivetable *origarchivelist = NULL;
-int origmaxarchive = 0;
-# endif
-# ifdef	_USEDOSEMU
-devinfo *origfdtype = NULL;
-# endif
-#endif	/* !_NOCUSTOMIZE */
+#endif
 int inruncom = 0;
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 int fdmode = 0;
 #endif
 
@@ -228,41 +247,51 @@ u_short far *devhdr;
 #endif
 
 VOID error(s)
-char *s;
+CONST char *s;
 {
+	static int doing = 0;
 	int duperrno;
 
+	if (doing == 1) exit2(2);
+	if (doing == 2) exit(2);
 	duperrno = errno;
+	doing = 1;
+
 	sigvecset(0);
 	if (isorgpid()) {
 		forcecleandir(deftmpdir, tmpfilename);
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		dosallclose();
+#endif
+#ifdef	DEP_URLPATH
+		urlallclose();
 #endif
 		Xstdiomode();
 		endterm();
 	}
 
 	if (!s) s = progname;
-	if (dumbterm <= 2) fputc('\007', stderr);
+	if (dumbterm <= 2) Xfputc('\007', Xstderr);
 	errno = duperrno;
 	if (errno) perror2(s);
 	else {
-		fputs(s, stderr);
-		fputnl(stderr);
+		Xfputs(s, Xstderr);
+		fputnl(Xstderr);
 	}
+	Xfclose(Xstderr);
+	doing = 2;
 
 	if (isorgpid()) {
 		inittty(1);
 		keyflush();
 		prepareexitfd(2);
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 # ifndef	NOJOB
 		killjob();
 # endif
 		prepareexit(0);
-#endif	/* !_NOORIGSHELL */
-#ifndef	_NOPTY
+#endif	/* DEP_ORIGSHELL */
+#ifdef	DEP_PTY
 		killallpty();
 #endif
 #ifdef	DEBUG
@@ -277,6 +306,15 @@ char *s;
 	exit(2);
 }
 
+#ifndef	DEP_ORIGSHELL
+static VOID NEAR Xexit2(n)
+int n;
+{
+	prepareexitfd(n);
+	exit2(n);
+}
+#endif	/* !DEP_ORIGSHELL */
+
 static VOID NEAR signalexit(sig)
 int sig;
 {
@@ -284,22 +322,25 @@ int sig;
 
 	if (isorgpid()) {
 		forcecleandir(deftmpdir, tmpfilename);
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		dosallclose();
 #endif
-#ifndef	_NOLOGGING
+#ifdef	DEP_URLPATH
+		urlallclose();
+#endif
+#ifdef	DEP_LOGGING
 		endlog(sig + 128);
 #endif
 		endterm();
 		inittty(1);
 		keyflush();
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 # if	defined (SIGHUP) && !defined (NOJOB)
 		if (sig == SIGHUP) killjob();
 # endif
 		prepareexit(0);
-#endif	/* !_NOORIGSHELL */
-#ifndef	_NOPTY
+#endif	/* DEP_ORIGSHELL */
+#ifdef	DEP_PTY
 		killallpty();
 #endif
 #ifdef	DEBUG
@@ -376,7 +417,7 @@ static int ignore_cont(VOID_A)
 	signal2(SIGCONT, (sigcst_t)ignore_cont);
 # if	!MSDOS
 	suspended = 1;
-#  ifndef	_NOORIGSHELL
+#  ifdef	DEP_ORIGSHELL
 	sigconted = 1;
 #  endif
 # endif	/* !MSDOS */
@@ -507,13 +548,13 @@ VOID setlinecol(VOID_A)
 {
 	char buf[MAXLONGWIDTH + 1];
 
-	if (getconstvar("LINES")) {
+	if (getconstvar(ENVLINES)) {
 		snprintf2(buf, sizeof(buf), "%d", n_line);
-		setenv2("LINES", buf, 1);
+		setenv2(ENVLINES, buf, 1);
 	}
-	if (getconstvar("COLUMNS")) {
+	if (getconstvar(ENVCOLUMNS)) {
 		snprintf2(buf, sizeof(buf), "%d", n_column);
-		setenv2("COLUMNS", buf, 1);
+		setenv2(ENVCOLUMNS, buf, 1);
 	}
 }
 
@@ -553,14 +594,16 @@ int n, max;
 VOID checkscreen(xmax, ymax)
 int xmax, ymax;
 {
-	char *cp, *tty;
+	CONST char *cp;
+	char *tty;
 	int i, row, wastty, dupn_line, dupdumbterm;
 
+	if (ttyio < 0) return;
 #ifdef	SIGWINCH
 	nowinch++;
 #endif
 	dupn_line = -1;
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	if (parentfd >= 0) row = WFILEMIN;
 	else
 #endif
@@ -604,7 +647,7 @@ int xmax, ymax;
 		Xputterm(T_BELL);
 		Xcputnl();
 		Xtflush();
-		if (kbhit2(1000000L) && getkey2(0) == K_ESC) {
+		if (kbhit2(1000000L) && getkey3(0, inputkcode, 0) == K_ESC) {
 			errno = 0;
 			error(INTR_K);
 		}
@@ -612,7 +655,7 @@ int xmax, ymax;
 	dumbterm = dupdumbterm;
 	loadtermio(ttyio, tty, NULL);
 	isttyiomode = wastty;
-	if (tty) free(tty);
+	free2(tty);
 
 	setlinecol();
 	if (n_line != dupn_line) {
@@ -649,7 +692,7 @@ int forced;
 		checkscreen(-1, -1);
 		if (isorgpid()) {
 			if (x != n_column || y != n_line) rewritefile(1);
-			if (subwindow) ungetch2(K_CTRL('L'));
+			if (subwindow) ungetkey2(K_CTRL('L'));
 		}
 	}
 }
@@ -681,7 +724,7 @@ int hide;
 		timersec = CLOCKUPDATE;
 	}
 	if (timersec-- < CLOCKUPDATE && !showsecond) return;
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	if (checkallpty() < 0) rewritefile(0);
 	if (isptymode() && !ptyinternal && hide <= 1) hide = 0;
 #endif
@@ -719,7 +762,7 @@ static int trapalrm(VOID_A)
 
 	duperrno = errno;
 	signal2(SIGALRM, SIG_IGN);
-# if	!defined (_NOORIGSHELL) && !defined (_NOPTY)
+# if	defined (DEP_ORIGSHELL) && defined (DEP_PTY)
 	if (isshptymode()) {
 		if (mypid == shellpid) printtime(hideclock);
 	}
@@ -736,6 +779,20 @@ static int trapalrm(VOID_A)
 }
 #endif	/* SIGALRM */
 
+#ifdef	DEP_DOSDRIVE
+static int dos_intrkey(VOID_A)
+{
+	return(intrkey(-1));
+}
+#endif
+
+static int read_intrkey(VOID_A)
+{
+	if (isttyiomode) return(intrkey(K_ESC));
+
+	return(0);
+}
+
 int sigvecset(set)
 int set;
 {
@@ -755,9 +812,15 @@ int set;
 #ifdef	SIGWINCH
 		signal2(SIGWINCH, (sigcst_t)wintr);
 #endif
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		doswaitfunc = waitmes;
-		dosintrfunc = intrkey;
+		dosintrfunc = dos_intrkey;
+#endif
+#ifdef	DEP_LSPARSE
+		lsintrfunc = read_intrkey;
+#endif
+#ifndef	NOSELECT
+		readintrfunc = read_intrkey;
 #endif
 		status = 1;
 	}
@@ -772,7 +835,7 @@ int set;
 #ifdef	SIGWINCH
 		signal2(SIGWINCH, (sigcst_t)ignore_winch);
 #endif
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		doswaitfunc = NULL;
 		dosintrfunc = NULL;
 #endif
@@ -782,15 +845,15 @@ int set;
 	return(old);
 }
 
-static char *NEAR getversion(lenp)
+char *getversion(lenp)
 int *lenp;
 {
 	char *cp, *eol;
 
-	cp = strchr(version, ' ');
+	cp = strchr2(version, ' ');
 	while (*(++cp) == ' ');
 	if (lenp) {
-		if (!(eol = strchr(cp, ' '))) eol = cp + strlen(cp);
+		if (!(eol = strchr2(cp, ' '))) eol = cp + strlen(cp);
 		*lenp = eol - cp;
 	}
 
@@ -799,8 +862,8 @@ int *lenp;
 
 VOID title(VOID_A)
 {
-	char *cp;
-	int i, len;
+	CONST char *cp;
+	int n, len;
 
 	Xlocate(0, L_TITLE);
 	Xputterm(T_STANDOUT);
@@ -817,16 +880,16 @@ VOID title(VOID_A)
 	}
 	Xcputs2(" Ver.");
 	len += 5;
-	cp = getversion(&i);
-	cputstr(i, cp);
+	cp = getversion(&n);
+	cputstr(n, cp);
 	if (distributor) {
 		Xputch2('#');
-		i++;
+		n++;
 	}
-	cp = (iswellomit()) ? nullstr : " (c)1995-2006 T.Shirai  ";
+	cp = (iswellomit()) ? nullstr : " (c)1995-2008 T.Shirai  ";
 	Xcputs2(cp);
-	i = n_column - len - strlen2(cp) - i;
-	while (i-- > 0) Xputch2(' ');
+	n = n_column - len - strlen2(cp) - n;
+	while (n-- > 0) Xputch2(' ');
 	Xputterm(END_STANDOUT);
 	timersec = 0;
 	printtime(0);
@@ -835,26 +898,40 @@ VOID title(VOID_A)
 #ifndef	_NOCUSTOMIZE
 VOID saveorigenviron(VOID_A)
 {
-	orighelpindex = copystrarray(NULL, helpindex, NULL, MAXHELPINDEX);
-	origbindlist = copybind(NULL, bindlist);
+# if	FD >= 3
+	char *cp;
+	int n;
+# endif
+
 # ifndef	_NOKEYMAP
 	origkeymaplist = copykeyseq(NULL);
 # endif
-# ifndef	_NOARCHIVE
+# ifndef	DEP_DYNAMICLIST
+	orighelpindex = copystrarray(NULL, helpindex, NULL, MAXHELPINDEX);
+	origbindlist = copybind(NULL, bindlist, &origmaxbind, maxbind);
+#  ifndef	_NOARCHIVE
 	origlaunchlist = copylaunch(NULL, launchlist,
 		&origmaxlaunch, maxlaunch);
 	origarchivelist = copyarch(NULL, archivelist,
 		&origmaxarchive, maxarchive);
-# endif
-# ifdef	_USEDOSEMU
-	origfdtype = copydosdrive(NULL, fdtype);
+#  endif
+#  ifdef	DEP_DOSEMU
+	origfdtype = copydosdrive(NULL, fdtype, &origmaxfdtype, maxfdtype);
+#  endif
+# endif	/* !DEP_DYNAMICLIST */
+
+# if	FD >= 3
+	cp = getversion(&n);
+	cp = strndup2(cp, n);
+	setenv2(FDVERSION, cp, 0);
+	free2(cp);
 # endif
 }
 #endif	/* !_NOCUSTOMIZE */
 
-#ifndef	_NOLOGGING
+#ifdef	DEP_LOGGING
 static VOID NEAR startlog(argv)
-char **argv;
+char *CONST *argv;
 {
 # ifndef	NOUID
 	uid_t uid;
@@ -868,9 +945,9 @@ char **argv;
 		if (cp > buf) *(cp++) = ' ';
 		tmp = killmeta(argv[i]);
 		len = snprintf2(cp, (int)sizeof(buf) - (cp - buf), "%s", tmp);
-		free(tmp);
+		free2(tmp);
 		if (len < 0) break;
-		cp += len;
+		cp += strlen(cp);
 	}
 	*cp = '\0';
 
@@ -894,7 +971,7 @@ int status;
 # endif
 	char cwd[MAXPATHLEN];
 
-	if (!Xgetwd(cwd)) strcpy(cwd, "?");
+	if (!Xgetwd(cwd)) strcpy2(cwd, "?");
 # ifdef	NOUID
 	logmessage(_LOG_DEBUG_, "%s ends; PWD=%k; STATUS=%d",
 		progname, cwd, status);
@@ -905,13 +982,14 @@ int status;
 # endif
 	logclose();
 }
-#endif	/* !_NOLOGGING */
+#endif	/* DEP_LOGGING */
 
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 static int NEAR execruncomline(command, file, n, line)
-char *command, *file;
+char *command;
+CONST char *file;
 int n;
-char *line;
+CONST char *line;
 {
 	char *cp;
 	int i;
@@ -926,35 +1004,40 @@ char *line;
 		Xcprintf2("\t%s", line);
 		Xcputnl();
 	}
-	free(command);
+	free2(command);
 
 	return((i) ? -1 : 0);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* !DEP_ORIGSHELL */
 
 /*ARGSUSED*/
 int loadruncom(file, exist)
-char *file;
+CONST char *file;
 int exist;
 {
-#ifdef	_NOORIGSHELL
-	FILE *fp;
+#ifndef	DEP_ORIGSHELL
+# if	!MSDOS
+	struct stat st;
+# endif
+	XFILE *fp;
 	char *cp, *tmp, *fold, *line;
 	int cont;
-#endif
+#endif	/* !DEP_ORIGSHELL */
 	int n, er;
 
-#ifdef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
+	if ((n = isttyiomode)) Xstdiomode();
+	er = execruncom(file, 1);
+	if (n) Xttyiomode(n - 1);
+#else	/* !DEP_ORIGSHELL */
 # if	!MSDOS
 	tmp = NULL;
-	if (!exist && (tmp = getconstvar("TERM"))) {
-		struct stat st;
-
+	if (!exist && (tmp = getconstvar(ENVTERM))) {
 		cp = malloc2(strlen(file) + strlen(tmp) + 1 + 1);
-		strcpy(strcpy2(strcpy2(cp, file), "."), tmp);
+		strcpy2(strcpy2(strcpy2(cp, file), "."), tmp);
 		tmp = evalpath(cp, 0);
 		if (stat2(tmp, &st) < 0 || !s_isreg(&st)) {
-			free(tmp);
+			free2(tmp);
 			tmp = NULL;
 		}
 	}
@@ -964,31 +1047,31 @@ int exist;
 	fp = Xfopen(tmp, "r");
 	if (!fp) {
 		if (!exist) {
-			free(tmp);
+			free2(tmp);
 			return(0);
 		}
 		Xcprintf2("%s: Not found", tmp);
 		Xcputnl();
-		free(tmp);
+		free2(tmp);
 		return(-1);
 	}
 
 	fold = NULL;
 	n = er = 0;
-	while ((line = fgets2(fp, 0))) {
+	while ((line = Xfgets(fp))) {
 		n++;
 		if (*line == ';' || *line == '#') {
-			free(line);
+			free2(line);
 			continue;
 		}
 
 		cp = line + strlen(line);
-		for (cp--; cp >= line && (*cp == ' ' || *cp == '\t'); cp--);
+		for (cp--; cp >= line; cp--) if (!isblank2(*cp)) break;
 		cp[1] = '\0';
 
 		cont = 0;
-		if (cp >= line && *cp == META
-		&& (cp - 1 < line || *(cp - 1) != META)
+		if (cp >= line && *cp == ESCAPE
+		&& (cp - 1 < line || *(cp - 1) != ESCAPE)
 		&& !onkanji1(line, cp - line - 1)) {
 			*cp = '\0';
 			cont = 1;
@@ -1001,31 +1084,26 @@ int exist;
 		}
 
 		if (cont) {
-			if (fold != line) free(line);
+			if (fold != line) free2(line);
 			continue;
 		}
 
 		if (execruncomline(fold, tmp, n, line) < 0) er++;
-		if (fold != line) free(line);
+		if (fold != line) free2(line);
 		fold = NULL;
 	}
 
 	if (fold && execruncomline(fold, tmp, n, line) < 0) er++;
-	fclose(fp);
-	free(tmp);
-#else	/* !_NOORIGSHELL */
-	if ((n = isttyiomode)) Xstdiomode();
-	er = execruncom(file, 1);
-	if (n) Xttyiomode(n - 1);
-#endif	/* !_NOORIGSHELL */
+	Xfclose(fp);
+	free2(tmp);
+#endif	/* !DEP_ORIGSHELL */
 
 	return(er ? -1 : 0);
 }
 
-/*ARGSUSED*/
-static int NEAR initoption(argc, argv, envp)
+static int NEAR initoption(argc, argv)
 int argc;
-char *argv[], *envp[];
+char *CONST argv[];
 {
 	char *cp, **optv;
 	int i, optc;
@@ -1053,16 +1131,16 @@ char *argv[], *envp[];
 	}
 	optv[optc] = NULL;
 
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	if (initshell(optc, optv) < 0) Xexit2(RET_FAIL);
-#endif	/* !_NOORIGSHELL */
-	free(optv);
+#endif
+	free2(optv);
 
 	return(argc);
 }
 
 static int NEAR evaloption(argv)
-char *argv[];
+char *CONST argv[];
 {
 	char *cp, *tmp;
 	int i;
@@ -1074,16 +1152,17 @@ char *argv[];
 			break;
 		}
 		tmp = strdup2(&(argv[i][1]));
-		if ((cp = strchr(tmp, '='))) *(cp++) = '\0';
+		if ((cp = strchr2(tmp, '='))) *(cp++) = '\0';
 		setenv2(tmp, cp, 0);
-		free(tmp);
+		free2(tmp);
 	}
 
 	return(i);
 }
 
 static char *NEAR searchenv(s, envp)
-char *s, *envp[];
+CONST char *s;
+char *CONST envp[];
 {
 	int i, len;
 
@@ -1095,8 +1174,36 @@ char *s, *envp[];
 	return(NULL);
 }
 
+#if	!MSDOS
+static char *NEAR searchexecname(argv, envp)
+CONST char *argv;
+char *CONST envp[];
+{
+	CONST char *env;
+	char *cp;
+	int n;
+
+	n = 0;
+	if (!(env = searchenv(ENVPATH, envp))) env = DEFPATH;
+	if ((cp = searchexecpath(argv, env))) return(cp);
+	if (*argv == '-') {
+		if ((cp = searchexecpath(++argv, env))) return(cp);
+		n++;
+	}
+	if (*argv == 'r' && (cp = searchexecpath(++argv, env))) return(cp);
+	if ((cp = searchexecpath(FDSHELL, env))) return(cp);
+	if ((cp = searchexecpath(FDPROG, env))) return(cp);
+	if (n) {
+		getlogininfo(NULL, &env);
+		cp = strdup2(env);
+	}
+
+	return(cp);
+}
+#endif	/* !MSDOS */
+
 static VOID NEAR setexecname(argv)
-char *argv;
+CONST char *argv;
 {
 #if	MSDOS || defined (CYGWIN)
 	char *cp;
@@ -1104,7 +1211,7 @@ char *argv;
 
 	progname = getbasename(argv);
 #if	MSDOS || defined (CYGWIN)
-	if ((cp = strchr(progname, '.')) && cp > progname)
+	if ((cp = strchr2(progname, '.')) && cp > progname)
 		progname = strndup2(progname, cp - progname);
 	else
 #endif
@@ -1112,46 +1219,49 @@ char *argv;
 }
 
 static VOID NEAR setexecpath(argv, envp)
-char *argv, *envp[];
+CONST char *argv;
+char *CONST envp[];
 {
-	char *cp, buf[MAXPATHLEN];
+	CONST char *cp;
+	char *tmp, buf[MAXPATHLEN];
 
 	if (!Xgetwd(buf)) error(NOCWD_K);
 	origpath = strdup2(buf);
-	if ((cp = searchenv("PWD", envp))) {
+	if ((cp = searchenv(ENVPWD, envp))) {
 		*fullpath = '\0';
 		realpath2(cp, fullpath, 0);
-		realpath2(fullpath, buf, 1);
+		realpath2(fullpath, buf, RLP_READLINK);
 		if (!strpathcmp(origpath, buf)) {
-			free(origpath);
+			free2(origpath);
 			origpath = strdup2(fullpath);
 		}
 	}
-	strcpy(fullpath, origpath);
+	strcpy2(fullpath, origpath);
 
+	tmp = NULL;
 #if	MSDOS
 	cp = argv;
 #else
 	if (strdelim(argv, 0)) cp = argv;
-	else if ((cp = searchenv("PATH", envp))) cp = searchexecpath(argv, cp);
-	if (!cp) progpath = strdup2(origpath);
+	else cp = tmp = searchexecname(argv, envp);
+	if (!cp) progpath = strdup2(BINDIR);
 	else
 #endif
 	{
-		realpath2(cp, buf, 1);
-		if (cp != argv) free(cp);
-		if ((cp = strrdelim(buf, 0))) *cp = '\0';
+		realpath2(cp, buf, RLP_READLINK);
+		free2(tmp);
+		if ((tmp = strrdelim(buf, 0))) *tmp = '\0';
 		progpath = strdup2(buf);
 	}
 }
 
 /*ARGSUSED*/
 VOID initfd(argv)
-char **argv;
+char *CONST *argv;
 {
 	int i;
 
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	if (!interactive) /*EMPTY*/;
 	else
 #endif
@@ -1164,9 +1274,9 @@ char **argv;
 		}
 #endif	/* !MSDOS */
 		for (i = 0; i < 2; i++) loadhistory(i);
-		entryhist(1, origpath, 1);
+		entryhist(origpath, HST_PATH | HST_UNIQ);
 	}
-#ifndef	_NOLOGGING
+#ifdef	DEP_LOGGING
 	startlog(argv);
 #endif
 }
@@ -1178,10 +1288,10 @@ int status;
 	char cwd[MAXPATHLEN];
 	int i;
 
-#ifndef	_NOLOGGING
+#ifdef	DEP_LOGGING
 	endlog(status);
 #endif
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	if (!interactive) /*EMPTY*/;
 	else
 #endif
@@ -1193,50 +1303,51 @@ int status;
 		if (!Xgetwd(cwd)) *cwd = '\0';
 		rawchdir(rootpath);
 	}
-	free(origpath);
-	free(progname);
-#ifdef	_USEUNICODE
-	free(unitblpath);
+	free2(origpath);
+	free2(progname);
+#ifdef	DEP_UNICODE
+	free2(unitblpath);
 #endif
-#ifndef	_NOIME
-	free(dicttblpath);
+#ifdef	DEP_IME
+	free2(dicttblpath);
 #endif
-#ifndef	_NODOSDRIVE
+#ifndef	_NOCATALOG
+	free2(cattblpath);
+#endif
+#ifdef	DEP_DOSDRIVE
 	dosallclose();
 #endif
-	free(progpath);
+#ifdef	DEP_URLPATH
+	urlallclose();
+#endif
+	free2(progpath);
 
 #ifdef	DEBUG
-	if (tmpfilename) free(tmpfilename);
-# ifdef	_NOORIGSHELL
+	free2(tmpfilename);
+	tmpfilename = NULL;
+# ifndef	DEP_ORIGSHELL
 	freevar(environ);
 	freevar(environ2);
 # endif
 # ifndef	_NOCUSTOMIZE
-	if (orighelpindex) {
-		freestrarray(orighelpindex, MAXHELPINDEX);
-		free(orighelpindex);
-	}
-	free(origbindlist);
 #  ifndef	_NOKEYMAP
 	freekeyseq(origkeymaplist);
 #  endif
-#  ifndef	_NOARCHIVE
-	if (origlaunchlist) {
-		freelaunchlist(origlaunchlist, origmaxlaunch);
-		free(origlaunchlist);
-	}
-	if (origarchivelist) {
-		freearchlist(origarchivelist, origmaxarchive);
-		free(origarchivelist);
-	}
-#  endif
-#  ifdef	_USEDOSEMU
-	if (origfdtype) {
-		freedosdrive(origfdtype);
-		free(origfdtype);
-	}
-#  endif
+#  ifndef	DEP_DYNAMICLIST
+	freestrarray(orighelpindex, MAXHELPINDEX);
+	free2(orighelpindex);
+	free2(origbindlist);
+#   ifndef	_NOARCHIVE
+	freelaunchlist(origlaunchlist, origmaxlaunch);
+	free2(origlaunchlist);
+	freearchlist(origarchivelist, origmaxarchive);
+	free2(origarchivelist);
+#   endif
+#   ifdef	DEP_DOSEMU
+	freedosdrive(origfdtype, origmaxfdtype);
+	free2(origfdtype);
+#   endif
+#  endif	/* !DEP_DYNAMICLIST */
 # endif	/* !_NOCUSTOMIZE */
 	freeenvpath();
 	freehistory(0);
@@ -1246,27 +1357,30 @@ int status;
 	freeidlist();
 # endif
 	chdir2(NULL);
-# if	!defined (_NOUSEHASH) && defined (_NOORIGSHELL)
+# if	!defined (_NOUSEHASH) && !defined (DEP_ORIGSHELL)
 	searchhash(NULL, NULL, NULL);
 # endif
 # ifndef	_NOROCKRIDGE
 	detranspath(NULL, NULL);
 # endif
-# ifdef	_USEUNICODE
+# ifdef	DEP_UNICODE
 	discardunitable();
 # endif
-# ifndef	_NOIME
+# ifdef	DEP_IME
 	ime_freebuf();
 	freeroman(0);
 	discarddicttable();
 # endif
 #endif	/* DEBUG */
+#ifndef	_NOCATALOG
+	freecatalog();
+#endif
 	if (*cwd) rawchdir(cwd);
 }
 
 int main(argc, argv, envp)
 int argc;
-char *argv[], *envp[];
+char *CONST argv[], *CONST envp[];
 {
 	char *cp;
 	int i;
@@ -1334,86 +1448,94 @@ char *argv[], *envp[];
 #endif
 	sigvecset(0);
 
-#ifndef	_NOARCHIVE
-	for (maxlaunch = 0; maxlaunch < MAXLAUNCHTABLE; maxlaunch++) {
-		if (!launchlist[maxlaunch].ext) break;
-		launchlist[maxlaunch].ext =
-			strdup2(launchlist[maxlaunch].ext);
-		launchlist[maxlaunch].comm =
-			strdup2(launchlist[maxlaunch].comm);
-# if	FD >= 2
-		launchlist[maxlaunch].format =
-			duplvar(launchlist[maxlaunch].format, -1);
-		launchlist[maxlaunch].lignore =
-			duplvar(launchlist[maxlaunch].lignore, -1);
-		launchlist[maxlaunch].lerror =
-			duplvar(launchlist[maxlaunch].lerror, -1);
+#ifdef	DEP_DYNAMICLIST
+	helpindex = copystrarray(NULL, orighelpindex, NULL, MAXHELPINDEX);
+	for (i = 0; origbindlist[i].key >= 0; i++) /*EMPTY*/;
+	origmaxbind = i;
+	bindlist = copybind(NULL, origbindlist, &maxbind, origmaxbind);
+# ifndef	_NOARCHIVE
+	for (i = 0; origlaunchlist[i].ext; i++) /*EMPTY*/;
+	origmaxlaunch = i;
+	launchlist = copylaunch(NULL, origlaunchlist,
+		&maxlaunch, origmaxlaunch);
+	for (i = 0; origarchivelist[i].ext; i++) /*EMPTY*/;
+	origmaxarchive = i;
+	archivelist = copyarch(NULL, origarchivelist,
+		&maxarchive, origmaxarchive);
 # endif
-	}
-	for (maxarchive = 0; maxarchive < MAXARCHIVETABLE; maxarchive++) {
-		if (!archivelist[maxarchive].ext) break;
-		archivelist[maxarchive].ext =
-			strdup2(archivelist[maxarchive].ext);
-		archivelist[maxarchive].p_comm =
-			strdup2(archivelist[maxarchive].p_comm);
-		archivelist[maxarchive].u_comm =
-			strdup2(archivelist[maxarchive].u_comm);
-	}
-#endif	/* !_NOARCHIVE */
-
-	for (i = 0; i < MAXHELPINDEX; i++)
-		helpindex[i] = strdup2(helpindex[i]);
-#ifdef	_USEDOSEMU
-	for (i = 0; fdtype[i].name; i++) {
-		fdtype[i].name = strdup2(fdtype[i].name);
-# ifdef	HDDMOUNT
-		fdtype[i].offset = 0;
+# ifdef	DEP_DOSEMU
+	for (i = 0; origfdtype[i].name; i++) /*EMPTY*/;
+	origmaxfdtype = i;
+	fdtype = copydosdrive(NULL, origfdtype, &maxfdtype, origmaxfdtype);
 # endif
-	}
-#endif
+#else	/* !DEP_DYNAMICLIST */
+	copystrarray(helpindex, helpindex, NULL, MAXHELPINDEX);
+	for (i = 0; i < MAXBINDTABLE && bindlist[i].key >= 0; i++) /*EMPTY*/;
+	maxbind = i;
+# ifndef	_NOARCHIVE
+	for (i = 0; i < MAXLAUNCHTABLE && launchlist[i].ext; i++) /*EMPTY*/;
+	maxlaunch = i;
+	copylaunch(launchlist, launchlist, &maxlaunch, maxlaunch);
+	for (i = 0; i < MAXARCHIVETABLE && archivelist[i].ext; i++) /*EMPTY*/;
+	maxarchive = i;
+	copyarch(archivelist, archivelist, &maxarchive, maxarchive);
+# endif
+# ifdef	DEP_DOSEMU
+	for (i = 0; i < MAXDRIVEENTRY && fdtype[i].name; i++) /*EMPTY*/;
+	maxfdtype = i;
+	copydosdrive(fdtype, fdtype, &maxfdtype, maxfdtype);
+# endif
+#endif	/* !DEP_DYNAMICLIST */
 
-#ifdef	_NOORIGSHELL
-	for (i = 0; envp[i]; i++);
+#ifndef	DEP_ORIGSHELL
+	i = countvar(envp);
 	environ = (char **)malloc2((i + 1) * sizeof(char *));
 	for (i = 0; envp[i]; i++) environ[i] = strdup2(envp[i]);
 	environ[i] = NULL;
 #endif
 
-	initenv();
 	setexecpath(argv[0], envp);
-#ifdef	_USEUNICODE
+#ifdef	DEP_UNICODE
 	unitblpath = strdup2(DATADIR);
 #endif
-#ifndef	_NOIME
+#ifdef	DEP_IME
 	dicttblpath = strdup2(DATADIR);
 #endif
+#ifndef	_NOCATALOG
+# ifdef	USEDATADIR
+	cp = getversion(&i);
+	cattblpath = malloc2(strsize(DATADIR) + 1 + i + 1);
+	snprintf2(cattblpath, strsize(DATADIR) + 1 + i + 1,
+		"%s%c%-.*s", DATADIR, _SC_, i, cp);
+# else
+	cattblpath = strdup2(DATADIR);
+# endif
+#endif	/* !_NOCATALOG */
 
-#ifdef	_NOORIGSHELL
-	inittty(0);
-	getterment(NULL);
-	argc = initoption(argc, argv, envp);
-	adjustpath();
-#else	/* !_NOORIGSHELL */
+#ifdef	DEP_ORIGSHELL
 	cp = getshellname(progname, NULL, NULL);
 	if (!strpathcmp(cp, FDSHELL) || !strpathcmp(cp, "su")) {
 		i = main_shell(argc, argv, envp);
-# ifndef	_NOPTY
+# ifdef	DEP_PTY
 		killallpty();
 # endif
 		Xexit2(i);
 	}
-#ifndef	_NOORIGSHELL
-	interactive =
-#endif
-	fdmode = 1;
+	interactive = fdmode = 1;
 	setshellvar(envp);
 	prepareterm();
-	argc = initoption(argc, argv, envp);
+	argc = initoption(argc, argv);
 	if (dumbterm > 1) {
 		errno = 0;
 		error(NTERM_K);
 	}
-#endif	/* !_NOORIGSHELL */
+#else	/* !DEP_ORIGSHELL */
+	inittty(0);
+	getterment(NULL);
+	argc = initoption(argc, argv);
+	adjustpath();
+#endif	/* !DEP_ORIGSHELL */
+	initenv();
 
 	Xttyiomode(0);
 	initterm();
@@ -1454,15 +1576,12 @@ char *argv[], *envp[];
 	main_fd(&(argv[i]), 0);
 #endif
 	sigvecset(0);
-#ifndef	_NOLOGGING
-	endlog(0);
-#endif
 
 	Xstdiomode();
-#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 	killjob();
 #endif
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	killallpty();
 #endif
 	Xexit2(0);
