@@ -6,24 +6,14 @@
 
 #define	__FD_PRIMAL__
 #include "fd.h"
-
-#include <sys/param.h>
 #include "version.h"
 
-#ifndef	NOUNISTDH
-#include <unistd.h>
-#endif
-
-#ifndef	NOSTDLIBH
-#include <stdlib.h>
-#endif
-
 #ifdef	USESELECTH
 #include <sys/select.h>
 #endif
 
 #if	defined (USESYSCONF) && defined (_SC_OPEN_MAX)
-#define	MAXOPENFILE	sysconf(_SC_OPEN_MAX)
+#define	MAXOPENFILE		sysconf(_SC_OPEN_MAX)
 #else	/* !USESYSCONF || !_SC_OPEN_MAX */
 # ifdef	NOFILE
 # define	MAXOPENFILE	NOFILE
@@ -37,32 +27,32 @@
 #endif	/* !USESYSCONF || !_SC_OPEN_MAX */
 
 #ifndef	PREFIX
-#define	PREFIX		"/usr/local"
+#define	PREFIX			"/usr/local"
 #endif
 #ifndef	CONFDIR
-#define	CONFDIR		"/etc"
+#define	CONFDIR			"/etc"
 #endif
 #ifndef	CCCOMMAND
-#define	CCCOMMAND	"cc"
+#define	CCCOMMAND		"cc"
 #endif
 #ifndef	DICTSRC
-#define	DICTSRC		""
+#define	DICTSRC			""
 #endif
 
-int main __P_((int, char *[]));
+int main __P_((int, char *CONST []));
 
 
 /*ARGSUSED*/
 int main(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp;
 	int n;
 
 	printf("s:__VERMAJ__:%d:\n", FD);
 #if	FD >= 2
-	printf("s:__RCVERSION__:%d:\n", FD);
+	printf("s:__RCVERSION__:%d:\n", 2);
 #else
 	printf("s:__RCVERSION__::\n");
 #endif
@@ -80,14 +70,9 @@ char *argv[];
 	printf("s:__EXE__::g\n");
 #endif
 	printf("s:__OBJ__:.o:g\n");
-	printf("s:__EMUOBJS__:dosemu.o:\n");
-#ifdef	_NOIME
-	printf("s:__IMEOBJS__::\n");
-	printf("s:__DICTTBL__::\n");
-	printf("s:__DICTSRC__::\n");
-	printf("s:__MKDICTOPTION__::\n");
-#else	/* !_NOIME */
-	printf("s:__IMEOBJS__:ime.o roman.o dict.o:\n");
+	printf("s:__DOSOBJS__::\n");
+#ifdef	DEP_IME
+	printf("s:__IMEOBJS__:$(IMEOBJS):\n");
 	printf("s:__DICTTBL__:$(DICTTBL):\n");
 	if (DICTSRC[0]) {
 		printf("s:__DICTSRC__:%s:\n", DICTSRC);
@@ -97,16 +82,37 @@ char *argv[];
 		printf("s:__DICTSRC__:$(DICTTXT):\n");
 		printf("s:__MKDICTOPTION__::\n");
 	}
-#endif	/* !_NOIME */
-	printf("s:__OBJLIST__:$(OBJ1) $(OBJ2) $(OBJ3):\n");
+#else	/* !DEP_IME */
+	printf("s:__IMEOBJS__::\n");
+	printf("s:__DICTTBL__::\n");
+	printf("s:__DICTSRC__::\n");
+	printf("s:__MKDICTOPTION__::\n");
+#endif	/* !DEP_IME */
+#ifdef	_NOCATALOG
+	printf("s:__CATTBL__::\n");
+#else
+	printf("s:__CATTBL__:$(CATTBL) $(ECATTBL):\n");
+#endif
+#ifdef	_NOSOCKET
+	printf("s:__SOCKETOBJS__::\n");
+	printf("s:__SOCKETLIBS__::\n");
+#else
+	printf("s:__SOCKETOBJS__:$(SCKOBJS):\n");
+	printf("s:__SOCKETLIBS__:%s:\n", SOCKETLIB);
+#endif
+	printf("s:__OBJLIST__:$(OBJ1) $(OBJ2) $(OBJ3) $(OBJ4) $(OBJ5) $(OBJ6):\n");
+	printf("s:__SOBJLIST__:$(SOBJ1) $(SOBJ2):\n");
+	printf("s:__NOBJLIST__:$(NOBJ1) $(NOBJ2) $(NOBJ3):\n");
 	printf("s:__DEFRC__:'\"'$(DEFRC)'\"':\n");
 
 #ifdef	USEDATADIR
-	printf("s:__TBLPATH__:-DDATADIR='\"'$(DATADIR)'\"':\n");
+	printf("s:__TBLPATH__:-DBINDIR='\"'$(BINDIR)'\"' -DDATADIR='\"'$(DATADIR)'\"':\n");
 	printf("s:__DATADIR__:$(DATADIR):g\n");
+	printf("s:__DATADIR2__:$(DATADIR)/$(VERSION):g\n");
 #else
-	printf("s:__TBLPATH__::\n");
+	printf("s:__TBLPATH__:-DBINDIR='\"'$(BINDIR)'\"':\n");
 	printf("s:__DATADIR__:$(BINDIR):g\n");
+	printf("s:__DATADIR2__:$(BINDIR):g\n");
 #endif
 
 	printf("s:	__RENAME__:#	mv:\n");
@@ -139,7 +145,12 @@ char *argv[];
 	printf("s:__LN__:ln:\n");
 
 	printf("s:__CC__:%s:\n", CCCOMMAND);
+#if	!defined (__GNUC__) || !defined (__GNUC_MINOR__) \
+|| __GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 1)
 	printf("s:__CCOPTIONS__:%s:\n", EXTENDCCOPT);
+#else
+	printf("s:__CCOPTIONS__:%s -Wno-attributes:\n", EXTENDCCOPT);
+#endif
 #ifdef	HOSTCCCOMMAND
 	if (strcmp(CCCOMMAND, HOSTCCCOMMAND)) {
 		printf("s:__HOSTCC__:%s:\n", HOSTCCCOMMAND);
@@ -160,6 +171,7 @@ char *argv[];
 	printf("s:__MEM__::\n");
 	printf("s:__SHMEM__::\n");
 	printf("s:__BSHMEM__::\n");
+	printf("s:__NSHMEM__::\n");
 #ifdef	CCOUTOPT
 	printf("s:__OUT__:%s:\n", CCOUTOPT);
 #else
@@ -202,6 +214,8 @@ char *argv[];
 #else
 	printf("s:__PREFIXOPTION__:-c:\n");
 #endif
+	printf("s:[\t ]*$::\n");
+	printf("/^[\t ][\t ]*-*\\$(RM) *$/d\n");
 
 	return(0);
 }
