@@ -4,26 +4,54 @@
  *	command line parser
  */
 
+#ifdef	FD
 #include "fd.h"
-#include "func.h"
+#include "pathname.h"
+#include "term.h"
+#include "types.h"
+#include "kconv.h"
+#else
+#include "headers.h"
+#include "depend.h"
+#include "printf.h"
+#include "kctype.h"
+#include "string.h"
+#include "malloc.h"
+#endif
 
+#include "sysemu.h"
+#include "unixemu.h"
+#include "realpath.h"
+#include "parse.h"
+
+#ifdef	DEP_ORIGSHELL
+#include "system.h"
+#endif
 #ifdef	USEUNAME
 #include <sys/utsname.h>
 #endif
 
+#ifdef	FD
+# ifndef	DEP_ORIGSHELL
+extern int setenv2 __P_((CONST char *, CONST char *, int));
+# endif
+extern int underhome __P_((char *));
+#endif	/* !FD */
+
+#ifdef	FD
 extern char fullpath[];
 extern short histno[];
 extern int physical_path;
-
-static int NEAR asc2int __P_((int));
-#ifdef	_NOORIGSHELL
-static char *NEAR strtkbrk __P_((char *, char *, int));
-static char *NEAR geteostr __P_((char **));
-#else
-#include "system.h"
 #endif
 
-strtable keyidentlist[] = {
+static int NEAR asc2int __P_((int));
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
+static char *NEAR strtkbrk __P_((CONST char *, CONST char *, int));
+static char *NEAR geteostr __P_((CONST char **));
+#endif
+
+#ifdef	FD
+CONST strtable keyidentlist[] = {
 	{K_DOWN,	"DOWN"},
 	{K_UP,		"UP"},
 	{K_LEFT,	"LEFT"},
@@ -92,7 +120,8 @@ strtable keyidentlist[] = {
 	{K_HELP,	"HELP"},
 	{0,		NULL}
 };
-#define	KEYIDENTSIZ	(arraysize(keyidentlist) - 1)
+#define	KEYIDENTSIZ		(arraysize(keyidentlist) - 1)
+#endif	/* FD */
 
 static CONST char escapechar[] = "abefnrtv";
 static CONST char escapevalue[] = {
@@ -101,11 +130,11 @@ static CONST char escapevalue[] = {
 
 
 char *skipspace(cp)
-char *cp;
+CONST char *cp;
 {
 	while (isblank2(*cp)) cp++;
 
-	return(cp);
+	return((char *)cp);
 }
 
 static int NEAR asc2int(c)
@@ -121,17 +150,21 @@ int c;
 
 #ifdef	USESTDARGH
 /*VARARGS2*/
-char *sscanf2(char *s, CONST char *fmt, ...)
+char *sscanf2(CONST char *s, CONST char *fmt, ...)
 #else
 /*VARARGS2*/
 char *sscanf2(s, fmt, va_alist)
-char *s;
-CONST char *fmt;
+CONST char *s, *fmt;
 va_dcl
 #endif
 {
+#ifndef	HAVELONGLONG
+	char *buf;
+	u_long_t tmp;
+	int hi;
+#endif
 	va_list args;
-	char *cp;
+	CONST char *cp;
 	long_t n;
 	u_long_t u, mask;
 	int i, c, len, base, width, flags;
@@ -151,14 +184,14 @@ va_dcl
 		i++;
 		flags = 0;
 		for (; fmt[i]; i++) {
-			if (!(cp = strchr(printfflagchar, fmt[i]))) break;
+			if (!(cp = strchr2(printfflagchar, fmt[i]))) break;
 			flags |= printfflag[cp - printfflagchar];
 		}
 		width = getnum(fmt, &i);
 
 		len = sizeof(int);
 		for (; fmt[i]; i++) {
-			if (!(cp = strchr(printfsizechar, fmt[i]))) break;
+			if (!(cp = strchr2(printfsizechar, fmt[i]))) break;
 			len = printfsize[cp - printfsizechar];
 		}
 		if (fmt[i] == '*') {
@@ -286,10 +319,6 @@ va_dcl
 
 #ifndef	HAVELONGLONG
 		if (len > (int)sizeof(u_long_t)) {
-			char *buf;
-			u_long_t tmp;
-			int hi;
-
 			hi = 0;
 			if (!(flags & VF_UNSIGNED)) {
 				mask = (MAXUTYPE(u_long_t) >> 1);
@@ -319,15 +348,25 @@ va_dcl
 	}
 	va_end(args);
 
-	return(s);
+	return((char *)s);
 }
 
-#ifdef	_NOORIGSHELL
+int atoi2(s)
+CONST char *s;
+{
+	int n;
+
+	if (!sscanf2(s, "%<d%$", &n)) return(-1);
+
+	return(n);
+}
+
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
 static char *NEAR strtkbrk(s, c, evaldq)
-char *s, *c;
+CONST char *s, *c;
 int evaldq;
 {
-	char *cp;
+	CONST char *cp;
 	int pc, quote;
 
 	for (cp = s, quote = '\0'; *cp; cp++) {
@@ -347,18 +386,19 @@ int evaldq;
 		}
 		else if (pc == PC_ESCAPE) {
 			cp++;
-			if (*cp == PMETA && strchr(c, *cp)) return(cp - 1);
+			if (*cp == PMETA && strchr2(c, *cp))
+				return((char *)&(cp[-1]));
 			continue;
 		}
 
-		if (strchr(c, *cp)) return(cp);
+		if (strchr2(c, *cp)) return((char *)cp);
 	}
 
 	return(NULL);
 }
 
 char *strtkchr(s, c, evaldq)
-char *s;
+CONST char *s;
 int c, evaldq;
 {
 	char tmp[2];
@@ -370,9 +410,10 @@ int c, evaldq;
 }
 
 static char *NEAR geteostr(strp)
-char **strp;
+CONST char **strp;
 {
-	char *cp, *tmp;
+	CONST char *cp;
+	char *tmp;
 	int len;
 
 	cp = *strp;
@@ -384,9 +425,10 @@ char **strp;
 }
 
 int getargs(s, argvp)
-char *s, ***argvp;
+CONST char *s;
+char ***argvp;
 {
-	char *cp;
+	CONST char *cp;
 	int i;
 
 	*argvp = (char **)malloc2(1 * sizeof(char *));
@@ -402,36 +444,36 @@ char *s, ***argvp;
 }
 
 char *gettoken(s)
-char *s;
+CONST char *s;
 {
 	if (!isidentchar(*s)) return(NULL);
-	for (s++; isidentchar2(*s); s++);
+	for (s++; isidentchar2(*s); s++) /*EMPTY*/;
 
-	return(s);
+	return((char *)s);
 }
 
 char *getenvval(argcp, argv)
 int *argcp;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp;
 	int i;
 
-	if (*argcp <= 0) return((char *)-1);
+	if (*argcp <= 0) return(vnullstr);
 	i = 0;
-	if (!isidentchar(argv[i][0])) return((char *)-1);
+	if (!isidentchar(argv[i][0])) return(vnullstr);
 
 	for (cp = &(argv[i][1]); *cp; cp++) if (!isidentchar2(*cp)) break;
 	cp = skipspace(cp);
 	if (!*cp) {
-		if (++i >= *argcp) return((char *)-1);
+		if (++i >= *argcp) return(vnullstr);
 		cp = skipspace(argv[i]);
 	}
-	if (*cp != '=') return((char *)-1);
+	if (*cp != '=') return(vnullstr);
 	*(cp++) = '\0';
 
 	if (!*cp) {
-		if (++i >= *argcp) return((char *)NULL);
+		if (++i >= *argcp) return(NULL);
 		cp = argv[i];
 	}
 	*argcp = i + 1;
@@ -440,20 +482,22 @@ char *argv[];
 }
 
 char *evalcomstr(path, delim)
-char *path, *delim;
+CONST char *path, *delim;
 {
-# ifndef	_NOKANJIFCONV
+# ifdef	DEP_FILECONV
 	char *tmp;
 # endif
-	char *cp, *next, *epath;
+	CONST char *cp, *next;
+	char *new, *epath;
 	int i, len, size;
 
-	epath = next = NULL;
+	next = epath = NULL;
 	size = 0;
 	for (cp = path; cp && *cp; cp = next) {
 		if ((next = strtkbrk(cp, delim, 0))) {
 			len = next - cp;
-			for (i = 1; next[i] && strchr(delim, next[i]); i++);
+			for (i = 1; next[i]; i++)
+				if (!strchr2(delim, next[i])) break;
 		}
 		else {
 			len = strlen(cp);
@@ -461,23 +505,30 @@ char *path, *delim;
 		}
 		next = cp + len;
 		if (len) {
-			cp = _evalpath(cp, next, EA_NOEVALQ | EA_NOUNIQDELIM);
-# ifndef	_NOKANJIFCONV
-			tmp = newkanjiconv(cp, DEFCODE, getkcode(cp), L_FNAME);
-			if (cp != tmp) free(cp);
-			cp = tmp;
+			new = _evalpath(cp, next, EA_NOEVALQ | EA_NOUNIQDELIM);
+# ifdef	DEP_FILECONV
+			tmp = newkanjiconv(new,
+				DEFCODE, getkcode(new), L_FNAME);
+			if (new != tmp) {
+				free2(new);
+				new = tmp;
+			}
 # endif
+			cp = new;
 			len = strlen(cp);
 		}
+# ifdef	FAKEUNINIT
+		else new = NULL;	/* fake for -Wuninitialized */
+# endif
 
 		epath = realloc2(epath, size + len + i + 1);
 		if (len) {
-			strcpy(&(epath[size]), cp);
-			free(cp);
+			strcpy2(&(epath[size]), cp);
+			free2(new);
 			size += len;
 		}
 		if (i) {
-			strncpy(&(epath[size]), next, i);
+			strncpy2(&(epath[size]), next, i);
 			size += i;
 			next += i;
 		}
@@ -488,37 +539,38 @@ char *path, *delim;
 
 	return(epath);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* FD && !DEP_ORIGSHELL */
 
 char *evalpaths(paths, delim)
-char *paths;
+CONST char *paths;
 int delim;
 {
-	char *cp, *tmp, *next, *epath, buf[MAXPATHLEN];
+	CONST char *cp, *next;
+	char *new, *epath, buf[MAXPATHLEN];
 	int len, size;
 
-	epath = next = NULL;
+	next = epath = NULL;
 	size = 0;
 	for (cp = paths; cp; cp = next) {
-#ifdef	_USEDOSPATH
-		if (_dospath(cp)) next = strchr(&(cp[2]), delim);
+#ifdef	DEP_DOSPATH
+		if (_dospath(cp)) next = strchr2(&(cp[2]), delim);
 		else
 #endif
-		next = strchr(cp, delim);
+		next = strchr2(cp, delim);
 		len = (next) ? (next++) - cp : strlen(cp);
 		if (len) {
-			tmp = _evalpath(cp, cp + len, 0);
-			if (!isrootdir(cp)) cp = tmp;
-			else cp = realpath2(tmp, buf, 1);
+			new = _evalpath(cp, &(cp[len]), 0);
+			cp = (isrootdir(cp))
+				? realpath2(new, buf, RLP_READLINK) : new;
 			len = strlen(cp);
 		}
 #ifdef	FAKEUNINIT
-		else tmp = NULL;	/* fake for -Wuninitialized */
+		else new = NULL;	/* fake for -Wuninitialized */
 #endif
 		epath = realloc2(epath, size + len + 1 + 1);
 		if (len) {
-			strcpy(&(epath[size]), cp);
-			free(tmp);
+			strcpy2(&(epath[size]), cp);
+			free2(new);
 		}
 		size += len;
 		if (next) epath[size++] = delim;
@@ -530,11 +582,12 @@ int delim;
 	return(epath);
 }
 
-#if	!MSDOS || !defined (_NOORIGSHELL)
+#if	!MSDOS || !defined (FD) || defined (DEP_ORIGSHELL)
 char *killmeta(name)
-char *name;
+CONST char *name;
 {
-	char *cp, *buf;
+	CONST char *cp;
+	char *buf, *new;
 	int i;
 
 	buf = malloc2(strlen(name) * 2 + 2 + 1);
@@ -546,60 +599,63 @@ char *name;
 # endif
 		else if (*cp == PMETA) {
 			*buf = '"';
-			if (strchr(DQ_METACHAR, *(cp + 1))) buf[i++] = PMETA;
+			if (strchr2(DQ_METACHAR, *(cp + 1))) buf[i++] = PMETA;
 		}
-		else if (strchr(METACHAR, *cp)) {
+		else if (strchr2(METACHAR, *cp)) {
 			*buf = '"';
-			if (strchr(DQ_METACHAR, *cp)) buf[i++] = PMETA;
+			if (strchr2(DQ_METACHAR, *cp)) buf[i++] = PMETA;
 		}
 		buf[i] = *cp;
 	}
 	if (*(cp = buf)) buf[i++] = *cp;
 	else cp++;
 	buf[i] = '\0';
-	cp = strdup2(cp);
-	free(buf);
+	new = strdup2(cp);
+	free2(buf);
 
-	return(cp);
+	return(new);
 }
-#endif	/* !MSDOS || !_NOORIGSHELL */
+#endif	/* !MSDOS || !FD || DEP_ORIGSHELL */
 
-#ifdef	_NOORIGSHELL
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
 VOID adjustpath(VOID_A)
 {
 	char *cp, *path;
 
-	if (!(cp = getconstvar("PATH"))) return;
+	if (!(cp = (char *)getenv(ENVPATH))) return;
 
 	path = evalpaths(cp, PATHDELIM);
-	if (strpathcmp(path, cp)) setenv2("PATH", path, 1);
-	free(path);
+	if (strpathcmp(path, cp)) setenv2(ENVPATH, path, 1);
+	free2(path);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* FD && !DEP_ORIGSHELL */
 
+#ifdef	FD
 char *includepath(path, plist)
-char *path, *plist;
+CONST char *path, *plist;
 {
-	char *cp, *tmp, *next;
+	CONST char *cp, *next;
+	char *tmp;
 	int len;
 
 	if (!plist || !*plist) return(NULL);
 	next = plist;
 	for (cp = next; cp && *cp; cp = next) {
-#ifdef	_USEDOSPATH
+# ifdef	DEP_DOSPATH
 		if (_dospath(cp)) next += 2;
-#endif
-		next = strchr(next, PATHDELIM);
+# endif
+		next = strchr2(next, PATHDELIM);
 		len = (next) ? (next++) - cp : strlen(cp);
 		if ((tmp = underpath(path, cp, len))) return(tmp);
 	}
 
 	return(NULL);
 }
+#endif	/* FD */
 
-#if	(FD < 2) && !defined (_NOARCHIVE)
+#if	defined (OLDPARSE) && !defined (_NOARCHIVE)
 char *getrange(cp, delim, fp, dp, wp)
-char *cp;
+CONST char *cp;
 int delim;
 u_char *fp, *dp, *wp;
 {
@@ -609,7 +665,7 @@ u_char *fp, *dp, *wp;
 	*fp = *dp = *wp = 0;
 
 	if (!(cp = sscanf2(cp, "%c%Cu", delim, &c))) return(NULL);
-	*fp = (c) ? c - 1 : 255;
+	*fp = (c) ? c - 1 : FLD_NONE;
 
 	if (*cp == '[') {
 		if (!(cp = sscanf2(++cp, "%Cu]", &c))) return(NULL);
@@ -631,12 +687,13 @@ u_char *fp, *dp, *wp;
 		else return(NULL);
 	}
 
-	return(cp);
+	return((char *)cp);
 }
-#endif	/* (FD < 2) && !_NOARCHIVE */
+#endif	/* OLDPARSE && !_NOARCHIVE */
 
 int evalprompt(bufp, prompt)
-char **bufp, *prompt;
+char **bufp;
+CONST char *prompt;
 {
 #ifndef	NOUID
 	uidtable *up;
@@ -644,14 +701,16 @@ char **bufp, *prompt;
 #if	!MSDOS && defined (USEUNAME)
 	struct utsname uts;
 #endif
-	char *cp, *tmp, line[MAXPATHLEN];
+	char *cp, *tmp, *new, line[MAXPATHLEN];
 	ALLOC_T size;
 	int i, j, k, len, unprint;
 
-#ifdef	_NOORIGSHELL
-	prompt = evalpath(strdup2(prompt), EA_NOUNIQDELIM);
+	cp = strdup2(prompt);
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
+	prompt = new = evalpath(cp, EA_NOUNIQDELIM);
 #else
-	prompt = evalvararg(prompt, '\0', EA_BACKQ | EA_KEEPMETA, 0);
+	prompt = new = evalvararg(cp, '\0', EA_BACKQ | EA_KEEPMETA, 0);
+	free2(cp);
 #endif
 	unprint = 0;
 #ifdef	FAKEUNINIT
@@ -676,10 +735,12 @@ char **bufp, *prompt;
 				*line = META;
 				line[1] = '\0';
 				break;
+#ifdef	FD
 			case '!':
 				snprintf2(line, sizeof(line), "%d",
 					(int)histno[0] + 1);
 				break;
+#endif
 #ifndef	NOUID
 			case 'u':
 				if ((up = finduid(getuid(), NULL)))
@@ -691,39 +752,50 @@ char **bufp, *prompt;
 			case 'H':
 # ifdef	USEUNAME
 				uname(&uts);
-				strcpy(line, uts.nodename);
+				strcpy2(line, uts.nodename);
 # else
 				gethostname(line, MAXPATHLEN);
 # endif
 				if (prompt[i] == 'h'
-				&& (tmp = strchr(line, '.')))
+				&& (tmp = strchr2(line, '.')))
 					*tmp = '\0';
 				break;
 			case '$':
 				*line = (getuid()) ? '$' : '#';
 				line[1] = '\0';
 				break;
+#endif	/* !MSDOS */
+			case '~':
+#ifdef	FD
+				if (underhome(&(line[1]))) {
+					line[0] = '~';
+					break;
+				}
 #endif
+/*FALLTHRU*/
 			case 'w':
+#ifdef	FD
 				if (!physical_path || !Xgetwd(line))
 					cp = fullpath;
+#else
+				if (!Xgetwd(line)) cp = vnullstr;
+#endif
 				break;
 			case 'W':
+#ifdef	FD
 				if (!physical_path || !Xgetwd(line))
 					tmp = fullpath;
+#else
+				if (!Xgetwd(line)) tmp = vnullstr;
+#endif
 				else tmp = line;
-#ifdef	_USEDOSPATH
+#ifdef	DEP_DOSPATH
 				if (_dospath(tmp)) tmp += 2;
 #endif
 				cp = strrdelim(tmp, 0);
 				if (cp && (cp != tmp || cp[1])) cp++;
 				else cp = tmp;
 				break;
-			case '~':
-				if (underhome(&(line[1]))) line[0] = '~';
-				else if (!physical_path || !Xgetwd(line))
-					cp = fullpath;
-				break;
 			case 'e':
 				*line = '\033';
 				line[1] = '\0';
@@ -772,20 +844,20 @@ char **bufp, *prompt;
 		}
 	}
 	(*bufp)[j] = '\0';
-	free(prompt);
+	free2(new);
 
 	return(len);
 }
 
-#ifndef	_NOARCHIVE
+#if	defined (FD) && !defined (_NOARCHIVE)
 char *getext(ext, flagsp)
-char *ext;
+CONST char *ext;
 u_char *flagsp;
 {
 	char *tmp;
 
 	*flagsp = 0;
-# if	FD >= 2
+# ifndef	OLDPARSE
 	if (*ext == '/') {
 		ext++;
 		*flagsp |= LF_IGNORECASE;
@@ -796,7 +868,7 @@ u_char *flagsp;
 	else {
 		tmp = malloc2(strlen(ext) + 2);
 		*tmp = '*';
-		strcpy(&(tmp[1]), ext);
+		strcpy2(&(tmp[1]), ext);
 	}
 
 	return(tmp);
@@ -804,9 +876,9 @@ u_char *flagsp;
 
 /*ARGSUSED*/
 int extcmp(ext1, flags1, ext2, flags2, strict)
-char *ext1;
+CONST char *ext1;
 int flags1;
-char *ext2;
+CONST char *ext2;
 int flags2, strict;
 {
 	if (*ext1 == '*') ext1++;
@@ -819,10 +891,11 @@ int flags2, strict;
 
 	return(strpathcmp(ext1, ext2));
 }
-#endif	/* !_NOARCHIVE */
+#endif	/* FD && !_NOARCHIVE */
 
+#ifdef	FD
 int getkeycode(cp, identonly)
-char *cp;
+CONST char *cp;
 int identonly;
 {
 	char *tmp;
@@ -855,12 +928,12 @@ int identonly;
 			if (identonly) return(-1);
 			ch = (isalpha2(*cp)) ? mkmetakey(*(cp++)) : -1;
 			break;
-#ifdef	CODEEUC
+# ifdef	CODEEUC
 		case C_EKANA:
 			if (identonly) return(-1);
 			ch = (iskana2(*cp)) ? mkekana(*(cp++)) : -1;
 			break;
-#endif
+# endif
 		case 'F':
 			if ((i = atoi2(cp)) >= 1 && i <= 20) return(K_F(i));
 /*FALLTHRU*/
@@ -877,10 +950,10 @@ int identonly;
 	return(ch);
 }
 
-char *getkeysym(c, tenkey)
+CONST char *getkeysym(c, tenkey)
 int c, tenkey;
 {
-	static char buf[5];
+	static char buf[4 + 1];
 	int i;
 
 	for (i = 0; i < KEYIDENTSIZ; i++)
@@ -898,19 +971,18 @@ int c, tenkey;
 		buf[i++] = '@';
 		buf[i++] = c & 0x7f;
 	}
-#ifdef	CODEEUC
+# ifdef	CODEEUC
 	else if (isekana2(c)) {
 		buf[i++] = (char)C_EKANA;
 		buf[i++] = c & 0xff;
 	}
-#endif
+# else
+	else if (iskana2(c)) buf[i++] = c;
+# endif
 	else if (c > (int)MAXUTYPE(u_char)) {
 		buf[i++] = '?';
 		buf[i++] = '?';
 	}
-#ifndef	CODEEUC
-	else if (iskana2(c)) buf[i++] = c;
-#endif
 	else if (iscntrl2(c)) {
 		for (i = 0; escapechar[i]; i++)
 			if (c == escapevalue[i]) break;
@@ -924,9 +996,10 @@ int c, tenkey;
 
 	return(buf);
 }
+#endif	/* FD */
 
 char *decodestr(s, lenp, evalhat)
-char *s;
+CONST char *s;
 u_char *lenp;
 int evalhat;
 {
@@ -953,18 +1026,18 @@ int evalhat;
 		else cp[j] = s[i];
 	}
 
-	s = malloc2(j + 1);
-	memcpy(s, cp, j);
-	s[j] = '\0';
-	free(cp);
+	tmp = malloc2(j + 1);
+	memcpy(tmp, cp, j);
+	tmp[j] = '\0';
+	free2(cp);
 	if (lenp) *lenp = j;
 
-	return(s);
+	return(tmp);
 }
 
-#ifndef	_NOKEYMAP
+#if	defined (FD) && !defined (_NOKEYMAP)
 char *encodestr(s, len)
-char *s;
+CONST char *s;
 int len;
 {
 	char *cp;
@@ -986,11 +1059,16 @@ int len;
 				cp[j++] = '\\';
 				cp[j++] = escapechar[n];
 			}
-			else if (iscntrl2(s[i]))
-				j = snprintf2(&(cp[j]), len * 4 + 1 - j,
+			else if (iscntrl2(s[i])) {
+				snprintf2(&(cp[j]), len * 4 + 1 - j,
 					"^%c", (s[i] + '@') & 0x7f);
-			else j += snprintf2(&(cp[j]), len * 4 + 1 - j,
-				"\\%03o", s[i] & 0xff);
+				j += strlen(&(cp[j]));
+			}
+			else {
+				snprintf2(&(cp[j]), len * 4 + 1 - j,
+					"\\%03o", s[i] & 0xff);
+				j += strlen(&(cp[j]));
+			}
 			continue;
 		}
 		cp[j++] = s[i];
@@ -999,4 +1077,4 @@ int len;
 
 	return(cp);
 }
-#endif	/* !_NOKEYMAP */
+#endif	/* FD && !_NOKEYMAP */
