@@ -4,105 +4,48 @@
  *	XSH command for POSIX
  */
 
-#include <fcntl.h>
 #ifdef	FD
 #include "fd.h"
-#else	/* !FD */
-#include "machine.h"
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
-
-#ifndef	NOUNISTDH
-#include <unistd.h>
-#endif
-
-#ifndef	NOSTDLIBH
-#include <stdlib.h>
-#endif
-
+#else
+#include "headers.h"
+#include "depend.h"
 #include "printf.h"
 #include "kctype.h"
-#include "pathname.h"
-#endif	/* !FD */
-
-#if	MSDOS
-#include "unixemu.h"
-#else
-#include <sys/param.h>
+#include "string.h"
+#include "malloc.h"
 #endif
 
+#include "sysemu.h"
+#include "unixemu.h"
+
+#if	!MSDOS
+#include "time.h"
+#endif
+#ifdef	DEP_ORIGSHELL
 #include "system.h"
-
-#if	!defined (MINIMUMSHELL) \
-&& (!defined (FD) || (FD >= 2 && !defined (_NOORIGSHELL)))
-
+#endif
+#if	defined (DEP_ORIGSHELL) && !defined (MINIMUMSHELL)
+#include "posixsh.h"
+#endif
 #ifdef	FD
 #include "term.h"
-extern int Xstat __P_((char *, struct stat *));
-extern int Xlstat __P_((char *, struct stat *));
-extern int Xaccess __P_((char *, int));
-# ifndef	_NOPTY
+#endif
+#ifdef	DEP_PTY
 #include "termemu.h"
-extern VOID sendparent __P_((int, ...));
-# endif
-#else	/* !FD */
-extern int ttyio;
-extern FILE *ttyout;
-# if	MSDOS
-extern int Xstat __P_((char *, struct stat *));
-# define	Xlstat		Xstat
-# else
-# define	Xstat(p, s)	((stat(p, s)) ? -1 : 0)
-# define	Xlstat(p, s)	((lstat(p, s)) ? -1 : 0)
-# endif
-#define	Xaccess(p, m)	((access(p, m)) ? -1 : 0)
-#endif	/* !FD */
-
-#ifndef	EPERM
-#define	EPERM		EACCES
 #endif
 
-extern VOID error __P_((char *));
-extern char *malloc2 __P_((ALLOC_T));
-extern char *realloc2 __P_((VOID_P, ALLOC_T));
-extern char *strdup2 __P_((char *));
-extern char *strndup2 __P_((char *, int));
-extern char *strchr2 __P_((char *, int));
-extern char *strncpy2 __P_((char *, char *, int));
-extern int setenv2 __P_((char *, char *, int));
-extern time_t time2 __P_((VOID_A));
-
-#ifndef	NOJOB
-static char *NEAR headstree __P_((syntaxtree *));
-#endif
-#ifndef	NOALIAS
-static int NEAR searchalias __P_((char *, int));
-static shaliastable *NEAR duplalias __P_((shaliastable *));
-static int cmpalias __P_((CONST VOID_P, CONST VOID_P));
-#endif
-static int NEAR _evalexpression __P_((char *, int, long *));
-static int NEAR evalexpression __P_((char *, int, long *, int));
-#if	!MSDOS
-static VOID NEAR addmailpath __P_((char *, char *, time_t));
-#endif
-static int NEAR testsub1 __P_((int, char *, int *));
-static int NEAR testsub2 __P_((int, char *[], int *));
-static int NEAR testsub3 __P_((int, char *[], int *, int));
-
-#define	ALIASDELIMIT	"();&|<>"
-#define	BUFUNIT		32
-#define	getconstvar(s)	(getshellvar(s, strsize(s)))
-#define	ER_COMNOFOUND	1
-#define	ER_NOTFOUND	2
-#define	ER_NOTIDENT	4
-#define	ER_BADOPTIONS	13
-#define	ER_NUMOUTRANGE	18
-#define	ER_NOTALIAS	20
-#define	ER_MISSARG	21
-#define	ER_NOSUCHJOB	22
-#define	ER_TERMINATED	23
-#define	ER_UNKNOWNSIG	26
+#define	ALIASDELIMIT		"();&|<>"
+#define	getconstvar(s)		(getshellvar(s, strsize(s)))
+#define	ER_COMNOFOUND		1
+#define	ER_NOTFOUND		2
+#define	ER_NOTIDENT		4
+#define	ER_BADOPTIONS		13
+#define	ER_NUMOUTRANGE		18
+#define	ER_NOTALIAS		20
+#define	ER_MISSARG		21
+#define	ER_NOSUCHJOB		22
+#define	ER_TERMINATED		23
+#define	ER_UNKNOWNSIG		26
 
 #if	!MSDOS
 typedef struct _mailpath_t {
@@ -112,42 +55,34 @@ typedef struct _mailpath_t {
 } mailpath_t;
 #endif
 
+#if	defined (DEP_ORIGSHELL) && !defined (MINIMUMSHELL)
+
+extern int setenv2 __P_((CONST char *, CONST char *, int));
+#ifdef	DEP_PTY
+extern VOID sendparent __P_((int, ...));
+#endif
+
+#ifndef	FD
+extern int ttyio;
+extern XFILE *ttyout;
+#endif
+
 #ifndef	NOJOB
-int gettermio __P_((p_id_t, int));
-VOID dispjob __P_((int, FILE *));
-int searchjob __P_((p_id_t, int *));
-int getjob __P_((char *));
-int stackjob __P_((p_id_t, int, syntaxtree *));
-int stoppedjob __P_((p_id_t));
-VOID killjob __P_((VOID_A));
-VOID checkjob __P_((FILE *));
-#endif	/* !NOJOB */
-char *evalposixsubst __P_((char *, int *));
+static CONST char *NEAR headstree __P_((syntaxtree *));
+#endif
+#ifndef	NOALIAS
+static int NEAR searchalias __P_((CONST char *, int));
+static shaliastable *NEAR duplalias __P_((shaliastable *));
+static int cmpalias __P_((CONST VOID_P, CONST VOID_P));
+#endif
+static int NEAR _evalexpression __P_((CONST char *, int, long *));
+static int NEAR evalexpression __P_((CONST char *, int, long *, int));
 #if	!MSDOS
-VOID replacemailpath __P_((char *, int));
-VOID checkmail __P_((int));
-#endif
-#ifndef	NOALIAS
-int addalias __P_((char *, char *));
-int deletealias __P_((char *));
-VOID freealias __P_((shaliastable *));
-int checkalias __P_((syntaxtree *, char *, int, int));
-#endif
-#ifndef	NOJOB
-int posixjobs __P_((syntaxtree *));
-int posixfg __P_((syntaxtree *));
-int posixbg __P_((syntaxtree *));
-#endif
-#ifndef	NOALIAS
-int posixalias __P_((syntaxtree *));
-int posixunalias __P_((syntaxtree *));
-#endif
-int posixkill __P_((syntaxtree *));
-int posixtest __P_((syntaxtree *));
-#ifndef	NOPOSIXUTIL
-int posixcommand __P_((syntaxtree *));
-int posixgetopts __P_((syntaxtree *));
+static VOID NEAR addmailpath __P_((CONST char *, char *, time_t));
 #endif
+static int NEAR testsub1 __P_((int, CONST char *, int *));
+static int NEAR testsub2 __P_((int, char *CONST *, int *));
+static int NEAR testsub3 __P_((int, char *CONST *, int *, int));
 
 #ifndef	NOJOB
 jobtable *joblist = NULL;
@@ -209,7 +144,7 @@ int job;
 	return(ret);
 }
 
-static char *NEAR headstree(trp)
+static CONST char *NEAR headstree(trp)
 syntaxtree *trp;
 {
 	int id;
@@ -231,7 +166,7 @@ syntaxtree *trp;
 
 VOID dispjob(n, fp)
 int n;
-FILE *fp;
+XFILE *fp;
 {
 	int i, sig;
 
@@ -276,9 +211,9 @@ int *np;
 }
 
 int getjob(s)
-char *s;
+CONST char *s;
 {
-	char *cp;
+	CONST char *cp;
 	int i, j;
 
 	if (!jobok) return(-1);
@@ -372,11 +307,7 @@ syntaxtree *trp;
 	joblist[i].pids[j] = pid;
 	joblist[i].stats[j] = sig;
 	if (!j && !(joblist[i].trp) && trp) {
-#ifdef	MINIMUMSHELL
-		joblist[i].trp = duplstree(trp, NULL);
-#else
 		joblist[i].trp = duplstree(trp, NULL, 0L);
-#endif
 		if (!isoppipe(joblist[i].trp) && joblist[i].trp -> next) {
 			freestree(joblist[i].trp -> next);
 			joblist[i].trp -> next = NULL;
@@ -411,7 +342,7 @@ p_id_t pid;
 }
 
 VOID checkjob(fp)
-FILE *fp;
+XFILE *fp;
 {
 	int i, j;
 
@@ -426,15 +357,15 @@ FILE *fp;
 		&& (isopbg(joblist[i].trp) || isopnown(joblist[i].trp)))
 			joblist[i].trp -> type = OP_NONE;
 		if (jobok && interactive && !nottyout) dispjob(i, fp);
-		free(joblist[i].pids);
-		free(joblist[i].stats);
+		free2(joblist[i].pids);
+		free2(joblist[i].stats);
 		if (joblist[i].trp) {
 			freestree(joblist[i].trp);
-			free(joblist[i].trp);
+			free2(joblist[i].trp);
 		}
-		if (joblist[i].tty) free(joblist[i].tty);
+		free2(joblist[i].tty);
 # ifdef	USESGTTY
-		if (joblist[i].ttyflag) free(joblist[i].ttyflag);
+		free2(joblist[i].ttyflag);
 # endif
 		joblist[i].pids = NULL;
 	}
@@ -456,7 +387,7 @@ VOID killjob(VOID_A)
 
 #ifndef	NOALIAS
 static int NEAR searchalias(ident, len)
-char *ident;
+CONST char *ident;
 int len;
 {
 	int i;
@@ -477,8 +408,8 @@ char *ident, *comm;
 
 	i = searchalias(ident, -1);
 	if (shellalias[i].ident) {
-		free(shellalias[i].ident);
-		free(shellalias[i].comm);
+		free2(shellalias[i].ident);
+		free2(shellalias[i].comm);
 	}
 	else {
 		shellalias = (shaliastable *)realloc2(shellalias,
@@ -488,7 +419,7 @@ char *ident, *comm;
 
 	shellalias[i].ident = ident;
 	shellalias[i].comm = comm;
-# if	defined (FD) && !defined (_NOPTY)
+# ifdef	DEP_PTY
 	sendparent(TE_ADDALIAS, ident, comm);
 # endif
 
@@ -496,29 +427,29 @@ char *ident, *comm;
 }
 
 int deletealias(ident)
-char *ident;
+CONST char *ident;
 {
 	reg_t *re;
 	int i, n, max;
 
 	n = 0;
 	re = regexp_init(ident, -1);
-	for (max = 0; shellalias[max].ident; max++);
+	for (max = 0; shellalias[max].ident; max++) /*EMPTY*/;
 	for (i = 0; i < max; i++) {
 		if (re) {
 			if (!regexp_exec(re, shellalias[i].ident, 0)) continue;
 		}
 		else if (strcommcmp(ident, shellalias[i].ident)) continue;
-		free(shellalias[i].ident);
-		free(shellalias[i].comm);
+		free2(shellalias[i].ident);
+		free2(shellalias[i].comm);
 		memmove((char *)&(shellalias[i]), (char *)&(shellalias[i + 1]),
 			(max-- - i) * sizeof(shaliastable));
 		i--;
 		n++;
 	}
-	if (re) regexp_free(re);
+	regexp_free(re);
 	if (!n) return(-1);
-# if	defined (FD) && !defined (_NOPTY)
+# ifdef	DEP_PTY
 	sendparent(TE_DELETEALIAS, ident);
 # endif
 
@@ -532,7 +463,7 @@ shaliastable *alias;
 	int i, n;
 
 	if (!alias) n = 0;
-	else for (n = 0; alias[n].ident; n++);
+	else for (n = 0; alias[n].ident; n++) /*EMPTY*/;
 	dupl = (shaliastable *)malloc2((n + 1) * sizeof(shaliastable));
 	for (i = 0; i < n; i++) {
 		dupl[i].ident = strdup2(alias[i].ident);
@@ -550,10 +481,10 @@ shaliastable *alias;
 
 	if (alias) {
 		for (i = 0; alias[i].ident; i++) {
-			free(alias[i].ident);
-			free(alias[i].comm);
+			free2(alias[i].ident);
+			free2(alias[i].comm);
 		}
-		free(alias);
+		free2(alias);
 	}
 }
 
@@ -582,7 +513,8 @@ int len, delim;
 	&& !(statementlist[i].type & STT_NEEDLIST)))
 		return(-1);
 
-	if ((!strchr(IFS_SET, delim) && !strchr(ALIASDELIMIT, delim)))
+	if (delim
+	&& (!strchr2(IFS_SET, delim) && !strchr2(ALIASDELIMIT, delim)))
 		return(-1);
 	i = searchalias(ident, len);
 	if (!(shellalias[i].ident) || (shellalias[i].flags & AL_USED))
@@ -596,14 +528,14 @@ int len, delim;
 #endif	/* NOALIAS */
 
 static int NEAR _evalexpression(s, ptr, resultp)
-char *s;
+CONST char *s;
 int ptr;
 long *resultp;
 {
 	long n;
 	int i, c, base;
 
-	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
+	while (s[ptr] && strchr2(IFS_SET, s[ptr])) ptr++;
 	if (!s[ptr]) return(-1);
 
 	if (s[ptr] == '+') {
@@ -628,7 +560,7 @@ long *resultp;
 	else if (s[ptr] == '(') {
 		if ((ptr = evalexpression(s, ptr + 1, resultp, 9)) < 0)
 			return(-1);
-		while (strchr(IFS_SET, s[ptr])) ptr++;
+		while (s[ptr] && strchr2(IFS_SET, s[ptr])) ptr++;
 		if (s[ptr++] != ')') return(-1);
 	}
 	else if (isdigit2(s[ptr])) {
@@ -660,13 +592,13 @@ long *resultp;
 	}
 	else return(-1);
 
-	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
+	while (s[ptr] && strchr2(IFS_SET, s[ptr])) ptr++;
 
 	return(ptr);
 }
 
 static int NEAR evalexpression(s, ptr, resultp, lvl)
-char *s;
+CONST char *s;
 int ptr;
 long *resultp;
 int lvl;
@@ -810,10 +742,14 @@ int lvl;
 }
 
 char *evalposixsubst(s, ptrp)
-char *s;
+CONST char *s;
 int *ptrp;
 {
-	char *cp;
+#ifndef	BASHBUG
+	syntaxtree *trp, *stree;
+	redirectlist red;
+#endif
+	char *cp, *new;
 	long n;
 	int i, len;
 
@@ -822,15 +758,12 @@ int *ptrp;
 
 	s += i;
 	len = *ptrp - i * 2;
-	s = strndup2(s, len);
+	s = new = strndup2(s, len);
 	if (i <= 1) {
 #ifndef	BASHBUG
 	/* bash cannot include 'case' statement within $() */
-		syntaxtree *trp, *stree;
-		redirectlist red;
-
 		red.fd = red.type = red.new = 0;
-		red.filename = nullstr;
+		red.filename = vnullstr;
 		i = len = 0;
 		stree = newstree(NULL);
 		trp = startvar(stree, &red, nullstr, &i, &len, 0);
@@ -838,7 +771,7 @@ int *ptrp;
 		trp = analyze(s, trp, -1);
 		if (trp && (trp -> cont & CN_SBST) != CN_CASE) trp = NULL;
 		freestree(stree);
-		free(stree);
+		free2(stree);
 		if (trp) cp = NULL;
 		else
 #endif	/* !BASHBUG */
@@ -846,36 +779,35 @@ int *ptrp;
 	}
 	else {
 		n = 0L;
-		if (!(cp = evalvararg(s, '\'', EA_BACKQ, 0))) *ptrp = -1;
+		if (!(cp = evalvararg(new, '\'', EA_BACKQ, 0))) *ptrp = -1;
 		else {
-			for (i = 0; cp[i] && strchr(IFS_SET, cp[i]); i++);
+			for (i = 0; cp[i]; i++)
+				if (!strchr2(IFS_SET, cp[i])) break;
 			if (cp[i]) {
 				i = evalexpression(cp, i, &n, 9);
 				if (i < 0 || cp[i]) *ptrp = -1;
 			}
-			free(cp);
+			free2(cp);
 		}
 
-		if (*ptrp < 0) cp = NULL;
-		else if (asprintf2(&cp, "%ld", n) < 0) error("malloc()");
+		cp = (*ptrp < 0) ? NULL : asprintf3("%ld", n);
 	}
-	free(s);
+	free2(new);
 
 	return(cp);
 }
 
 #if	!MSDOS
 static VOID NEAR addmailpath(path, msg, mtime)
-char *path, *msg;
+CONST char *path;
+char *msg;
 time_t mtime;
 {
 	int i;
 
 	for (i = 0; i < maxmailpath; i++)
 		if (!strpathcmp(path, mailpathlist[i].path)) break;
-	if (i < maxmailpath) {
-		if (mailpathlist[i].msg) free(mailpathlist[i].msg);
-	}
+	if (i < maxmailpath) free2(mailpathlist[i].msg);
 	else {
 		mailpathlist = (mailpath_t *)realloc2(mailpathlist,
 			(i + 1) * sizeof(mailpath_t));
@@ -887,12 +819,13 @@ time_t mtime;
 }
 
 VOID replacemailpath(s, multi)
-char *s;
+CONST char *s;
 int multi;
 {
 	mailpath_t *mailpath;
 	time_t mtime;
-	char *cp, *msg, path[MAXPATHLEN];
+	CONST char *cp;
+	char *msg, path[MAXPATHLEN];
 	int i, max, len;
 
 	mailpath = mailpathlist;
@@ -907,12 +840,12 @@ int multi;
 		addmailpath(s, NULL, mtime);
 	}
 	else while (*s) {
-		if ((cp = strchr(s, ':'))) len = (cp++) - s;
+		if ((cp = strchr2(s, ':'))) len = (cp++) - s;
 		else {
 			len = strlen(s);
-			cp = s + len;
+			cp = &(s[len]);
 		}
-		if ((msg = strchr(s, '%'))) {
+		if ((msg = strchr2(s, '%'))) {
 			if (msg > s + len) msg = NULL;
 			else {
 				i = s + len - (msg + 1);
@@ -929,10 +862,10 @@ int multi;
 	}
 
 	for (i = 0; i < max; i++) {
-		free(mailpath[i].path);
-		if (mailpath[i].msg) free(mailpath[i].msg);
+		free2(mailpath[i].path);
+		free2(mailpath[i].msg);
 	}
-	if (mailpath) free(mailpath);
+	free2(mailpath);
 }
 
 VOID checkmail(reset)
@@ -944,10 +877,10 @@ int reset;
 
 	if (reset) {
 		for (i = 0; i < maxmailpath; i++) {
-			free(mailpathlist[i].path);
-			if (mailpathlist[i].msg) free(mailpathlist[i].msg);
+			free2(mailpathlist[i].path);
+			free2(mailpathlist[i].msg);
 		}
-		if (mailpathlist) free(mailpathlist);
+		free2(mailpathlist);
 		mailpathlist = NULL;
 		maxmailpath = 0;
 		return;
@@ -979,16 +912,16 @@ syntaxtree *trp;
 		j = joblist[i].npipe;
 
 		if (joblist[i].stats[j] >= 0 && joblist[i].stats[j] < 128)
-			dispjob(i, stdout);
+			dispjob(i, Xstdout);
 		else {
 			if (isopbg(joblist[i].trp) || isopnown(joblist[i].trp))
 				joblist[i].trp -> type = OP_NONE;
-			dispjob(i, stdout);
-			free(joblist[i].pids);
-			free(joblist[i].stats);
+			dispjob(i, Xstdout);
+			free2(joblist[i].pids);
+			free2(joblist[i].stats);
 			if (joblist[i].trp) {
 				freestree(joblist[i].trp);
-				free(joblist[i].trp);
+				free2(joblist[i].trp);
 			}
 			joblist[i].pids = NULL;
 		}
@@ -1008,8 +941,9 @@ syntaxtree *trp;
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
 	checkjob(NULL);
 	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1],
-			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 0);
+		execerror((trp -> comm) -> argv,
+			(trp -> comm) -> argv[1],
+			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
 	n = joblist[i].npipe;
@@ -1018,8 +952,8 @@ syntaxtree *trp;
 		prepareexit(-1);
 		Xexit(RET_FATALERR);
 	}
-	fprintf2(stderr, "[%d] %id", i + 1, joblist[i].pids[n]);
-	fputnl(stderr);
+	fprintf2(Xstderr, "[%d] %id", i + 1, joblist[i].pids[n]);
+	fputnl(Xstderr);
 	if (joblist[i].tty) tioctl(ttyio, REQSETP, joblist[i].tty);
 # ifdef	USESGTTY
 	if (joblist[i].ttyflag) ioctl(ttyio, TIOCLSET, joblist[i].ttyflag);
@@ -1052,8 +986,9 @@ syntaxtree *trp;
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
 	checkjob(NULL);
 	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1],
-			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 0);
+		execerror((trp -> comm) -> argv,
+			(trp -> comm) -> argv[1],
+			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
 	n = joblist[i].npipe;
@@ -1063,7 +998,7 @@ syntaxtree *trp;
 	}
 	if (!isopbg(joblist[i].trp) || !isopnown(joblist[i].trp))
 		joblist[i].trp -> type = OP_BG;
-	if (interactive && !nottyout) dispjob(i, stderr);
+	if (interactive && !nottyout) dispjob(i, Xstderr);
 	prevjob = lastjob;
 	lastjob = i;
 
@@ -1082,12 +1017,12 @@ syntaxtree *trp;
 	argv = (trp -> comm) -> argv;
 	if ((trp -> comm) -> argc <= 1) {
 		alias = duplalias(shellalias);
-		for (i = 0; alias[i].ident; i++);
+		for (i = 0; alias[i].ident; i++) /*EMPTY*/;
 		if (i > 1) qsort(alias, i, sizeof(shaliastable), cmpalias);
 		for (i = 0; alias[i].ident; i++) {
-			fprintf2(stdout, "alias %k='%k'",
+			fprintf2(Xstdout, "alias %k='%k'",
 				alias[i].ident, alias[i].comm);
-			fputnl(stdout);
+			fputnl(Xstdout);
 		}
 		freealias(alias);
 		return(RET_SUCCESS);
@@ -1107,15 +1042,16 @@ syntaxtree *trp;
 			i = searchalias(argv[n], len);
 			if (!(shellalias[i].ident)) {
 				if (interactive) {
-					fputs("alias: ", stderr);
-					execerror(argv[n], ER_NOTFOUND, 0);
+					Xfputs("alias: ", Xstderr);
+					execerror(argv,
+						argv[n], ER_NOTFOUND, 2);
 				}
 				ret = RET_FAIL;
 				ERRBREAK;
 			}
-			fprintf2(stdout, "alias %k='%k'",
+			fprintf2(Xstdout, "alias %k='%k'",
 				shellalias[i].ident, shellalias[i].comm);
-			fputnl(stdout);
+			fputnl(Xstdout);
 		}
 	}
 
@@ -1133,8 +1069,8 @@ syntaxtree *trp;
 	for (n = 1; n < (trp -> comm) -> argc; n++) {
 		if (deletealias(argv[n]) < 0) {
 			if (interactive) {
-				fputs("alias: ", stderr);
-				execerror(argv[n], ER_NOTALIAS, 0);
+				Xfputs("alias: ", Xstderr);
+				execerror(argv, argv[n], ER_NOTALIAS, 2);
 			}
 			ret = RET_FAIL;
 			ERRBREAK;
@@ -1153,9 +1089,9 @@ syntaxtree *trp;
 	int i, n, sig;
 
 	if ((trp -> comm) -> argc <= 1) {
-		fputs("usage: kill [ -sig ] pid ...\n", stderr);
-		fputs("for a list of signals: kill -l", stderr);
-		fputnl(stderr);
+		Xfputs("usage: kill [ -sig ] pid ...\n", Xstderr);
+		Xfputs("for a list of signals: kill -l", Xstderr);
+		fputnl(Xstderr);
 		return(RET_SYNTAXERR);
 	}
 #ifdef	SIGTERM
@@ -1171,12 +1107,12 @@ syntaxtree *trp;
 				for (i = 0; signallist[i].sig >= 0; i++)
 					if (sig == signallist[i].sig) break;
 				if (signallist[i].sig < 0) continue;
-				fprintf2(stdout, "%s%c",
+				fprintf2(Xstdout, "%s%c",
 					signallist[i].ident,
 					(++n % 16) ? ' ' : '\n');
 			}
-			if (n % 16) fputnl(stdout);
-			fflush(stdout);
+			if (n % 16) fputnl(Xstdout);
+			Xfflush(Xstdout);
 			return(RET_SUCCESS);
 		}
 		if ((sig = isnumeric(&(s[1]))) < 0) {
@@ -1184,13 +1120,14 @@ syntaxtree *trp;
 				if (!strcmp(&(s[1]), signallist[i].ident))
 					break;
 			if (signallist[i].sig < 0) {
-				execerror(&(s[1]), ER_UNKNOWNSIG, 0);
+				execerror((trp -> comm) -> argv,
+					&(s[1]), ER_UNKNOWNSIG, 1);
 				return(RET_FAIL);
 			}
 			sig = signallist[i].sig;
 		}
 		if (sig < 0 || sig >= NSIG) {
-			execerror(s, ER_NUMOUTRANGE, 0);
+			execerror((trp -> comm) -> argv, s, ER_NUMOUTRANGE, 1);
 			return(RET_FAIL);
 		}
 		i = 2;
@@ -1204,8 +1141,10 @@ syntaxtree *trp;
 #ifndef	NOJOB
 		if (*s == '%') {
 			if ((n = getjob(s)) < 0) {
-				execerror(s, (n < -1)
-					? ER_TERMINATED : ER_NOSUCHJOB, 0);
+				execerror((trp -> comm) -> argv,
+					s,
+					(n < -1)
+					? ER_TERMINATED : ER_NOSUCHJOB, 2);
 				return(RET_FAIL);
 			}
 			n = Xkillpg(joblist[n].pids[0], sig);
@@ -1213,16 +1152,13 @@ syntaxtree *trp;
 		else
 #endif	/* !NOJOB */
 		if ((pid = isnumeric(s)) < (p_id_t)0) {
-			fputs("usage: kill [ -sig ] pid ...\n", stderr);
-			fputs("for a list of signals: kill -l", stderr);
-			fputnl(stderr);
+			Xfputs("usage: kill [ -sig ] pid ...\n", Xstderr);
+			Xfputs("for a list of signals: kill -l", Xstderr);
+			fputnl(Xstderr);
 			return(RET_SYNTAXERR);
 		}
 #if	MSDOS
-		else if (pid && pid != mypid) {
-			errno = EPERM;
-			n = -1;
-		}
+		else if (pid && pid != mypid) n = seterrno(EPERM);
 #endif
 		else n = kill(pid, sig);
 
@@ -1237,7 +1173,7 @@ syntaxtree *trp;
 
 static int NEAR testsub1(c, s, ptrp)
 int c;
-char *s;
+CONST char *s;
 int *ptrp;
 {
 	struct stat st;
@@ -1353,10 +1289,10 @@ int *ptrp;
 
 static int NEAR testsub2(argc, argv, ptrp)
 int argc;
-char *argv[];
+char *CONST *argv;
 int *ptrp;
 {
-	char *s, *a1, *a2;
+	CONST char *s, *a1, *a2;
 	int ret, v1, v2;
 
 	if (*ptrp >= argc) return(-1);
@@ -1433,7 +1369,7 @@ int *ptrp;
 
 static int NEAR testsub3(argc, argv, ptrp, lvl)
 int argc;
-char *argv[];
+char *CONST *argv;
 int *ptrp, lvl;
 {
 	int ret1, ret2;
@@ -1466,7 +1402,7 @@ int *ptrp, lvl;
 int posixtest(trp)
 syntaxtree *trp;
 {
-	char **argv;
+	char *CONST *argv;
 	int ret, ptr, argc;
 
 	argc = (trp -> comm) -> argc;
@@ -1475,8 +1411,8 @@ syntaxtree *trp;
 	if (argc <= 0) return(RET_FAIL);
 	if (argv[0][0] == '[' && !argv[0][1]
 	&& (--argc <= 0 || argv[argc][0] != ']' || argv[argc][1])) {
-		fputs("] missing", stderr);
-		fputnl(stderr);
+		Xfputs("] missing", Xstderr);
+		fputnl(Xstderr);
 		return(RET_NOTICE);
 	}
 	if (argc <= 1) return(RET_FAIL);
@@ -1486,17 +1422,17 @@ syntaxtree *trp;
 	if (ret < 0) {
 		switch (ret) {
 			case -1:
-				fputs("argument expected", stderr);
+				Xfputs("argument expected", Xstderr);
 				break;
 			case -2:
-				fprintf2(stderr, "unknown operator %k",
+				fprintf2(Xstderr, "unknown operator %k",
 					argv[ptr]);
 				break;
 			default:
-				fputs("syntax error", stderr);
+				Xfputs("syntax error", Xstderr);
 				break;
 		}
-		fputnl(stderr);
+		fputnl(Xstderr);
 		return(RET_NOTICE);
 	}
 
@@ -1526,7 +1462,7 @@ syntaxtree *trp;
 # endif
 	if (flag == 'V') {
 		for (i = n; i < argc; i++) {
-			if (typeone(argv[i], stdout) >= 0) {
+			if (typeone(argv[i], Xstdout) >= 0) {
 # ifdef	BASHSTYLE
 				ret = RET_SUCCESS;
 # endif
@@ -1552,14 +1488,14 @@ syntaxtree *trp;
 # endif
 				if ((type & CM_NOTFOUND)
 				|| Xaccess(cp, X_OK) < 0) {
-					execerror(cp, ER_COMNOFOUND, 0);
+					execerror(argv, cp, ER_COMNOFOUND, 1);
 					cp = NULL;
 				}
 			}
 
 			if (cp) {
-				kanjifputs(cp, stdout);
-				fputnl(stdout);
+				kanjifputs(cp, Xstdout);
+				fputnl(Xstdout);
 # ifdef	BASHSTYLE
 				ret = RET_SUCCESS;
 # endif
@@ -1574,10 +1510,10 @@ syntaxtree *trp;
 	type = checktype(argv[n], &id, 0, 0);
 #endif
 	if (verboseexec) {
-		fprintf2(stderr, "+ %k", argv[n]);
+		fprintf2(Xstderr, "+ %k", argv[n]);
 		for (i = n + 1; i < argc; i++)
-			fprintf2(stderr, " %k", argv[i]);
-		fputnl(stderr);
+			fprintf2(Xstderr, " %k", argv[i]);
+		fputnl(Xstderr);
 	}
 
 	(trp -> comm) -> argc -= n;
@@ -1589,15 +1525,15 @@ syntaxtree *trp;
 		ret = exec_simplecom(trp, type, id, 0);
 #endif
 	else {
-		path = strdup2(getconstvar("PATH"));
-		setenv2("PATH", DEFPATH, 1);
+		path = strdup2(getconstvar(ENVPATH));
+		setenv2(ENVPATH, DEFPATH, 1);
 #if	MSDOS
 		ret = exec_simplecom(trp, type, id);
 #else
 		ret = exec_simplecom(trp, type, id, 0);
 #endif
-		setenv2("PATH", path, 1);
-		if (path) free(path);
+		setenv2(ENVPATH, path, 1);
+		free2(path);
 	}
 	(trp -> comm) -> argc = argc;
 	(trp -> comm) -> argv = argv;
@@ -1616,11 +1552,11 @@ syntaxtree *trp;
 	argv = (trp -> comm) -> argv;
 
 	if (argc <= 2) {
-		execerror(NULL, ER_MISSARG, 0);
+		execerror(argv, NULL, ER_MISSARG, 1);
 		return(RET_FAIL);
 	}
 	if (identcheck(argv[2], '\0') <= 0) {
-		execerror(argv[2], ER_NOTIDENT, 0);
+		execerror(argv, argv[2], ER_NOTIDENT, 0);
 		return(RET_FAIL);
 	}
 
@@ -1673,7 +1609,8 @@ syntaxtree *trp;
 			buf[1] = '\0';
 			n = '?';
 			if (quiet) posixoptarg = buf;
-			else execerror(buf, ER_BADOPTIONS, 2);
+			else execerror((trp -> comm) -> argv,
+				buf, ER_BADOPTIONS, 2);
 		}
 		else if (*(cp + 1) == ':') {
 			if (argv[posixoptind][ptr])
@@ -1689,15 +1626,16 @@ syntaxtree *trp;
 				}
 				else {
 					n = '?';
-					execerror(buf, ER_MISSARG, 2);
+					execerror((trp -> comm) -> argv,
+						buf, ER_MISSARG, 2);
 				}
 			}
 			ptr = 0;
 		}
 	}
 
-	if (posixoptarg) setenv2("OPTARG", posixoptarg, 0);
-	else if (ret == RET_SUCCESS) unset("OPTARG", sizeof("OPTARG") - 1);
+	if (posixoptarg) setenv2(ENVOPTARG, posixoptarg, 0);
+	else if (ret == RET_SUCCESS) unset(ENVOPTARG, strsize(ENVOPTARG));
 
 	buf[0] = n;
 	buf[1] = '\0';
@@ -1705,10 +1643,10 @@ syntaxtree *trp;
 
 	n = posixoptind;
 	snprintf2(buf, sizeof(buf), "%d", n);
-	setenv2("OPTIND", buf, 0);
+	setenv2(ENVOPTIND, buf, 0);
 	posixoptind = n;
 
 	return(ret);
 }
 #endif	/* NOPOSIXUTIL */
-#endif	/* !MINIMUMSHELL && (!FD || (FD >= 2 && !_NOORIGSHELL)) */
+#endif	/* DEP_ORIGSHELL && !MINIMUMSHELL */
