@@ -4,33 +4,45 @@
  *	shell command module
  */
 
-#include <fcntl.h>
 #include "fd.h"
+#include "realpath.h"
+#include "parse.h"
+#include "kconv.h"
 #include "func.h"
 #include "kanji.h"
 
-#if	MSDOS && !defined (_NOUSELFN)
-extern char *shortname __P_((char *, char *));
-#endif
-
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 #include "system.h"
 #endif
-
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 #include "termemu.h"
 #endif
 
-#ifndef	O_TEXT
-#define	O_TEXT		0
-#endif
-
 typedef struct _localetable {
-	char *env;
-	char *val;
+	CONST char *env;
+	CONST char *val;
 	char *org;
 } localetable;
 
+#if	0
+#define	MACROMETA		((char)-1)
+#endif
+#if	defined (USESYSCONF) && defined (_SC_ARG_MAX)
+#define	MAXARGNUM		sysconf(_SC_ARG_MAX)
+#else
+# ifdef	ARG_MAX
+# define	MAXARGNUM	ARG_MAX
+# else
+#  ifdef	NCARGS
+#  define	MAXARGNUM	NCARGS
+#  endif
+# endif
+#endif
+
+#ifdef	DEP_DOSLFN
+extern char *shortname __P_((CONST char *, char *));
+#endif
+
 extern int mark;
 extern off_t marksize;
 extern char fullpath[];
@@ -41,58 +53,44 @@ extern int lcmdline;
 extern int hideclock;
 extern int internal_status;
 
-#if	0
-#define	MACROMETA	((char)-1)
-#endif
-#if	defined (USESYSCONF) && defined (_SC_ARG_MAX)
-#define	MAXARGNUM	sysconf(_SC_ARG_MAX)
-#else
-# ifdef	ARG_MAX
-# define	MAXARGNUM	ARG_MAX
-# else
-#  ifdef	NCARGS
-#  define	MAXARGNUM	NCARGS
-#  endif
-# endif
-#endif
-
-static int NEAR checksc __P_((char *, int, char *));
-#ifndef	_NOKANJICONV
+static int NEAR checksc __P_((CONST char *, int, CONST char *));
+#ifdef	DEP_KCONV
 static int NEAR extconv __P_((char **, int, int, ALLOC_T *, int));
 #endif
-static int NEAR isneedargs __P_((char *, int, int *));
-static int NEAR setarg __P_((char **, int, ALLOC_T *, char *, char *, int));
+static int NEAR isneedargs __P_((CONST char *, int, int *));
+static int NEAR setarg __P_((char **, int, ALLOC_T *,
+		CONST char *, CONST char *, int));
 static int NEAR flag2str __P_((char *, int, int));
-static int NEAR skipquote __P_((char *, int *));
+static int NEAR skipquote __P_((CONST char *, int *));
 #if	!defined (MACROMETA) || !defined (_NOEXTRAMACRO)
-static char *NEAR _restorearg __P_((char *));
+static char *NEAR _restorearg __P_((CONST char *));
 static char *NEAR _demacroarg __P_((char *));
 #endif
 #ifdef	_NOEXTRAMACRO
 static int NEAR setargs __P_((char **, int, ALLOC_T *,
-	int, int, int, macrostat *, int));
-static char *NEAR insertarg __P_((char *, char *, int));
+		int, int, int, macrostat *, int));
+static char *NEAR insertarg __P_((CONST char *, CONST char *, int));
 #else
-static int NEAR _replaceargs __P_((int *, char ***, char **, int));
+static int NEAR _replaceargs __P_((int *, char ***, char *CONST *, int));
 #endif
-static char *NEAR addoption __P_((char *, char *, macrostat *));
-#ifdef	_NOORIGSHELL
-static int NEAR system3 __P_((char *, int));
-static char *NEAR evalargs __P_((char *, int, char *[]));
-static char *NEAR evalalias __P_((char *));
+static char *NEAR addoption __P_((char *, CONST char *, macrostat *));
+#ifndef	DEP_ORIGSHELL
+static int NEAR system3 __P_((CONST char *, int));
+static char *NEAR evalargs __P_((CONST char *, int, char *CONST *));
+static char *NEAR evalalias __P_((CONST char *));
 #else
 #define	system3			system2
 #endif
 
-#ifdef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
+CONST char *promptstr2 = NULL;
+#else
 aliastable aliaslist[MAXALIASTABLE];
 int maxalias = 0;
 userfunctable userfunclist[MAXFUNCTABLE];
 int maxuserfunc = 0;
-#else
-char *promptstr2 = NULL;
 #endif
-char *promptstr = NULL;
+CONST char *promptstr = NULL;
 char **history[2] = {NULL, NULL};
 char *histfile[2] = {NULL, NULL};
 short histsize[2] = {0, 0};
@@ -111,13 +109,13 @@ static localetable localelist[] = {
 	{"LC_ALL", NULL, NULL},
 	{"LANG", "C", NULL},
 };
-#define	LOCALELISTSIZ	arraysize(localelist)
+#define	LOCALELISTSIZ		arraysize(localelist)
 
 
 static int NEAR checksc(buf, ptr, arg)
-char *buf;
+CONST char *buf;
 int ptr;
-char *arg;
+CONST char *arg;
 {
 	if (!arg || !*arg) /*EMPTY*/;
 	else if (*arg == _SC_ || isdotdir(arg)) /*EMPTY*/;
@@ -129,30 +127,31 @@ char *arg;
 	if (buf) {
 		if (ptr < 2) return(ptr);
 		if (buf[ptr - 1] != _SC_ || buf[ptr - 2] != '.') return(ptr);
-		if (ptr > 2 && !strchr(IFS_SET, buf[ptr - 3])) return(ptr);
+		if (ptr > 2 && !strchr2(IFS_SET, buf[ptr - 3])) return(ptr);
 	}
 
 	return(ptr - 2);
 }
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 static int NEAR extconv(bufp, ptr, eol, sizep, code)
 char **bufp;
 int ptr, eol;
 ALLOC_T *sizep;
 int code;
 {
+# ifdef	DEP_FILECONV
+	char rpath[MAXPATHLEN];
+# endif
 	char *cp;
 	int len;
 
-# ifndef	_NOKANJIFCONV
+# ifdef	DEP_FILECONV
 	if (code < 0) {
-		char rpath[MAXPATHLEN];
-
 		cp = _evalpath(&((*bufp)[ptr]), &((*bufp)[eol]),
 			EA_NOEVALQ | EA_NOUNIQDELIM);
-		realpath2(cp, rpath, 1);
-		free(cp);
+		realpath2(cp, rpath, RLP_READLINK);
+		free2(cp);
 		code = getkcode(rpath);
 	}
 # endif
@@ -163,14 +162,14 @@ int code;
 
 	*bufp = c_realloc(*bufp, ptr + len + 1, sizep);
 	memcpy(&((*bufp)[ptr]), cp, len);
-	free(cp);
+	free2(cp);
 
 	return(ptr + len);
 }
-#endif	/* !_NOKANJICONV */
+#endif	/* DEP_KCONV */
 
 static int NEAR isneedargs(s, n, flagsp)
-char *s;
+CONST char *s;
 int n, *flagsp;
 {
 	int i;
@@ -215,15 +214,15 @@ static int NEAR setarg(bufp, ptr, sizep, dir, arg, flags)
 char **bufp;
 int ptr;
 ALLOC_T *sizep;
-char *dir, *arg;
+CONST char *dir, *arg;
 int flags;
 {
-	char *cp, path[MAXPATHLEN], conv[MAXPATHLEN];
+	char *cp, *new, path[MAXPATHLEN], conv[MAXPATHLEN];
 	int len, optr;
 
 	if (!arg || !*arg) return(checksc(*bufp, ptr, NULL) - ptr);
 	if (!dir) {
-#if	MSDOS && !defined (_NOUSELFN)
+#ifdef	DEP_DOSLFN
 		if ((flags & F_TOSFN) && shortname(arg, path) == path)
 			arg = path;
 		else
@@ -234,12 +233,12 @@ int flags;
 		strcatdelim2(path, dir, arg);
 		arg = path;
 #if	defined (BSPATHDELIM) && !defined (_NOARCHIVE)
-		if (flags & F_ISARCH) for (len = 0; arg[len]; len++) {
-			if (iskanji1(arg, len)) len++;
-			if (arg[len] == _SC_) arg[len] = '/';
+		if (flags & F_ISARCH) for (len = 0; path[len]; len++) {
+			if (iskanji1(path, len)) len++;
+			if (path[len] == _SC_) path[len] = '/';
 		}
 #endif
-#ifndef	_NOKANJIFCONV
+#ifdef	DEP_FILECONV
 		arg = kanjiconv2(conv, arg,
 			MAXPATHLEN - 1, DEFCODE, fnamekcode, L_FNAME);
 #endif
@@ -247,15 +246,15 @@ int flags;
 
 	optr = ptr;
 	ptr = checksc(*bufp, ptr, arg);
-	arg = killmeta(arg);
+	arg = new = killmeta(arg);
 
-	if ((flags & F_NOEXT) && (cp = strrchr(arg, '.')) && cp != arg)
+	if ((flags & F_NOEXT) && (cp = strrchr2(arg, '.')) && cp != arg)
 		len = cp - arg;
 	else len = strlen(arg);
 
 	*bufp = c_realloc(*bufp, ptr + len + 1, sizep);
-	strncpy(&((*bufp)[ptr]), arg, len);
-	free(arg);
+	strncpy2(&((*bufp)[ptr]), arg, len);
+	free2(new);
 
 	return(len + ptr - optr);
 }
@@ -280,7 +279,7 @@ int ptr, flags;
 }
 
 static int NEAR skipquote(s, ptrp)
-char *s;
+CONST char *s;
 int *ptrp;
 {
 	static int q = '\0';
@@ -299,7 +298,7 @@ int *ptrp;
 
 #if	!defined (MACROMETA) || !defined (_NOEXTRAMACRO)
 static char *NEAR _restorearg(s)
-char *s;
+CONST char *s;
 {
 	char *buf;
 	int i, j, m, flags;
@@ -329,8 +328,8 @@ char *s;
 # endif	/* !MACROMETA */
 	}
 	if (!flags) {
-		free(buf);
-		return(s);
+		free2(buf);
+		return((char *)s);
 	}
 	buf[j] = '\0';
 
@@ -342,7 +341,7 @@ char *s;
 {
 	char *cp;
 
-	if ((cp = _restorearg(s)) != s && s) free(s);
+	if ((cp = _restorearg(s)) != s) free2(s);
 
 	return(cp);
 }
@@ -359,7 +358,7 @@ macrostat *stp;
 int flags;
 {
 # ifdef	MAXCOMMSTR
-	int optr;
+	int m, f, optr;
 # endif
 	char *cp, *s, *dir, *tmp;
 	int i, n, len, flen, rlen;
@@ -375,7 +374,7 @@ int flags;
 	tmp[len] = '\0';
 	cp = &(tmp[blen + mlen]);
 	for (s = cp; *s; s++) {
-		if (strchr(CMDLINE_DELIM, *s)) break;
+		if (strchr2(CMDLINE_DELIM, *s)) break;
 		if (iskanji1(s, 0)) s++;
 	}
 	flen = s - cp;
@@ -388,12 +387,12 @@ int flags;
 		ptr += len;
 # ifdef	MAXCOMMSTR
 		if (ptr + flen > MAXCOMMSTR) {
-			free(tmp);
+			free2(tmp);
 			return(-1);
 		}
 # endif
 		*bufp = c_realloc(*bufp, ptr + flen + 1, sizep);
-		strncpy(&((*bufp)[ptr]), cp, flen);
+		strncpy2(&((*bufp)[ptr]), cp, flen);
 		ptr += flen;
 	}
 	else for (i = n = 0; i < maxfile; i++) {
@@ -407,7 +406,7 @@ int flags;
 # endif
 			*bufp = c_realloc(*bufp, ptr + blen + 1 + 1, sizep);
 			(*bufp)[ptr++] = ' ';
-			strncpy(&((*bufp)[ptr]), tmp, blen);
+			strncpy2(&((*bufp)[ptr]), tmp, blen);
 			ptr += blen;
 		}
 		len = setarg(bufp, ptr, sizep, dir, filelist[i].name, flags);
@@ -415,7 +414,7 @@ int flags;
 # ifdef	MAXCOMMSTR
 		if (ptr + flen > MAXCOMMSTR) {
 			if (!n) {
-				free(tmp);
+				free2(tmp);
 				return(-1);
 			}
 			ptr = optr;
@@ -423,7 +422,7 @@ int flags;
 		}
 # endif
 		*bufp = c_realloc(*bufp, ptr + flen + 1, sizep);
-		strncpy(&((*bufp)[ptr]), cp, flen);
+		strncpy2(&((*bufp)[ptr]), cp, flen);
 		ptr += flen;
 		filelist[i].tmpflags &= ~F_ISARG;
 		n_args--;
@@ -432,8 +431,6 @@ int flags;
 
 # ifdef	MAXCOMMSTR
 	if (ptr + rlen > MAXCOMMSTR) {
-		int m, f;
-
 		cp = &(s[MAXCOMMSTR - ptr]);
 		len = rlen - (MAXCOMMSTR - ptr);
 		for (i = 0; i < len; i++) {
@@ -451,23 +448,25 @@ int flags;
 	*bufp = c_realloc(*bufp, ptr + rlen + 1, sizep);
 	memcpy(&((*bufp)[ptr]), s, rlen);
 	ptr += rlen;
-	free(tmp);
+	free2(tmp);
 
 	return(ptr - eol);
 }
 
 static char *NEAR insertarg(format, arg, needmark)
-char *format, *arg;
+CONST char *format, *arg;
 int needmark;
 {
-# if	MSDOS && !defined (_NOUSELFN)
-	char *org, alias[MAXPATHLEN];
+# ifdef	DEP_DOSLFN
+	CONST char *org;
+	char alias[MAXPATHLEN];
 # endif
-	char *cp, *src, *ins, *buf, conv[MAXPATHLEN];
+	CONST char *src, *ins;
+	char *cp, *new, *buf, conv[MAXPATHLEN];
 	ALLOC_T size;
 	int i, j, m, len, flags;
 
-# if	MSDOS && !defined (_NOUSELFN)
+# ifdef	DEP_DOSLFN
 	org = arg;
 	alias[0] = '\0';
 # endif
@@ -493,31 +492,31 @@ int needmark;
 	{
 		buf = c_realloc(buf, j + 1, &size);
 		memcpy(buf, format, j);
-		src = format + j + m;
+		src = &(format[j + m]);
 		for (i = 0; i < needmark; i++) {
 			ins = arg;
-# if	MSDOS && !defined (_NOUSELFN)
+# ifdef	DEP_DOSLFN
 			if (flags & F_TOSFN) {
 				if (!alias[0]) {
 					if (shortname(org, alias) == alias)
 						org = alias;
-					else strcpy(alias, org);
+					else strcpy2(alias, org);
 				}
 				ins = alias;
 			}
 # endif
 			j = checksc(buf, j, ins);
-			ins = killmeta(ins);
+			ins = new = killmeta(ins);
 
-			if ((flags & F_NOEXT) && (cp = strrchr(ins, '.')))
+			if ((flags & F_NOEXT) && (cp = strrchr2(ins, '.')))
 				len = cp - ins;
 			else len = strlen(ins);
 # ifdef	MAXCOMMSTR
 			if (j + len > MAXCOMMSTR) break;
 # endif
 			buf = c_realloc(buf, j + len + 1, &size);
-			strncpy(&(buf[j]), ins, len);
-			free(ins);
+			strncpy2(&(buf[j]), ins, len);
+			free2(new);
 			j += len;
 			for (len = 0; src[len]; len++) {
 # ifndef	MACROMETA
@@ -539,7 +538,7 @@ int needmark;
 	}
 	buf[j] = '\0';
 	if (i < needmark) {
-		free(buf);
+		free2(buf);
 		return(NULL);
 	}
 
@@ -548,14 +547,17 @@ int needmark;
 #endif	/* _NOEXTRAMACRO */
 
 char *evalcommand(command, arg, stp)
-char *command, *arg;
+CONST char *command, *arg;
 macrostat *stp;
 {
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	int code, cnvcode, defcode, cnvptr;
+#endif
+#ifdef	_NOEXTRAMACRO
+	int m, f;
 #endif
 	macrostat st;
-	char *cp, *line;
+	char *cp, *line, *new, conv[MAXPATHLEN];
 	ALLOC_T size;
 	int i, j, c, pc, len, setflag, flags;
 
@@ -566,14 +568,14 @@ macrostat *stp;
 	}
 	stp -> addopt = -1;
 	stp -> needmark = stp -> needburst = 0;
-#ifndef	_NOKANJICONV
-# ifdef	_NOKANJIFCONV
-	cnvcode = defcode = NOCNV;
-# else
+#ifdef	DEP_KCONV
+# ifdef	DEP_FILECONV
 	cnvcode = defcode = (flags & F_NOKANJICONV) ? NOCNV : defaultkcode;
+# else
+	cnvcode = defcode = NOCNV;
 # endif
 	cnvptr = 0;
-#endif	/* !_NOKANJICONV */
+#endif	/* DEP_KCONV */
 
 	line = c_realloc(NULL, 0, &size);
 	for (i = j = 0; command[i]; i++) {
@@ -590,7 +592,7 @@ macrostat *stp;
 			continue;
 		}
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 		if (cnvcode != NOCNV)
 			j = extconv(&line, cnvptr, j, &size, cnvcode);
 #endif
@@ -668,7 +670,7 @@ macrostat *stp;
 			case 'K':
 				if (!(flags & F_ISARCH)) flags ^= F_NOCONFIRM;
 				break;
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 			case 'J':
 				c = command[i + 1];
 				if (c == 'S') code = SJIS;
@@ -684,7 +686,7 @@ macrostat *stp;
 				else if (c == 'U') code = UTF8;
 				else if (c == 'M') code = M_UTF8;
 				else if (c == 'I') code = I_UTF8;
-#  ifndef	_NOKANJIFCONV
+#  ifdef	DEP_FILECONV
 				else if (c == 'A') code = -1;
 #  endif
 # endif	/* FD >= 2 */
@@ -696,7 +698,7 @@ macrostat *stp;
 				i++;
 				cnvcode = (cnvcode == code) ? defcode : code;
 				break;
-#endif	/* !_NOKANJICONV */
+#endif	/* DEP_KCONV */
 			case '%':
 #ifndef	MACROMETA
 				line[j++] = '%';
@@ -710,7 +712,7 @@ macrostat *stp;
 		}
 		if (!setflag) flags &= ~(F_NOEXT | F_TOSFN);
 		j += len;
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 		cnvptr = j;
 #endif
 	}
@@ -721,23 +723,21 @@ macrostat *stp;
 			errno = E2BIG;
 			perror2(command);
 		}
-		free(line);
+		free2(line);
 		return(NULL);
 	}
 #endif
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	if (cnvcode != NOCNV) j = extconv(&line, cnvptr, j, &size, cnvcode);
 #endif
 
 #ifdef	_NOEXTRAMACRO
 	if (stp -> needburst) for (i = c = 0; i < j; i++) {
-		int m, f;
-
 # ifndef	MACROMETA
 		if (skipquote(line, &i)) continue;
 # endif
-		if (strchr(CMDLINE_DELIM, line[i])) c = -1;
+		if (strchr2(CMDLINE_DELIM, line[i])) c = -1;
 		else if (c < 0) c = i;
 		if (!(m = isneedargs(line, i, &f))) continue;
 		else if (!(f & F_BURST)) {
@@ -773,19 +773,17 @@ macrostat *stp;
 #endif	/* _NOEXTRAMACRO */
 
 	if (!(flags & F_ARGSET) && arg && *arg) {
-		char conv[MAXPATHLEN];
-
 		arg = convput(conv, arg, 1, 1, NULL, NULL);
 		if (checksc(NULL, 0, arg) < 0) cp = NULL;
 		else {
 			cp = malloc2(strlen(arg) + 2 + 1);
 			cp[0] = '.';
 			cp[1] = _SC_;
-			strcpy(&(cp[2]), arg);
+			strcpy2(&(cp[2]), arg);
 			arg = cp;
 		}
-		arg = killmeta(arg);
-		if (cp) free(cp);
+		arg = new = killmeta(arg);
+		free2(cp);
 
 		len = strlen(arg);
 #if	defined (_NOEXTRAMACRO) && defined (MAXCOMMSTR)
@@ -795,10 +793,10 @@ macrostat *stp;
 		{
 			line = c_realloc(line, j + len + 1, &size);
 			line[j++] = ' ';
-			strncpy(&(line[j]), arg, len);
+			strncpy2(&(line[j]), arg, len);
 			j += len;
 		}
-		free(arg);
+		free2(new);
 	}
 	if (!(flags & F_IGNORELIST)
 	&& !(stp -> needburst) && !(stp -> needmark)) {
@@ -809,7 +807,7 @@ macrostat *stp;
 	stp -> flags = flags;
 
 	cp = strndup2(line, j);
-	free(line);
+	free2(line);
 
 	return(cp);
 }
@@ -818,7 +816,7 @@ macrostat *stp;
 /*ARGSUSED*/
 int replaceargs(argcp, argvp, env, iscomm)
 int *argcp;
-char ***argvp, **env;
+char ***argvp, *CONST *env;
 int iscomm;
 {
 	return(0);
@@ -835,7 +833,7 @@ char **argp;
 
 static int NEAR _replaceargs(argcp, argvp, env, iscomm)
 int *argcp;
-char ***argvp, **env;
+char ***argvp, *CONST *env;
 int iscomm;
 {
 	static int lastptr = 0;
@@ -888,19 +886,13 @@ int iscomm;
 	if (!i) return(ret);
 
 # ifdef	MAXCOMMSTR
-	if (iscomm > 0 && maxarg > MAXCOMMSTR) {
-		errno = E2BIG;
-		return(-1);
-	}
+	if (iscomm > 0 && maxarg > MAXCOMMSTR) return(seterrno(E2BIG));
 # else	/* !MAXCOMMSTR */
 #  ifdef	MAXARGNUM
 	maxarg = MAXARGNUM - 2048;
 	if (maxarg > 20 * 1024) maxarg = 20 * 1024;
 	maxarg -= countvar(env);
-	if (iscomm > 0 && argc > maxarg) {
-		errno = E2BIG;
-		return(-1);
-	}
+	if (iscomm > 0 && argc > maxarg) return(seterrno(E2BIG));
 #  endif
 # endif	/* !MAXCOMMSTR */
 
@@ -949,10 +941,9 @@ int iscomm;
 # ifdef	MAXCOMMSTR
 				maxarg += len;
 				if (iscomm > 0 && maxarg > MAXCOMMSTR) {
-					free(buf);
+					free2(buf);
 					freevar(argv2);
-					errno = E2BIG;
-					return(-1);
+					return(seterrno(E2BIG));
 				}
 # endif
 				j += len;
@@ -1030,7 +1021,7 @@ int iscomm;
 	*argvp = argv = argv2;
 
 	if (iscomm < 0) {
-		free(buf);
+		free2(buf);
 		return(ret);
 	}
 
@@ -1073,28 +1064,27 @@ int iscomm;
 # ifdef	MAXCOMMSTR
 				maxarg += len;
 				if (iscomm > 0 && maxarg > MAXCOMMSTR) {
-					free(buf);
+					free2(buf);
 					freevar(argv);
-					errno = E2BIG;
-					return(-1);
+					return(seterrno(E2BIG));
 				}
 # endif
 				j += len;
 			}
 		}
 		buf[j++] = '\0';
-		free(argv[n]);
+		free2(argv[n]);
 		argv[n] = strdup2(buf);
 	}
 	if (hit) lastptr = next;
-	free(buf);
+	free2(buf);
 
 	return(ret);
 }
 
 int replaceargs(argcp, argvp, env, iscomm)
 int *argcp;
-char ***argvp, **env;
+char ***argvp, *CONST *env;
 int iscomm;
 {
 # ifdef	MACROMETA
@@ -1122,19 +1112,19 @@ char **argp;
 	argc = 1;
 	n = replaceargs(&argc, &argv, NULL, 0);
 	*argp = argv[0];
-	free(argv);
+	free2(argv);
 
 	return(n);
 }
 #endif	/* !_NOEXTRAMACRO */
 
 char *restorearg(s)
-char *s;
+CONST char *s;
 {
 #if	defined (MACROMETA) && !defined (_NOEXTRAMACRO)
 	return(_restorearg(s));
 #else
-	return(s);
+	return((char *)s);
 #endif
 }
 
@@ -1148,36 +1138,37 @@ char **argp;
 }
 
 char *inputshellstr(prompt, ptr, def)
-char *prompt;
+CONST char *prompt;
 int ptr;
-char *def;
+CONST char *def;
 {
-	char *cp, *tmp, *duppromptstr;
+	CONST char *duppromptstr;
+	char *cp, *tmp;
 	int wastty;
 
 	duppromptstr = promptstr;
 	if (prompt) promptstr = prompt;
-	cp = inputstr(NULL, 0, ptr, def, HST_COM);
+	cp = inputstr(NULL, 0, ptr, def, HST_COMM);
 	promptstr = duppromptstr;
-	if (!cp) return((char *)-1);
+	if (!cp) return(vnullstr);
 
 	if ((wastty = isttyiomode)) Xstdiomode();
 	tmp = evalhistory(cp);
 	if (wastty) Xttyiomode(wastty - 1);
 	if (!tmp) {
-		free(cp);
+		free2(cp);
 		return(NULL);
 	}
-	entryhist(0, tmp, 0);
+	entryhist(tmp, HST_COMM);
 
 	return(tmp);
 }
 
 char *inputshellloop(ptr, def)
 int ptr;
-char *def;
+CONST char *def;
 {
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	syntaxtree *trp, *stree;
 	char *buf;
 	int l, len;
@@ -1185,7 +1176,7 @@ char *def;
 	char *cp;
 
 	cp = inputshellstr(NULL, ptr, def);
-	if (cp == (char *)-1) return(NULL);
+	if (cp == vnullstr) return(NULL);
 	else if (!cp) {
 		hideclock = 1;
 		warning(0, HITKY_K);
@@ -1193,42 +1184,43 @@ char *def;
 		return(NULL);
 	}
 
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	l = -1;
 	len = strlen(cp);
 	buf = malloc2(len + 1);
-	strcpy(buf, cp);
+	strcpy2(buf, cp);
 	trp = stree = newstree(NULL);
 	for (;;) {
 		if (cp) {
 			trp = analyze(cp, trp, 1);
-			free(cp);
+			free2(cp);
 			if (!trp || !(trp -> cont)) break;
 		}
 
 		lcmdline = -1;
 		hideclock = 1;
 		cp = inputshellstr(promptstr2, -1, NULL);
-		if (cp == (char *)-1) break;
+		if (cp == vnullstr) break;
 		else if (!cp) continue;
 
 		l = strlen(cp);
 		buf = realloc2(buf, len + l + 1 + 1);
 		buf[len++] = '\n';
-		strcpy(&(buf[len]), cp);
+		strcpy2(&(buf[len]), cp);
 		len += l;
 	}
 	freestree(stree);
-	free(stree);
+	free2(stree);
 	cp = buf;
 	if (l >= 0) rewritefile(1);
-#endif	/* !_NOORIGSHELL */
+#endif	/* DEP_ORIGSHELL */
 
 	return(cp);
 }
 
 static char *NEAR addoption(command, arg, stp)
-char *command, *arg;
+char *command;
+CONST char *arg;
 macrostat *stp;
 {
 	char *cp;
@@ -1277,7 +1269,7 @@ macrostat *stp;
 		j = flag2str(line, j, flags);
 	}
 	line[j] = '\0';
-	free(command);
+	free2(command);
 
 	if (p < 0) p = strlen(line);
 	command = line;
@@ -1286,31 +1278,31 @@ macrostat *stp;
 #endif	/* MACROMETA */
 
 	if (!(wastty = isttyiomode)) Xttyiomode(1);
-	cp = inputstr(nullstr, 0, p, command, HST_COM);
+	cp = inputstr(nullstr, 0, p, command, HST_COMM);
 	if (!wastty) Xstdiomode();
-	free(command);
+	free2(command);
 	if (!cp) {
-		if (!wastty) fputnl(stdout);
+		if (!wastty) fputnl(Xstdout);
 		return(NULL);
 	}
 	if (!*cp) {
-		free(cp);
+		free2(cp);
 		return(NULL);
 	}
 	command = evalcommand(cp, arg, stp);
-	free(cp);
+	free2(cp);
 	if (!command) return(NULL);
 	if (!*command) {
-		free(command);
+		free2(command);
 		return(NULL);
 	}
 
 	return(command);
 }
 
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 static int NEAR system3(command, flags)
-char *command;
+CONST char *command;
 int flags;
 {
 	char *cp, *tmp;
@@ -1323,19 +1315,19 @@ int flags;
 	if ((ret = execpseudoshell(command, flags)) < 0) {
 		tmp = evalcomstr(command, CMDLINE_DELIM);
 		ret = system2(tmp, flags);
-		free(tmp);
+		free2(tmp);
 	}
 	sigvecset(n);
-	if (cp) free(cp);
+	free2(cp);
 
 	return(ret);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* !DEP_ORIGSHELL */
 
 int isinternalcomm(command)
-char *command;
+CONST char *command;
 {
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	syntaxtree *stree;
 	char **subst;
 	int i, *len;
@@ -1344,10 +1336,7 @@ char *command;
 	int ret;
 
 	ret = 1;
-#ifdef	_NOORIGSHELL
-	if (getargs(command, &argv) > 0 && checkinternal(argv[0]) < 0) ret = 0;
-	freevar(argv);
-#else	/* !_NOORIGSHELL */
+#ifdef	DEP_ORIGSHELL
 	stree = newstree(NULL);
 	if (analyze(command, stree, 1)) {
 		if ((argv = getsimpleargv(stree))) {
@@ -1357,25 +1346,28 @@ char *command;
 			stripquote(argv[0], EA_STRIPQ);
 			if (argv[0] && checkinternal(argv[0]) < 0) ret = 0;
 			freevar(subst);
-			free(len);
+			free2(len);
 		}
 	}
 	freestree(stree);
-	free(stree);
-#endif	/* !_NOORIGSHELL */
+	free2(stree);
+#else	/* !DEP_ORIGSHELL */
+	if (getargs(command, &argv) > 0 && checkinternal(argv[0]) < 0) ret = 0;
+	freevar(argv);
+#endif	/* !DEP_ORIGSHELL */
 
 	return(ret);
 }
 
 int execmacro(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
 #ifdef	_NOEXTRAMACRO
 	char *buf;
 	int r, status;
 #endif
-	static char *duparg = NULL;
+	static CONST char *duparg = NULL;
 	macrostat st;
 	char *tmp;
 	int i, haslist, ret;
@@ -1403,11 +1395,12 @@ int flags;
 		return(-1);
 	}
 	st.flags |= F_ARGSET;
-	while (st.addopt >= 0 && !(st.flags & F_NOADDOPT))
+	while (st.addopt >= 0 && !(st.flags & F_NOADDOPT)) {
 		if (!(tmp = addoption(tmp, duparg, &st))) {
 			if (arg) duparg = NULL;
 			return(-1);
 		}
+	}
 
 #ifdef	_NOEXTRAMACRO
 	status = internal_status;
@@ -1416,7 +1409,7 @@ int flags;
 		tmp = _demacroarg(tmp);
 # endif
 		r = system3(tmp, st.flags);
-		free(tmp);
+		free2(tmp);
 		tmp = NULL;
 		if (!(flags & F_ARGSET)) st.flags &= ~F_ARGSET;
 
@@ -1437,7 +1430,7 @@ int flags;
 			buf = _demacroarg(buf);
 # endif
 			ret = system3(buf, st.flags);
-			free(buf);
+			free2(buf);
 			status = internal_status;
 		}
 	}
@@ -1448,7 +1441,7 @@ int flags;
 			buf = _demacroarg(buf);
 # endif
 			r = system3(buf, st.flags);
-			free(buf);
+			free2(buf);
 			if (r > ret && (ret = r) >= 127) break;
 			if (internal_status <= FNC_FAIL) status = FNC_EFFECT;
 			else if (internal_status > status)
@@ -1461,7 +1454,7 @@ int flags;
 	if (internal_status <= FNC_FAIL) internal_status = FNC_EFFECT;
 #endif	/* !_NOEXTRAMACRO */
 
-	if (tmp) free(tmp);
+	free2(tmp);
 	if (haslist && internal_status <= FNC_FAIL) {
 		for (i = 0; i < maxfile; i++)
 			filelist[i].tmpflags &= ~(F_ISARG | F_ISMRK);
@@ -1473,12 +1466,12 @@ int flags;
 	return(ret);
 }
 
-FILE *popenmacro(command, arg, flags)
-char *command, *arg;
+XFILE *popenmacro(command, arg, flags)
+CONST char *command, *arg;
 int flags;
 {
 	macrostat st;
-	FILE *fp;
+	XFILE *fp;
 	char *tmp, *lang;
 	int i;
 
@@ -1501,18 +1494,18 @@ int flags;
 	fp = popen2(tmp);
 	for (i = 0; i < LOCALELISTSIZ; i++) {
 		setenv2(localelist[i].env, localelist[i].org, 1);
-		if (localelist[i].org) free(localelist[i].org);
+		free2(localelist[i].org);
 	}
-	free(tmp);
+	free2(tmp);
 
 	return(fp);
 }
 
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 static char *NEAR evalargs(command, argc, argv)
-char *command;
+CONST char *command;
 int argc;
-char *argv[];
+char *CONST *argv;
 {
 	char *cp, *line;
 	ALLOC_T size;
@@ -1547,7 +1540,7 @@ char *argv[];
 }
 
 int execusercomm(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
 	char *cp, **argv;
@@ -1562,7 +1555,7 @@ int flags;
 		for (j = 0; userfunclist[i].comm[j]; j++) {
 			cp = evalargs(userfunclist[i].comm[j], argc, argv);
 			r = execmacro(cp, arg, flags);
-			free(cp);
+			free2(cp);
 			if (r > ret && (ret = r) >= 127) break;
 			if (internal_status <= FNC_FAIL) status = FNC_EFFECT;
 			else if (internal_status > status)
@@ -1576,7 +1569,7 @@ int flags;
 }
 
 static char *NEAR evalalias(command)
-char *command;
+CONST char *command;
 {
 	char *cp;
 	int i, len;
@@ -1586,19 +1579,20 @@ char *command;
 	if ((i = searchalias(command, len)) >= maxalias) return(NULL);
 	cp = malloc2((int)strlen(command) - len
 		+ strlen(aliaslist[i].comm) + 1);
-	strcpy(strcpy2(cp, aliaslist[i].comm), &(command[len]));
+	strcpy2(strcpy2(cp, aliaslist[i].comm), &(command[len]));
 
 	return(cp);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* !DEP_ORIGSHELL */
 
-int entryhist(n, s, uniq)
-int n;
-char *s;
-int uniq;
+int entryhist(s, flags)
+CONST char *s;
+int flags;
 {
-	int i, size;
+	char *new;
+	int i, n, size;
 
+	n = (flags & HST_TYPE);
 	size = (int)histsize[n];
 	if (!history[n]) {
 		history[n] = (char **)malloc2((size + 1) * sizeof(char *));
@@ -1615,26 +1609,26 @@ int uniq;
 	}
 
 	if (!s || !*s) return(0);
-	s = (n == 1) ? killmeta(s) : strdup2(s);
+	new = (n == 1) ? killmeta(s) : strdup2(s);
 
 	if (histno[n]++ >= MAXHISTNO) histno[n] = (short)0;
 
-	if (uniq) {
+	if (flags & HST_UNIQ) {
 		for (i = 0; i <= size; i++) {
 			if (!history[n][i]) continue;
-#if	defined (PATHNOCASE) && defined (_USEDOSPATH)
-			if (n == 1 && *s != *(history[n][i])) continue;
+#if	defined (PATHNOCASE) && defined (DEP_DOSPATH)
+			if (n == 1 && *new != *(history[n][i])) continue;
 #endif
-			if (!strpathcmp(s, history[n][i])) break;
+			if (!strpathcmp(new, history[n][i])) break;
 		}
 		if (i < size) size = i;
 	}
 
-	if (history[n][size]) free(history[n][size]);
+	free2(history[n][size]);
 	for (i = size; i > 0; i--) history[n][i] = history[n][i - 1];
-	history[n][0] = s;
-#ifndef	_NOPTY
-	sendparent(TE_SETHISTORY, n, s, uniq);
+	*(history[n]) = new;
+#ifdef	DEP_PTY
+	sendparent(TE_SETHISTORY, s, flags);
 #endif
 
 	return(1);
@@ -1649,7 +1643,7 @@ int n;
 	size = (int)histsize[n];
 	if (!history[n] || size > histbufsize[n] || size <= 0) return(NULL);
 	if (--histno[n] < (short)0) histno[n] = MAXHISTNO;
-	tmp = history[n][0];
+	tmp = *(history[n]);
 	for (i = 0; i < size; i++) history[n][i] = history[n][i + 1];
 	history[n][size--] = NULL;
 
@@ -1663,7 +1657,7 @@ int n;
 	char *line;
 	int i, j, size;
 
-	if (!histfile[n] || !histfile[n][0]) return(0);
+	if (!histfile[n] || !*(histfile[n])) return(0);
 	lck = lockfopen(histfile[n], "r", O_TEXT | O_RDONLY);
 	if (!lck || !(lck -> fp)) {
 		lockclose(lck);
@@ -1676,12 +1670,13 @@ int n;
 	histno[n] = (short)0;
 
 	i = -1;
-	while ((line = fgets2(lck -> fp, 1))) {
+	Xsetflags(lck -> fp, XF_NULLCONV);
+	while ((line = Xfgets(lck -> fp))) {
 		if (histno[n]++ >= MAXHISTNO) histno[n] = (short)0;
 		if (i < size) i++;
-		else free(history[n][i]);
+		else free2(history[n][i]);
 		for (j = i; j > 0; j--) history[n][j] = history[n][j - 1];
-		history[n][0] = line;
+		*(history[n]) = line;
 	}
 	lockclose(lck);
 
@@ -1691,15 +1686,15 @@ int n;
 }
 
 VOID convhistory(s, fp)
-char *s;
-FILE *fp;
+CONST char *s;
+XFILE *fp;
 {
 	char *eol;
 
 	if (!s || !*s) return;
 
-	while ((eol = strchr(s, '\n'))) {
-		VOID_C Xfwrite(s, sizeof(char), eol++ - s, fp);
+	while ((eol = strchr2(s, '\n'))) {
+		VOID_C Xfwrite(s, eol++ - s, fp);
 		VOID_C Xfputc('\0', fp);
 		s = eol;
 	}
@@ -1713,8 +1708,8 @@ int n;
 	lockbuf_t *lck;
 	int i, size;
 
-	if (!histfile[n] || !histfile[n][0] || savehist[n] <= 0) return(0);
-	if (!history[n] || !history[n][0]) return(-1);
+	if (!histfile[n] || !*(histfile[n]) || savehist[n] <= 0) return(0);
+	if (!history[n] || !*(history[n])) return(-1);
 	lck = lockfopen(histfile[n], "w",
 		O_TEXT | O_WRONLY | O_CREAT | O_TRUNC);
 	if (!lck || !(lck -> fp)) {
@@ -1731,7 +1726,7 @@ int n;
 }
 
 int parsehist(str, ptrp, quote)
-char *str;
+CONST char *str;
 int *ptrp, quote;
 {
 	char *cp;
@@ -1740,7 +1735,7 @@ int *ptrp, quote;
 	ptr = (ptrp) ? *ptrp : 0;
 	size = (int)histsize[0];
 	if (str[ptr] == '!') n = 0;
-	else if (str[ptr] == '=' || strchr(IFS_SET, str[ptr])) n = ptr = -1;
+	else if (str[ptr] == '=' || strchr2(IFS_SET, str[ptr])) n = ptr = -1;
 	else if ((cp = sscanf2(&(str[ptr]), "%-d", &n))) {
 		if (!n) n = -1;
 		else if (n < 0) {
@@ -1803,15 +1798,15 @@ char *command;
 					continue;
 				}
 				command[++i] = '\0';
-				fprintf2(stderr, "%k: Event not found.\n",
+				fprintf2(Xstderr, "%k: Event not found.\n",
 					&(command[len]));
-				free(cp);
+				free2(cp);
 				return(NULL);
 			}
 			hit++;
 			len = strlen(history[0][n]);
 			cp = c_realloc(cp, j + len + 1, &size);
-			strncpy(&cp[j], history[0][n], len);
+			strncpy2(&cp[j], history[0][n], len);
 			j += len;
 			continue;
 		}
@@ -1819,12 +1814,12 @@ char *command;
 	}
 	cp[j] = '\0';
 	if (hit) {
-		kanjifputs(cp, stderr);
-		fputnl(stderr);
-		free(command);
+		kanjifputs(cp, Xstderr);
+		fputnl(Xstderr);
+		free2(command);
 		return(cp);
 	}
-	free(cp);
+	free2(cp);
 
 	return(command);
 }
@@ -1835,16 +1830,15 @@ int n;
 {
 	int i;
 
-	if (!history[n] || !history[n][0]) return;
-	for (i = 0; i <= histbufsize[n]; i++)
-		if (history[n][i]) free(history[n][i]);
-	free(history[n]);
+	if (!history[n] || !*(history[n])) return;
+	for (i = 0; i <= histbufsize[n]; i++) free2(history[n][i]);
+	free2(history[n]);
 }
 #endif
 
-#if	!defined (_NOCOMPLETE) && defined (_NOORIGSHELL)
+#if	!defined (_NOCOMPLETE) && !defined (DEP_ORIGSHELL)
 int completealias(com, len, argc, argvp)
-char *com;
+CONST char *com;
 int len, argc;
 char ***argvp;
 {
@@ -1863,7 +1857,7 @@ char ***argvp;
 }
 
 int completeuserfunc(com, len, argc, argvp)
-char *com;
+CONST char *com;
 int len, argc;
 char ***argvp;
 {
@@ -1880,4 +1874,4 @@ char ***argvp;
 
 	return(argc);
 }
-#endif	/* !_NOCOMPLETE && _NOORIGSHELL */
+#endif	/* !_NOCOMPLETE && !DEP_ORIGSHELL */
