@@ -5,43 +5,30 @@
  */
 
 #include "fd.h"
-#ifndef	_NOPTY
-
-#include <fcntl.h>
-#include <sys/time.h>
+#include "wait.h"
+#include "termio.h"
 #include "func.h"
 #include "kanji.h"
-#include "termemu.h"
 
-#ifdef	USESELECTH
-#include <sys/select.h>
-#endif
-
-#ifdef	_NOORIGSHELL
-#include <signal.h>
-#include "termio.h"
-#include "wait.h"
-#else
+#ifdef	DEP_ORIGSHELL
 #include "system.h"
-# ifndef	NOJOB
-extern jobtable *joblist;
-extern int maxjobs;
-# endif
+#endif
+#ifdef	DEP_PTY
+#include "termemu.h"
 #endif
 
-#ifndef	FD_SET
-typedef struct fd_set {
-	u_int fds_bits[1];
-} fd_set;
-# define	FD_ZERO(p)	(((p) -> fds_bits[0]) = 0)
-# define	FD_SET(n, p)	(((p) -> fds_bits[0]) |= ((u_int)1 << (n)))
-#endif	/* !FD_SET */
+#ifdef	DEP_PTY
 
-extern int internal_status;
+extern char fullpath[];
 extern int hideclock;
 extern int fdmode;
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
+extern jobtable *joblist;
+extern int maxjobs;
+#endif
 
 int ptymode = 0;
+int ptyinternal = 0;
 char *ptyterm = NULL;
 int ptymenukey = -1;
 ptyinfo_t ptylist[MAXWINDOWS];
@@ -51,20 +38,21 @@ int parentfd = -1;
 char *ptytmpfile = NULL;
 
 static VOID NEAR doscroll __P_((int, int, int, int));
+static VOID NEAR closeallpty __P_((int));
 static int NEAR genbackend __P_((VOID_A));
 static VOID NEAR sendvar __P_((int, char **));
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 static VOID NEAR sendheredoc __P_((int, heredoc_t *));
 static VOID NEAR sendrlist __P_((int, redirectlist *));
 static VOID NEAR sendcommand __P_((int, command_t *, syntaxtree *));
 static VOID NEAR sendstree __P_((int, syntaxtree *));
 #endif
-static VOID NEAR awakechild __P_((char *, char *, int));
-#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
+static VOID NEAR awakechild __P_((CONST char *, CONST char *, int));
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 static int trap_hup __P_((VOID_A));
 static int NEAR recvmacro __P_((char **, char **, int *));
 #endif
-static int NEAR callmacro __P_((char *, char *, int));
+static int NEAR callmacro __P_((CONST char *, CONST char *, int));
 
 
 static VOID NEAR doscroll(n, c, x, y)
@@ -94,7 +82,7 @@ int n, c, x, y, min, max;
 	}
 
 	y1 = y2 = -1;
-	switch(n) {
+	switch (n) {
 		case C_DOWN:
 		case C_SCROLLFORW:
 		case C_NEWLINE:
@@ -132,29 +120,12 @@ int n, c, x, y, min, max;
 	}
 }
 
-int selectpty(fd, fds, result, timeout)
-int fd, fds[];
+int selectpty(max, fds, result, timeout)
+int max, fds[];
 char result[];
 int timeout;
 {
-	fd_set readfds;
 	struct timeval tv, *t;
-	int i, n, max;
-
-	max = -1;
-	FD_ZERO(&readfds);
-	if (fd < 0) max = 0;
-	else {
-		max = fd;
-		FD_SET(fd, &readfds);
-	}
-	for (i = 0; i < MAXWINDOWS; i++) {
-		if (!(ptylist[i].pid)) continue;
-		if (fds[i] > max) max = fds[i];
-		FD_SET(fds[i], &readfds);
-	}
-
-	if (!max) return(0);
 
 	if (timeout < 0) t = NULL;
 	else if (!timeout) {
@@ -168,13 +139,18 @@ int timeout;
 		t = &tv;
 	}
 
-	if ((n = select(max + 1, &readfds, NULL, NULL, t)) < 0) return(-1);
-	for (i = 0; i < MAXWINDOWS; i++)
-		result[i] = (ptylist[i].pid && FD_ISSET(fds[i], &readfds))
-			? 1 : 0;
-	if (fd >= 0) result[i] = (FD_ISSET(fd, &readfds)) ? 1 : 0;
+	return(readselect(max, fds, result, t));
+}
 
-	return(n);
+static VOID NEAR closeallpty(n)
+int n;
+{
+	int i;
+
+	for (i = 0; i < n; i++) {
+		safeclose(ptylist[i].fd);
+		ptylist[i].fd = -1;
+	}
 }
 
 static int NEAR genbackend(VOID_A)
@@ -200,41 +176,41 @@ static int NEAR genbackend(VOID_A)
 	if (i < MAXWINDOWS || (pid = Xfork()) < (p_id_t)0) {
 		safeclose(fds[0]);
 		safeclose(fds[1]);
+		closeallpty(i);
 		while (--i >= 0) {
-			free(ptylist[i].path);
+			free2(ptylist[i].path);
 			ptylist[i].path = NULL;
-			safeclose(ptylist[i].fd);
-			ptylist[i].fd = -1;
 		}
 		return(-1);
 	}
 
 	if (!pid) {
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 		mypid = getpid();
 #endif
 		safeclose(fds[1]);
 		emufd = fds[0];
 		emupid = (p_id_t)0;
 		for (i = 0; i < MAXWINDOWS; i++) {
-			free(ptylist[i].path);
+			free2(ptylist[i].path);
 			ptylist[i].path = NULL;
 			resetptyterm(i, 1);
 		}
 		resetptyterm(i, 1);
 
 		backend();
-		for (i = 0; i < MAXWINDOWS; i++) safeclose(ptylist[i].fd);
+		closeallpty(MAXWINDOWS);
 		_exit(0);
 	}
 
 	safeclose(fds[0]);
-	emufd = fds[1];
+	emufd = newdup(fds[1]);
 	emupid = pid;
-	for (i = 0; i < MAXWINDOWS; i++) {
-		safeclose(ptylist[i].fd);
-		ptylist[i].fd = -1;
-	}
+#ifdef	CYGWIN
+	for (i = 0; i < MAXWINDOWS; i++) ptylist[i].fd = newdup(ptylist[i].fd);
+#else
+	closeallpty(MAXWINDOWS);
+#endif
 
 	return(0);
 }
@@ -261,7 +237,7 @@ int fd, cmd;
 	VOID_C getch2();
 	keyflush();
 	loadtermio(ttyio, tty, NULL);
-	if (tty) free(tty);
+	free2(tty);
 }
 
 int recvbuf(fd, buf, nbytes)
@@ -288,7 +264,7 @@ int nbytes;
 
 VOID sendbuf(fd, buf, nbytes)
 int fd;
-VOID_P buf;
+CONST VOID_P buf;
 int nbytes;
 {
 	VOID_C surewrite(fd, buf, nbytes);
@@ -326,7 +302,7 @@ char **cpp;
 		if (recvbuf(fd, &len, sizeof(len)) < 0) return(-1);
 		cp = malloc2(len + 1);
 		if (recvbuf(fd, cp, len) < 0) {
-			free(cp);
+			free2(cp);
 			return(-1);
 		}
 		cp[len] = '\0';
@@ -338,7 +314,7 @@ char **cpp;
 
 VOID sendstring(fd, s)
 int fd;
-char *s;
+CONST char *s;
 {
 	ALLOC_T len;
 
@@ -364,7 +340,7 @@ char **var;
 	for (i = 0; i < c; i++) sendstring(fd, var[i]);
 }
 
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 static VOID NEAR sendheredoc(fd, hdp)
 int fd;
 heredoc_t *hdp;
@@ -422,7 +398,7 @@ syntaxtree *trp;
 	sendcommand(fd, trp -> comm, trp);
 	sendstree(fd, trp -> next);
 }
-#endif	/* !_NOORIGSHELL */
+#endif	/* DEP_ORIGSHELL */
 
 #ifdef	USESTDARGH
 /*VARARGS1*/
@@ -434,14 +410,14 @@ int cmd;
 va_dcl
 #endif
 {
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	syntaxtree *trp;
 #endif
 #ifndef	_NOARCHIVE
-	launchtable *lp;
-	archivetable *ap;
+	lsparse_t *lp;
+	archive_t *ap;
 #endif
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 	devinfo *devp;
 #endif
 	bindtable *bindp;
@@ -452,7 +428,7 @@ va_dcl
 	int n, fd, val;
 
 	if (parentfd < 0) return;
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	if (mypid != shellpid) return;
 #endif
 
@@ -480,35 +456,20 @@ va_dcl
 			sendbuf(fd, &varp, sizeof(varp));
 			break;
 		case TE_CHDIR:
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 		case TE_SETEXPORT:
 		case TE_SETRONLY:
 #endif
-#if	defined (_NOORIGSHELL) || !defined (NOALIAS)
+#if	!defined (DEP_ORIGSHELL) || !defined (NOALIAS)
 		case TE_DELETEALIAS:
 #endif
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 		case TE_DELETEFUNCTION:
 #endif
 			cp = va_arg(args, char *);
 			sendstring(fd, cp);
 			break;
-#ifdef	_NOORIGSHELL
-		case TE_PUTSHELLVAR:
-			cp = va_arg(args, char *);
-			func1 = va_arg(args, char *);
-			n = va_arg(args, int);
-			sendbuf(fd, &n, sizeof(n));
-			sendstring(fd, func1);
-			sendstring(fd, cp);
-			break;
-		case TE_ADDFUNCTION:
-			cp = va_arg(args, char *);
-			var = va_arg(args, char **);
-			sendvar(fd, var);
-			sendstring(fd, cp);
-			break;
-#else	/* !_NOORIGSHELL */
+#ifdef	DEP_ORIGSHELL
 		case TE_PUTEXPORTVAR:
 		case TE_PUTSHELLVAR:
 		case TE_UNSET:
@@ -530,21 +491,34 @@ va_dcl
 			sendstring(fd, cp);
 			sendstree(fd, trp);
 			break;
-#endif	/* !_NOORIGSHELL */
-#if	defined (_NOORIGSHELL) || !defined (NOALIAS)
+#else	/* !DEP_ORIGSHELL */
+		case TE_PUTSHELLVAR:
+			cp = va_arg(args, char *);
+			func1 = va_arg(args, char *);
+			n = va_arg(args, int);
+			sendbuf(fd, &n, sizeof(n));
+			sendstring(fd, func1);
+			sendstring(fd, cp);
+			break;
+		case TE_ADDFUNCTION:
+			cp = va_arg(args, char *);
+			var = va_arg(args, char **);
+			sendvar(fd, var);
+			sendstring(fd, cp);
+			break;
+#endif	/* !DEP_ORIGSHELL */
+#if	!defined (DEP_ORIGSHELL) || !defined (NOALIAS)
 		case TE_ADDALIAS:
 			cp = va_arg(args, char *);
 			func1 = va_arg(args, char *);
 			sendstring(fd, func1);
 			sendstring(fd, cp);
 			break;
-#endif	/* _NOORIGSHELL || !NOALIAS */
+#endif	/* !DEP_ORIGSHELL || !NOALIAS */
 		case TE_SETHISTORY:
-			n = va_arg(args, int);
 			cp = va_arg(args, char *);
-			val = va_arg(args, int);
+			n = va_arg(args, int);
 			sendbuf(fd, &n, sizeof(n));
-			sendbuf(fd, &val, sizeof(val));
 			sendstring(fd, cp);
 			break;
 		case TE_ADDKEYBIND:
@@ -564,11 +538,14 @@ va_dcl
 		case TE_DELETELAUNCH:
 		case TE_DELETEARCH:
 #endif
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 		case TE_DELETEDRV:
 #endif
-#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 		case TE_CHANGESTATUS:
+#endif
+#ifdef	DEP_IME
+		case TE_FREEROMAN:
 #endif
 			n = va_arg(args, int);
 			sendbuf(fd, &n, sizeof(n));
@@ -582,7 +559,7 @@ va_dcl
 #ifndef	_NOARCHIVE
 		case TE_ADDLAUNCH:
 			n = va_arg(args, int);
-			lp = va_arg(args, launchtable *);
+			lp = va_arg(args, lsparse_t *);
 			sendbuf(fd, &n, sizeof(n));
 			sendbuf(fd, lp, sizeof(*lp));
 			sendstring(fd, lp -> ext);
@@ -593,7 +570,7 @@ va_dcl
 			break;
 		case TE_ADDARCH:
 			n = va_arg(args, int);
-			ap = va_arg(args, archivetable *);
+			ap = va_arg(args, archive_t *);
 			sendbuf(fd, &n, sizeof(n));
 			sendbuf(fd, &(ap -> flags), sizeof(ap -> flags));
 			sendstring(fd, ap -> ext);
@@ -601,7 +578,7 @@ va_dcl
 			sendstring(fd, ap -> u_comm);
 			break;
 #endif	/* !_NOARCHIVE */
-#ifdef	_USEDOSEMU
+#ifdef	DEP_DOSEMU
 		case TE_INSERTDRV:
 			n = va_arg(args, int);
 			devp = va_arg(args, devinfo *);
@@ -609,7 +586,7 @@ va_dcl
 			sendbuf(fd, devp, sizeof(*devp));
 			sendstring(fd, devp -> name);
 			break;
-#endif	/* _USEDOSEMU */
+#endif	/* DEP_DOSEMU */
 		case TE_SAVETTYIO:
 			n = va_arg(args, int);
 			val = (n >= 0 && duptty[n]) ? TIO_BUFSIZ : 0;
@@ -617,6 +594,20 @@ va_dcl
 			sendbuf(fd, &val, sizeof(val));
 			if (n >= 0 && duptty[n]) sendbuf(fd, duptty[n], val);
 			break;
+#ifdef	DEP_IME
+		case TE_ADDROMAN:
+			cp = va_arg(args, char *);
+			func1 = va_arg(args, char *);
+			sendstring(fd, cp);
+			sendstring(fd, func1);
+			break;
+#endif
+		case TE_INTERNAL:
+			n = va_arg(args, int);
+			cp = va_arg(args, char *);
+			sendbuf(fd, &n, sizeof(n));
+			sendstring(fd, cp);
+			break;
 		default:
 			break;
 	}
@@ -625,7 +616,7 @@ va_dcl
 }
 
 static VOID NEAR awakechild(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
 	if (isttyiomode) {
@@ -638,9 +629,10 @@ int flags;
 	sendbuf(emufd, &flags, sizeof(flags));
 	sendstring(emufd, command);
 	sendstring(emufd, arg);
+	sendstring(emufd, fullpath);
 }
 
-#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 static int trap_hup(VOID_A)
 {
 	int i;
@@ -664,7 +656,7 @@ static int NEAR recvmacro(commandp, argp, flagsp)
 char **commandp, **argp;
 int *flagsp;
 {
-	char *command, *arg;
+	char *command, *arg, *cwd;
 	int flags;
 
 	Xttyiomode(1);
@@ -672,45 +664,51 @@ int *flagsp;
 	|| recvstring(ttyio, &command) < 0)
 		return(-1);
 	if (recvstring(ttyio, &arg) < 0) {
-		if (command) free(command);
+		free2(command);
 		return(-1);
 	}
+	if (recvstring(ttyio, &cwd) >= 0 && cwd) {
+		VOID_C chdir2(cwd);
+		free2(cwd);
+	}
 
 	keyflush();
 	if (!(flags & F_TTYIOMODE)) Xstdiomode();
 	else if (!(flags & F_TTYNL)) Xttyiomode(0);
 
-	if (*commandp) free(*commandp);
-	if (*argp) free(*argp);
+	free2(*commandp);
+	free2(*argp);
 	*commandp = command;
 	*argp = arg;
 	*flagsp = flags;
 
 	return(0);
 }
-#endif	/* !_NOORIGSHELL && !NOJOB */
+#endif	/* DEP_ORIGSHELL && !NOJOB */
 
 static int NEAR callmacro(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
 	int i, n;
 
 	if (parentfd < 0) {
+		changewin(MAXWINDOWS, (p_id_t)-1);
 		if (ptymode) {
-			hideclock = 1;
+			hideclock = 2;
 			warning(0, NOPTY_K);
 		}
+		killpty(win, NULL);
 		for (i = 0; i < MAXWINDOWS; i++) if (ptylist[i].pid) break;
 		if (i < MAXWINDOWS) {
-			hideclock = 1;
+			hideclock = 2;
 			if (!yesno(KILL_K)) return(-1);
-			killallpty();
 		}
+		killallpty();
 	}
 
 	if (flags & F_DOSYSTEM) n = dosystem(command);
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 	else if (flags & F_EVALMACRO) n = execusercomm(command, arg, flags);
 #endif
 	else n = execmacro(command, arg, flags);
@@ -719,22 +717,23 @@ int flags;
 }
 
 int ptymacro(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
+	char *command2, *arg2;
+#endif
 	p_id_t pid;
 	char *tty, *ws, path[MAXPATHLEN], buf[TIO_BUFSIZ + TIO_WINSIZ];
 	u_char uc;
 	int i, n, fd, fds[2];
 
-#ifndef	_NOORIGSHELL
-	if (isshptymode()) {
-		if (!(flags & F_DOSYSTEM))
-			return(callmacro(command, arg, flags));
-	}
+#ifdef	DEP_ORIGSHELL
+	if (isshptymode()) n = (flags & F_DOSYSTEM) ? 0 : -1;
 	else
 #endif
-	if (!ptymode) return(callmacro(command, arg, flags));
+	n = (ptymode && !ptyinternal && ttyio >= 0) ? 0 : -1;
+	if (n < 0) return(callmacro(command, arg, flags));
 
 	if (ptylist[win].pid && emufd >= 0) {
 		awakechild(command, arg, flags);
@@ -744,12 +743,12 @@ int flags;
 	savetermio(ttyio, &tty, &ws);
 	if (tty) {
 		memcpy(buf, tty, TIO_BUFSIZ);
-		free(tty);
+		free2(tty);
 		tty = buf;
 	}
 	if (ws) {
 		memcpy(&(buf[TIO_BUFSIZ]), ws, TIO_WINSIZ);
-		free(ws);
+		free2(ws);
 		ws = &(buf[TIO_BUFSIZ]);
 	}
 
@@ -764,7 +763,7 @@ int flags;
 		VOID_C Xclose(fd);
 	}
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	changeinkcode();
 	changeoutkcode();
 #endif
@@ -773,7 +772,7 @@ int flags;
 	if ((pid = Xfork()) < (p_id_t)0) {
 		if (fd >= 0 && ptytmpfile) {
 			rmtmpfile(ptytmpfile);
-			free(ptytmpfile);
+			free2(ptytmpfile);
 			ptytmpfile = NULL;
 		}
 		sigvecset(n);
@@ -783,11 +782,14 @@ int flags;
 	}
 
 	if (!pid) {
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 		mypid = getpid();
+#endif
+#ifdef	CYGWIN
+		closeallpty(MAXWINDOWS);
 #endif
 		if (Xlogin_tty(ptylist[win].path, tty, ws) < 0) _exit(1);
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 		if (isshptymode()) /*EMPTY*/;
 		else
 #endif
@@ -796,7 +798,7 @@ int flags;
 
 		for (i = 0; i < MAXWINDOWS; i++) {
 			ptylist[i].pid = (p_id_t)0;
-			free(ptylist[i].path);
+			free2(ptylist[i].path);
 			ptylist[i].path = NULL;
 			ptylist[i].pipe = -1;
 			ptylist[i].status = 0;
@@ -804,15 +806,14 @@ int flags;
 		emupid = (p_id_t)0;
 		safeclose(emufd);
 		emufd = -1;
-		if (fileno(ttyout) == ttyio) dup2(STDIN_FILENO, ttyio);
+		if (Xfileno(ttyout) == ttyio) Xdup2(STDIN_FILENO, ttyio);
 		else {
-			fd = fileno(ttyout);
+			fd = Xfileno(ttyout);
 			closetty(&fd, &ttyout);
 			opentty(&fd, &ttyout);
-			dup2(fd, ttyio);
+			Xdup2(fd, ttyio);
 			safeclose(fd);
 		}
-		maxfile = -1;
 
 		setdefterment();
 		setdefkeyseq();
@@ -823,20 +824,25 @@ int flags;
 		uc = '\n';
 		sendbuf(parentfd, &uc, sizeof(uc));
 
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
+		command2 = arg2 = NULL;
+#endif
 		for (;;) {
 			syncptyout(-1, -1);
 			setlinecol();
 			n = evalstatus(callmacro(command, arg, flags));
-#if	defined (_NOORIGSHELL) || defined (NOJOB)
-			break;
-#else	/* !_NOORIGSHELL && !NOJOB */
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 			for (i = 0; i < maxjobs; i++)
 				if (joblist[i].pids) break;
 			if (i >= maxjobs) break;
 			VOID_C signal2(SIGHUP, (sigcst_t)trap_hup);
 			sendparent(TE_CHANGESTATUS, n);
-			if (recvmacro(&command, &arg, &flags) < 0) break;
-#endif	/* !_NOORIGSHELL && !NOJOB */
+			if (recvmacro(&command2, &arg2, &flags) < 0) break;
+			command = command2;
+			arg = arg2;
+#else	/* !DEP_ORIGSHELL || NOJOB */
+			break;
+#endif	/* !DEP_ORIGSHELL || NOJOB */
 		}
 
 		safeclose(parentfd);
@@ -846,7 +852,7 @@ int flags;
 
 	safeclose(fds[1]);
 	ptylist[win].pid = pid;
-	ptylist[win].pipe = fds[0];
+	ptylist[win].pipe = newdup(fds[0]);
 	ptylist[win].status = -1;
 
 	sigvecset(n);
@@ -867,8 +873,9 @@ int n, *statusp;
 		VOID_C waitstatus(ptylist[n].pid, 0, statusp);
 		ptylist[n].pid = (p_id_t)0;
 		ptylist[n].status = 0;
-		changewin(n, (p_id_t)0);
 	}
+	changewin(n, (p_id_t)0);
+	changewin(MAXWINDOWS, (p_id_t)-1);
 
 	safeclose(ptylist[n].pipe);
 	ptylist[n].pipe = -1;
@@ -880,10 +887,13 @@ VOID killallpty(VOID_A)
 
 	for (i = 0; i < MAXWINDOWS; i++) {
 		killpty(i, NULL);
-		free(ptylist[i].path);
+		free2(ptylist[i].path);
 		ptylist[i].path = NULL;
 	}
 
+#ifdef	CYGWIN
+	closeallpty(MAXWINDOWS);
+#endif
 	if (emupid) {
 		VOID_C kill(emupid, SIGHUP);
 #ifdef	SIGCONT
@@ -894,11 +904,35 @@ VOID killallpty(VOID_A)
 	}
 	if (ptytmpfile) {
 		rmtmpfile(ptytmpfile);
-		free(ptytmpfile);
+		free2(ptytmpfile);
 		ptytmpfile = NULL;
 	}
 
 	safeclose(emufd);
 	emufd = -1;
 }
-#endif	/* !_NOPTY */
+
+int checkpty(n)
+int n;
+{
+	int status;
+
+	if (!(ptylist[n].pid)) return(0);
+	if (waitstatus(ptylist[n].pid, WNOHANG, &status) < 0) return(0);
+	ptylist[n].pid = (p_id_t)0;
+	ptylist[n].status = status;
+	killpty(n, &status);
+
+	return(-1);
+}
+
+int checkallpty(VOID_A)
+{
+	int i, n;
+
+	n = 0;
+	for (i = 0; i < MAXWINDOWS; i++) if (checkpty(i) < 0) n = -1;
+
+	return(n);
+}
+#endif	/* DEP_PTY */
