@@ -4,62 +4,27 @@
  *	UNIX-like disk accessing module
  */
 
-#include <stdio.h>
-#include <fcntl.h>
-#include <io.h>
-#include "machine.h"
-
-#ifndef	NOUNISTDH
-#include <unistd.h>
-#endif
-
-#if	MSDOS && defined (_NOUSELFN) && !defined (_NODOSDRIVE)
-#define	_NODOSDRIVE
-#endif
-
-#include "unixdisk.h"
-#include "dosdisk.h"
+#include "headers.h"
+#include "depend.h"
 #include "kctype.h"
+#include "string.h"
+#include "sysemu.h"
+#include "pathname.h"
+#include "dosdisk.h"
+#include "unixdisk.h"
 
-#ifndef	ENODEV
-#define	ENODEV	EACCES
-#endif
-#ifndef	EIO
-#define	EIO	ENODEV
-#endif
-#ifndef	O_ACCMODE
-#define	O_ACCMODE	(O_RDONLY | O_WRONLY | O_RDWR)
+#if	MSDOS
+#include <io.h>
 #endif
 
-#define	KC_SJIS1	0001
-#define	KC_SJIS2	0002
-#define	KC_EUCJP	0010
+#define	KC_SJIS1		0001
+#define	KC_SJIS2		0002
+#define	KC_EUCJP		0010
 #define	int21call(reg, sreg)	intcall(0x21, reg, sreg)
 
-#ifdef	FD
-extern int _dospath __P_((char *));
-extern char *strdelim __P_((char *, int));
-extern char *strrdelim __P_((char *, int));
-extern int isdelim __P_((char *, int));
-extern char *strcatdelim __P_((char *));
-extern char *strcatdelim2 __P_((char *, char *, char *));
-extern int isdotdir __P_((char *));
-#else
-static int NEAR _dospath __P_((char *));
-static char *NEAR strdelim __P_((char *, int));
-static char *NEAR strrdelim __P_((char *, int));
-static int NEAR isdelim __P_((char *, int));
-static char *NEAR strcatdelim __P_((char *));
-static char *NEAR strcatdelim2 __P_((char *, char *, char *));
-static int NEAR isdotdir __P_((char *));
-#endif
-
-#ifdef	DJGPP
-static int NEAR dos_putpath __P_((char *, int));
-#endif
-static char *NEAR duplpath __P_((char *));
+static char *NEAR duplpath __P_((CONST char *));
 #ifndef	_NODOSDRIVE
-static char *NEAR regpath __P_((char *, char *));
+static char *NEAR regpath __P_((CONST char *, char *));
 static VOID NEAR biosreset __P_((int));
 static int NEAR _biosdiskio __P_((int, int, int, int,
 		u_char *, int, int, int));
@@ -74,17 +39,17 @@ static int NEAR lockdrive __P_((int));
 static int NEAR unlockdrive __P_((int, int));
 static int NEAR dosrawdiskio __P_((int, u_long, u_char *, int, int, int));
 #endif
-static int NEAR dos_findfirst __P_((char *, u_int, struct dosfind_t *));
+static int NEAR dos_findfirst __P_((CONST char *, u_int, struct dosfind_t *));
 static int NEAR dos_findnext __P_((struct dosfind_t *));
 #ifndef	_NOUSELFN
-static int NEAR lfn_findfirst __P_((u_int *, char *,
+static int NEAR lfn_findfirst __P_((u_int *, CONST char *,
 		u_int, struct lfnfind_t *));
 static int NEAR lfn_findnext __P_((u_int, struct lfnfind_t *));
 static int NEAR lfn_findclose __P_((u_int));
 #endif
 #ifndef	_NOUSELFN
 static int NEAR gendosname __P_((char *));
-static int NEAR unixrenamedir __P_((char *, char *));
+static int NEAR unixrenamedir __P_((CONST char *, CONST char *));
 #endif
 static u_int NEAR getdosmode __P_((u_int));
 static u_int NEAR putdosmode __P_((u_int));
@@ -92,18 +57,16 @@ static time_t NEAR getdostime __P_((u_int, u_int));
 #ifndef	_NOUSELFN
 static int NEAR putdostime __P_((u_short *, u_short *, time_t));
 #endif
-
-#ifndef	_NODOSDRIVE
-int dosdrive = 0;
-#endif
+static int NEAR alter_findfirst __P_((CONST char *, u_int *,
+		struct dosfind_t *, struct lfnfind_t *));
 
 #ifndef	_NODOSDRIVE
 static drvinfo drvlist['Z' - 'A' + 1];
 static int maxdrive = -1;
 #endif
 static int dos7access = 0;
-#define	D7_DOSVER	017
-#define	D7_CAPITAL	020
+#define	D7_DOSVER		017
+#define	D7_CAPITAL		020
 #if	!defined (DJGPP) && !defined (_NODOSDRIVE)
 static u_char int25bin[] = {
 	0x1e,				/* 0000: push ds */
@@ -140,194 +103,33 @@ static u_char int25bin[] = {
 };
 static int (far *doint25)__P_((VOID_A)) = NULL;
 #endif	/* !DJGPP && !_NODOSDRIVE */
-
-
-#ifndef	FD
-static int NEAR _dospath(path)
-char *path;
-{
-	return((isalpha2(*path) && path[1] == ':') ? *path : 0);
-}
-
-static char *NEAR strdelim(s, d)
-char *s;
-int d;
-{
-	int i;
-
-	if (d && _dospath(s)) return(s + 1);
-	for (i = 0; s[i]; i++) {
-		if (s[i] == _SC_) return(&(s[i]));
-# ifdef	BSPATHDELIM
-		if (iskanji1(s, i)) i++;
-# endif
-	}
-
-	return(NULL);
-}
-
-char *strrdelim(s, d)
-char *s;
-int d;
-{
-	char *cp;
-	int i;
-
-	if (d && _dospath(s)) cp = s + 1;
-	else cp = NULL;
-	for (i = 0; s[i]; i++) {
-		if (s[i] == _SC_) cp = &(s[i]);
-# ifdef	BSPATHDELIM
-		if (iskanji1(s, i)) i++;
-# endif
-	}
-
-	return(cp);
-}
-
-static int NEAR isdelim(s, ptr)
-char *s;
-int ptr;
-{
-# ifdef	BSPATHDELIM
-	int i;
-# endif
-
-	if (ptr < 0 || s[ptr] != _SC_) return(0);
-# ifdef	BSPATHDELIM
-	if (--ptr < 0) return(1);
-	if (!ptr) return(!iskanji1(s, 0));
-
-	for (i = 0; s[i] && i < ptr; i++) if (iskanji1(s, i)) i++;
-	if (!s[i] || i > ptr) return(1);
-
-	return(!iskanji1(s, i));
-# else
-	return(1);
-# endif
-}
-
-static char *NEAR strcatdelim(s)
-char *s;
-{
-	char *cp;
-	int i;
-
-	if (_dospath(s)) i = 2;
-	else i = 0;
-	if (!s[i]) return(&(s[i]));
-	if (s[i] == _SC_ && !s[i + 1]) return(&(s[i + 1]));
-
-	cp = NULL;
-	for (; s[i]; i++) {
-		if (s[i] == _SC_) {
-			if (!cp) cp = &(s[i]);
-			continue;
-		}
-		cp = NULL;
-# ifdef	BSPATHDELIM
-		if (iskanji1(s, i)) i++;
-# endif
-	}
-	if (!cp) {
-		cp = &(s[i]);
-		if (i >= MAXPATHLEN - 1) return(cp);
-		*cp = _SC_;
-	}
-	*(++cp) = '\0';
-
-	return(cp);
-}
-
-static char *NEAR strcatdelim2(buf, s1, s2)
-char *buf, *s1, *s2;
-{
-	char *cp;
-	int i, len;
-
-	if (_dospath(s1)) {
-		buf[0] = s1[0];
-		buf[1] = s1[1];
-		i = 2;
-	}
-	else i = 0;
-	if (!s1[i]) cp = &(buf[i]);
-	else if (s1[i] == _SC_ && !s1[i + 1]) {
-		cp = &(buf[i]);
-		*(cp++) = _SC_;
-	}
-	else {
-		cp = NULL;
-		for (; s1[i]; i++) {
-			buf[i] = s1[i];
-			if (s1[i] == _SC_) {
-				if (!cp) cp = &(buf[i]) + 1;
-				continue;
-			}
-			cp = NULL;
-# ifdef	BSPATHDELIM
-			if (iskanji1(s1, i)) {
-				if (!s1[++i]) break;
-				buf[i] = s1[i];
-			}
-# endif
-		}
-		if (!cp) {
-			cp = &(buf[i]);
-			if (i >= MAXPATHLEN - 1) {
-				*cp = '\0';
-				return(cp);
-			}
-			*(cp++) = _SC_;
-		}
-	}
-	if (s2) {
-		len = MAXPATHLEN - 1 - (cp - buf);
-		for (i = 0; s2[i] && i < len; i++) *(cp++) = s2[i];
-	}
-	*cp = '\0';
-
-	return(cp);
-}
-
-int isdotdir(s)
-char *s;
-{
-	if (s[0] != '.') /*EMPTY*/;
-	else if (!s[1]) return(2);
-	else if (s[1] != '.') /*EMPTY*/;
-	else if (!s[2]) return(1);
-
-	return(0);
-}
-#endif	/* !FD */
-
-#ifdef	DJGPP
-static int NEAR dos_putpath(path, offset)
-char *path;
-int offset;
-{
-	int i;
-
-	i = strlen(path) + 1;
-	dosmemput(path, i, __tb + offset);
-
-	return(i);
-}
+#ifdef	DOUBLESLASH
+static int dsdrive = 0;
 #endif
 
+
 static char *NEAR duplpath(path)
-char *path;
+CONST char *path;
 {
+#ifdef	DOUBLESLASH
+	int len;
+#endif
 	static char buf[MAXPATHLEN];
 	int i, j, ps;
 
-	if (path == buf) return(path);
+	if (path == buf) return((char *)path);
 	i = j = ps = 0;
 	if (_dospath(path)) {
 		buf[j++] = path[i++];
 		buf[j++] = path[i++];
 	}
+#ifdef	DOUBLESLASH
+	if ((len = isdslash(path))) {
+		memcpy(buf, path, len);
+		i = j = len;
+	}
+	else
+#endif
 	if (path[i] == _SC_) buf[j++] = path[i++];
 	for (; path[i]; i++) {
 		if (path[i] == _SC_) ps = 1;
@@ -349,6 +151,9 @@ int getcurdrv(VOID_A)
 {
 	int drive;
 
+#ifdef	DOUBLESLASH
+	if (dsdrive) return(dsdrive);
+#endif
 	drive = (dos7access & D7_CAPITAL) ? 'A' : 'a';
 #ifndef	_NODOSDRIVE
 	if (lastdrive >= 0) drive += toupper2(lastdrive) - 'A';
@@ -362,11 +167,22 @@ int getcurdrv(VOID_A)
 int setcurdrv(drive, nodir)
 int drive, nodir;
 {
+#ifndef	_NODOSDRIVE
+	char tmp[3];
+#endif
 	struct SREGS sreg;
 	__dpmi_regs reg;
-	char *path;
+	CONST char *path;
 	int drv, olddrv;
 
+#ifdef	DOUBLESLASH
+	if (drive == '_') {
+		dsdrive = drive;
+		return(0);
+	}
+	dsdrive = '\0';
+#endif
+
 	errno = EINVAL;
 	if (islower2(drive)) {
 		drv = drive - 'a';
@@ -380,11 +196,7 @@ int drive, nodir;
 
 #ifndef	_NODOSDRIVE
 	if (checkdrive(drv) > 0) {
-		char tmp[3];
-
-		tmp[0] = drive;
-		tmp[1] = ':';
-		tmp[2] = '\0';
+		VOID_C gendospath(tmp, drive, '\0');
 		return(doschdir(tmp));
 	}
 #endif
@@ -393,12 +205,12 @@ int drive, nodir;
 	if ((bdos(0x0e, drv, 0) & 0xff) < drv
 	|| (drive = (bdos(0x19, 0, 0) & 0xff)) != drv) {
 		bdos(0x0e, olddrv, 0);
-		seterrno(0x0f);		/* Invarid drive */
+		dosseterrno(0x0f);	/* Invarid drive */
 		return(-1);
 	}
 
 	if (!nodir) {
-		path = ".";
+		path = curpath;
 		reg.x.ax = 0x3b00;
 #ifdef	DJGPP
 		dos_putpath(path, 0);
@@ -445,6 +257,7 @@ int getdosver(VOID_A)
 }
 
 /*
+ *	 3: enable LFN access, but shared folder
  *	 2: enable LFN access, but FAT32
  *	 1: enable LFN access
  *	 0: do SFN access
@@ -455,28 +268,31 @@ int getdosver(VOID_A)
  */
 
 int supportLFN(path)
-char *path;
+CONST char *path;
 {
 	struct SREGS sreg;
 	__dpmi_regs reg;
-	char drv[4], buf[128];
+	char drive, drv[4], buf[128];
 
-	if (!path || !(drv[0] = _dospath(path))) drv[0] = getcurdrv();
+#ifdef	DOUBLESLASH
+	if (isdslash(path)) return(3);
+#endif
+	if (!path || !(drive = _dospath(path))) drive = getcurdrv();
+#ifdef	DOUBLESLASH
+	if (drive == '_') return(3);
+#endif
 #ifndef	_NODOSDRIVE
-	if (checkdrive(toupper2(drv[0]) - 'A') > 0) return(-2);
+	if (checkdrive(toupper2(drive) - 'A') > 0) return(-2);
 #endif
 	if (getdosver() < 7) {
 #ifndef	_NODOSDRIVE
-		if ((dosdrive & 1) && islower2(drv[0])) return(-1);
+		if ((dosdrive & 1) && islower2(drive)) return(-1);
 #endif
 		return(-3);
 	}
-	if (isupper2(drv[0])) return(0);
-
-	drv[1] = ':';
-	drv[2] = '\\';
-	drv[3] = '\0';
+	if (isupper2(drive)) return(0);
 
+	VOID_C gendospath(drv, drive, _SC_);
 	reg.x.ax = 0x71a0;
 	reg.x.bx = 0;
 	reg.x.cx = sizeof(buf);
@@ -532,30 +348,28 @@ int drive;
 #ifndef	_NOUSELFN
 # ifndef	_NODOSDRIVE
 static char *NEAR regpath(path, buf)
-char *path, *buf;
+CONST char *path;
+char *buf;
 {
-	char *cp;
+	CONST char *cp;
 	int drive;
 
 	cp = path;
-	if ((drive = _dospath(path))) cp += 2;
-	else drive = getcurdrv();
-	buf[0] = drive;
-	buf[1] = ':';
-	if (!*cp) {
-		buf[2] = '.';
-		buf[3] = '\0';
-	}
-	else if (cp == &(path[2])) return(path);
-	else strcpy(&(buf[2]), cp);
+	if (!(drive = _dospath(path))) drive = getcurdrv();
+	else if (*(cp += 2)) return((char *)path);
+	strcpy2(gendospath(buf, drive, (*cp) ? '\0' : '.'), cp);
 
 	return(buf);
 }
 # endif	/* !_NODOSDRIVE */
 
 char *shortname(path, alias)
-char *path, *alias;
+CONST char *path;
+char *alias;
 {
+# ifndef	_NODOSDRIVE
+	char buf[MAXPATHLEN];
+# endif
 	struct SREGS sreg;
 	__dpmi_regs reg;
 	int i;
@@ -564,15 +378,13 @@ char *path, *alias;
 	if ((i = supportLFN(path)) <= 0) {
 # ifndef	_NODOSDRIVE
 		if (i == -1) {
-			char buf[MAXPATHLEN];
-
 			dependdosfunc = 0;
 			if (dosshortname(regpath(path, buf), alias))
 				return(alias);
 			else if (!dependdosfunc) return(NULL);
 		}
 # endif	/* !_NODOSDRIVE */
-		return(path);
+		return((char *)path);
 	}
 	reg.x.ax = 0x7160;
 	reg.x.cx = 0x8001;
@@ -590,6 +402,9 @@ char *path, *alias;
 # ifndef	BSPATHDELIM
 	adjustpname(alias);
 # endif
+# ifdef	DOUBLESLASH
+	if (isdslash(alias)) return(alias);
+# endif
 
 	if (!path || !(i = _dospath(path))) i = getcurdrv();
 	if (islower2(i)) *alias = tolower2(*alias);
@@ -599,7 +414,8 @@ char *path, *alias;
 #endif	/* !_NOUSELFN */
 
 char *unixrealpath(path, resolved)
-char *path, *resolved;
+CONST char *path;
+char *resolved;
 {
 #ifndef	_NODOSDRIVE
 	char buf[MAXPATHLEN];
@@ -614,6 +430,7 @@ char *path, *resolved;
 	reg.x.cx = 0;
 #else	/* !_NOUSELFN */
 	switch (supportLFN(path)) {
+		case 3:
 		case 2:
 		case 1:
 			reg.x.ax = 0x7160;
@@ -630,7 +447,7 @@ char *path, *resolved;
 			if (doslongname(regpath(path, buf), resolved))
 				return(resolved);
 			else if (!dependdosfunc) {
-				strcpy(resolved, path);
+				strcpy2(resolved, path);
 				return(resolved);
 			}
 /*FALLTHRU*/
@@ -655,6 +472,9 @@ char *path, *resolved;
 #ifndef	BSPATHDELIM
 	adjustpname(resolved);
 #endif
+#ifdef	DOUBLESLASH
+	if (isdslash(resolved)) return(resolved);
+#endif
 
 	if (!path || !(i = _dospath(path))) i = getcurdrv();
 	else path += 2;
@@ -673,12 +493,9 @@ char *path, *resolved;
 			resolved[j + 3] = '\0';
 		}
 		else {
-			resolved[0] = i;
-			resolved[1] = ':';
-			resolved[2] = _SC_;
-			resolved[3] = '\0';
+			VOID_C gendospath(resolved, i, _SC_);
 			if (path && *path == _SC_)
-				strcpy(&(resolved[2]), path);
+				strcpy2(&(resolved[2]), path);
 		}
 	}
 	else *resolved = (isupper2(i))
@@ -712,7 +529,8 @@ char *path;
 	int i;
 
 # ifndef	_NOUSELFN
-	if (supportLFN(path) > 0 && unixrealpath(path, tmp)) strcpy(path, tmp);
+	if (supportLFN(path) > 0 && unixrealpath(path, tmp))
+		strcpy2(path, tmp);
 	else
 # endif
 	for (i = (_dospath(path)) ? 2 : 0; path[i]; i++) {
@@ -731,21 +549,22 @@ char *path;
 
 #ifndef	_NOUSELFN
 char *preparefile(path, alias)
-char *path, *alias;
+CONST char *path;
+char *alias;
 {
 	int i;
 
 	path = duplpath(path);
 	i = supportLFN(path);
 #ifdef	_NODOSDRIVE
-	if (i < 0) return(path);
+	if (i < 0) return((char *)path);
 #else	/* !_NODOSDRIVE */
-	if (i < -2) return(path);
+	if (i < -2) return((char *)path);
 #endif	/* !_NODOSDRIVE */
 
 	if ((alias = shortname(path, alias))) return(alias);
 #ifndef	_NODOSDRIVE
-	else if (i < 0 && errno == EACCES) return(path);
+	else if (i < 0 && errno == EACCES) return((char *)path);
 #endif
 
 	return(NULL);
@@ -826,10 +645,8 @@ int nsect, mode, retry;
 				sect -= drvlist[drive].sect;
 				if ((++head) >= drvlist[drive].head) {
 					head = 0;
-					if ((++cyl) >= drvlist[drive].cyl) {
-						errno = EACCES;
-						return(-1);
-					}
+					if ((++cyl) >= drvlist[drive].cyl)
+						return(seterrno(EACCES));
 				}
 			}
 			n = (nsect -= n);
@@ -877,9 +694,8 @@ int nsect, mode, retry;
 		i = -1;
 	}
 	if (cp != buf) free(cp);
-	errno = EACCES;
 
-	return(-1);
+	return(seterrno(EACCES));
 }
 
 #ifndef	PC98
@@ -897,10 +713,7 @@ int nsect, mode, retry;
 	u_char *cp;
 	int i, j, cy, n, ofs;
 
-	if (!(drvlist[drive].flags & DI_LBA)) {
-		errno = EACCES;
-		return(-1);
-	}
+	if (!(drvlist[drive].flags & DI_LBA)) return(seterrno(EACCES));
 
 	n = nsect;
 	size = (u_long)n * drvlist[drive].sectsize;
@@ -994,9 +807,8 @@ int nsect, mode, retry;
 		i = -1;
 	}
 	if (cp != buf) free(cp);
-	errno = EACCES;
 
-	return(-1);
+	return(seterrno(EACCES));
 }
 #endif	/* !PC98 */
 
@@ -1484,10 +1296,7 @@ int n, size, iswrite;
 	int drv, ret, level;
 
 	drv = toupper2(drive) - 'A';
-	if (drv < 0 || drv > 'Z' - 'A') {
-		errno = EINVAL;
-		return(-1);
-	}
+	if (drv < 0 || drv > 'Z' - 'A') return(seterrno(EINVAL));
 
 	ret = 0;
 #ifdef	FAKEUNINIT
@@ -1530,7 +1339,7 @@ int n, size, iswrite;
 #endif	/* !_NOUSELFN */
 
 static int NEAR dos_findfirst(path, attr, result)
-char *path;
+CONST char *path;
 u_int attr;
 struct dosfind_t *result;
 {
@@ -1550,7 +1359,10 @@ struct dosfind_t *result;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, sizeof(struct dosfind_t));
-	if (int21call(&reg, &sreg) < 0) return(-1);
+	if (int21call(&reg, &sreg) < 0) {
+		if (!errno) errno = ENOENT;
+		return(-1);
+	}
 #ifdef	DJGPP
 	dosmemget(__tb, sizeof(struct dosfind_t), result);
 #endif
@@ -1584,7 +1396,7 @@ struct dosfind_t *result;
 #ifndef	_NOUSELFN
 static int NEAR lfn_findfirst(fdp, path, attr, result)
 u_int *fdp;
-char *path;
+CONST char *path;
 u_int attr;
 struct lfnfind_t *result;
 {
@@ -1602,7 +1414,10 @@ struct lfnfind_t *result;
 	reg.x.dx = PTR_OFF(path, sizeof(struct lfnfind_t));
 	sreg.es = PTR_SEG(result);
 	reg.x.di = PTR_OFF(result, 0);
-	if (int21call(&reg, &sreg) < 0) return(-1);
+	if (int21call(&reg, &sreg) < 0) {
+		if (!errno) errno = ENOENT;
+		return(-1);
+	}
 	*fdp = reg.x.ax;
 #ifdef	DJGPP
 	dosmemget(__tb, sizeof(struct lfnfind_t), result);
@@ -1645,7 +1460,7 @@ u_int fd;
 #endif	/* !_NOUSELFN */
 
 DIR *unixopendir(dir)
-char *dir;
+CONST char *dir;
 {
 	DIR *dirp;
 	char *cp, path[MAXPATHLEN];
@@ -1668,7 +1483,7 @@ char *dir;
 #ifndef	_NOUSELFN
 	if (supportLFN(path) > 0) {
 		dirp -> dd_id = DID_IFLFN;
-		strcpy(cp, "*");
+		strcpy2(cp, "*");
 		dirp -> dd_buf = (char *)malloc(sizeof(struct lfnfind_t));
 		if (!dirp -> dd_buf) {
 			free(dirp);
@@ -1691,7 +1506,7 @@ char *dir;
 #endif	/* !_NOUSELFN */
 	{
 		dirp -> dd_id = DID_IFNORMAL;
-		strcpy(cp, "*.*");
+		strcpy2(cp, "*.*");
 		dirp -> dd_buf = (char *)malloc(sizeof(struct dosfind_t));
 		if (!dirp -> dd_buf) {
 			free(dirp);
@@ -1725,7 +1540,7 @@ char *dir;
 		free(dirp);
 		return(NULL);
 	}
-	strcpy(dirp -> dd_path, dir);
+	strcpy2(dirp -> dd_path, dir);
 
 	return(dirp);
 }
@@ -1749,6 +1564,10 @@ DIR *dirp;
 struct dirent *unixreaddir(dirp)
 DIR *dirp;
 {
+#ifndef	_NOUSELFN
+	struct lfnfind_t *lbufp;
+#endif
+	struct dosfind_t *dbufp;
 	static struct dirent d;
 	char *cp, path[MAXPATHLEN];
 	int i;
@@ -1761,19 +1580,15 @@ DIR *dirp;
 
 #ifndef	_NOUSELFN
 	if ((dirp -> dd_id & DID_IFLFN) && !(dirp -> dd_id & DID_IFLABEL)) {
-		struct lfnfind_t *bufp;
-
-		bufp = (struct lfnfind_t *)(dirp -> dd_buf);
-		strcpy(d.d_name, bufp -> name);
-		strcpy(d.d_alias, bufp -> alias);
+		lbufp = (struct lfnfind_t *)(dirp -> dd_buf);
+		strcpy2(d.d_name, lbufp -> name);
+		strcpy2(d.d_alias, lbufp -> alias);
 	}
 	else
 #endif	/* !_NOUSELFN */
 	{
-		struct dosfind_t *bufp;
-
-		bufp = (struct dosfind_t *)(dirp -> dd_buf);
-		strcpy(d.d_name, bufp -> name);
+		dbufp = (struct dosfind_t *)(dirp -> dd_buf);
+		strcpy2(d.d_name, dbufp -> name);
 		d.d_alias[0] = '\0';
 	}
 
@@ -1793,7 +1608,7 @@ DIR *dirp;
 		if (dirp -> dd_id & DID_IFLFN) {
 			u_int fd;
 
-			strcpy(cp, "*");
+			strcpy2(cp, "*");
 			i = lfn_findfirst(&fd, path, SEARCHATTRS,
 				(struct lfnfind_t *)(dirp -> dd_buf));
 			if (i >= 0) dirp -> dd_fd = fd;
@@ -1802,7 +1617,7 @@ DIR *dirp;
 		else
 #endif	/* !_NOUSELFN */
 		{
-			strcpy(cp, "*.*");
+			strcpy2(cp, "*.*");
 			i = dos_findfirst(path, SEARCHATTRS,
 				(struct dosfind_t *)(dirp -> dd_buf));
 		}
@@ -1847,23 +1662,23 @@ DIR *dirp;
 # ifndef	DJGPP
 /*ARGSUSED*/
 int unixmkdir(path, mode)
-char * path;
+CONST char * path;
 int mode;
 {
 	struct stat st;
 
-	if (unixstat(path, &st) >= 0) {
-		errno = EEXIST;
-		return(-1);
-	}
+	if (unixstat(path, &st) >= 0) return(seterrno(EEXIST));
 
 	return(mkdir(path) ? -1 : 0);
 }
 # endif
 #else	/* !_NOUSELFN */
 int unixunlink(path)
-char *path;
+CONST char *path;
 {
+#ifndef	_NODOSDRIVE
+	char buf[MAXPATHLEN];
+#endif
 	struct SREGS sreg;
 	__dpmi_regs reg;
 	int i;
@@ -1872,8 +1687,6 @@ char *path;
 	i = supportLFN(path);
 #ifndef	_NODOSDRIVE
 	if (i < 0 && i > -3) {
-		char buf[MAXPATHLEN];
-
 		if (dosunlink(regpath(path, buf)) >= 0) return(0);
 	}
 #endif	/* !_NODOSDRIVE */
@@ -1910,7 +1723,7 @@ char *s;
 	s += i;
 	if (!*s) return(1);
 	if (*s == '.') cp = s + 1;
-	else if ((cp = strchr(&(s[1]), '.'))) cp++;
+	else if ((cp = strchr2(&(s[1]), '.'))) cp++;
 
 	for (i = 0; i < 3 && cp && cp[i]; i++) {
 		if (cp[i] == ' ') return(0);
@@ -1924,27 +1737,28 @@ char *s;
 }
 
 static int NEAR unixrenamedir(from, to)
-char *from, *to;
+CONST char *from, *to;
 {
 	DIR *dirp;
 	struct dirent *dp;
+	CONST char *cp;
 	char *fp, *tp, fbuf[MAXPATHLEN], tbuf[MAXPATHLEN];
 	int i;
 
-	strcpy(fbuf, to);
+	strcpy2(fbuf, to);
 	if (!(tp = strrdelim(fbuf, 1))) {
-		tp = to;
-		strcpy(fbuf, ".");
+		cp = to;
+		strcpy2(fbuf, curpath);
 	}
 	else {
 		if (*tp == _SC_) tp++;
 		*tp = '\0';
-		tp = &(to[tp - fbuf]);
+		cp = &(to[tp - fbuf]);
 	}
 	if (!(to = unixrealpath(fbuf, tbuf))
 	|| !(from = unixrealpath(from, fbuf)))
 		return(-1);
-	strcpy(strcatdelim(tbuf), tp);
+	strcpy2(strcatdelim(tbuf), cp);
 
 	for (i = 0; from[i]; i++) if (from[i] != to[i]) break;
 	if (!strdelim(&(from[i]), 0) && !strdelim(&(to[i]), 0)) return(-1);
@@ -1955,43 +1769,44 @@ char *from, *to;
 		return(-1);
 	}
 	i = strlen(from);
-	fp = strcatdelim(from);
-	tp = strcatdelim(to);
+	fp = strcatdelim(fbuf);
+	tp = strcatdelim(tbuf);
 	while ((dp = unixreaddir(dirp))) {
 		if (isdotdir(dp -> d_name)) continue;
-		strcpy(fp, dp -> d_name);
-		strcpy(tp, dp -> d_name);
+		strcpy2(fp, dp -> d_name);
+		strcpy2(tp, dp -> d_name);
 		if (unixrename(from, to) < 0) {
 			unixclosedir(dirp);
 			return(-1);
 		}
 	}
 	unixclosedir(dirp);
-	from[i] = '\0';
+	fbuf[i] = '\0';
 	if (unixrmdir(from) < 0) return(-1);
 
 	return(0);
 }
 
 int unixrename(from, to)
-char *from, *to;
+CONST char *from, *to;
 {
+#ifndef	_NODOSDRIVE
+	char buf1[MAXPATHLEN], buf2[MAXPATHLEN];
+#endif
 	struct dosfind_t dbuf;
 	struct SREGS sreg;
 	__dpmi_regs reg;
 	char fbuf[MAXPATHLEN], tbuf[MAXPATHLEN];
 	int i, ax, f, t;
 
-	strcpy(fbuf, duplpath(from));
+	strcpy2(fbuf, duplpath(from));
 	from = fbuf;
-	strcpy(tbuf, duplpath(to));
+	strcpy2(tbuf, duplpath(to));
 	to = tbuf;
 	f = supportLFN(from);
 	t = supportLFN(to);
 #ifndef	_NODOSDRIVE
 	if (((f < 0 && f > -3) || (t < 0 && t > -3)) && (f != -3 && t != -3)) {
-		char buf1[MAXPATHLEN], buf2[MAXPATHLEN];
-
 		dependdosfunc = 0;
 		if (dosrename(regpath(from, buf1), regpath(to, buf2)) >= 0)
 			return(0);
@@ -2022,7 +1837,7 @@ char *from, *to;
 				break;
 
 			if (t > -3) {
-				if ((f = gendosname(to)) < 0) break;
+				if ((f = gendosname(tbuf)) < 0) break;
 				if (f > 0) {
 					ax = 0x7156;
 					continue;
@@ -2032,16 +1847,18 @@ char *from, *to;
 			return(0);
 		}
 	}
-	errno = EACCES;
 
-	return(-1);
+	return(seterrno(EACCES));
 }
 
 /*ARGSUSED*/
 int unixmkdir(path, mode)
-char *path;
+CONST char *path;
 int mode;
 {
+#ifndef	_NODOSDRIVE
+	char buf[MAXPATHLEN];
+#endif
 	struct SREGS sreg;
 	__dpmi_regs reg;
 	int i;
@@ -2050,8 +1867,6 @@ int mode;
 	i = supportLFN(path);
 #ifndef	_NODOSDRIVE
 	if (i < 0 && i > -3) {
-		char buf[MAXPATHLEN];
-
 		dependdosfunc = 0;
 		if (dosmkdir(regpath(path, buf), mode) >= 0) return(0);
 		else if (!dependdosfunc) return(-1);
@@ -2070,8 +1885,11 @@ int mode;
 }
 
 int unixrmdir(path)
-char *path;
+CONST char *path;
 {
+#ifndef	_NODOSDRIVE
+	char buf[MAXPATHLEN];
+#endif
 	struct SREGS sreg;
 	__dpmi_regs reg;
 	int i;
@@ -2080,8 +1898,6 @@ char *path;
 	i = supportLFN(path);
 #ifndef	_NODOSDRIVE
 	if (i < 0 && i > -3) {
-		char buf[MAXPATHLEN];
-
 		if (dosrmdir(regpath(path, buf)) >= 0) return(0);
 	}
 #endif	/* !_NODOSDRIVE */
@@ -2096,8 +1912,11 @@ char *path;
 }
 
 int unixchdir(path)
-char *path;
+CONST char *path;
 {
+#ifndef	_NODOSDRIVE
+	char buf[MAXPATHLEN];
+#endif
 	struct SREGS sreg;
 	__dpmi_regs reg;
 	int i;
@@ -2107,8 +1926,6 @@ char *path;
 	i = supportLFN(path);
 #ifndef	_NODOSDRIVE
 	if (i == -1) {
-		char buf[MAXPATHLEN];
-
 		if (!(path = preparefile(path, buf))) return(-1);
 	}
 #endif	/* !_NODOSDRIVE */
@@ -2130,6 +1947,7 @@ int size;
 #ifdef	DJGPP
 	char tmp[MAXPATHLEN];
 #endif
+	char *cp;
 	int drive;
 
 	drive = getcurdrv();
@@ -2141,17 +1959,18 @@ int size;
 #endif
 	if (!pathname && !(pathname = (char *)malloc(size))) return(NULL);
 	else {
-		pathname[0] = drive;
-		pathname[1] = ':';
-		pathname[2] = _SC_;
-
-		if (!unixgetcurdir(&(pathname[3]), 0)) return(NULL);
+#ifdef	DOUBLESLASH
+		if (drive == '_') cp = pathname;
+		else
+#endif
+		cp = gendospath(pathname, drive, _SC_);
+		if (!unixgetcurdir(cp, 0)) return(NULL);
 	}
 
 	*pathname = (dos7access & D7_CAPITAL)
 		? toupper2(*pathname) : tolower2(*pathname);
 #ifdef	DJGPP
-	if (unixrealpath(pathname, tmp)) strcpy(pathname, tmp);
+	if (unixrealpath(pathname, tmp)) strcpy2(pathname, tmp);
 #endif
 
 	return(pathname);
@@ -2164,9 +1983,9 @@ u_int attr;
 
 	mode = 0;
 	if (attr & DS_IARCHIVE) mode |= S_ISVTX;
-	if (!(attr & DS_IHIDDEN)) mode |= S_IREAD;
-	if (!(attr & DS_IRDONLY)) mode |= S_IWRITE;
-	if (attr & DS_IFDIR) mode |= (S_IFDIR | S_IEXEC);
+	if (!(attr & DS_IHIDDEN)) mode |= S_IRUSR;
+	if (!(attr & DS_IRDONLY)) mode |= S_IWUSR;
+	if (attr & DS_IFDIR) mode |= (S_IFDIR | S_IXUSR);
 	else if (attr & DS_IFLABEL) mode |= S_IFIFO;
 	else if (attr & DS_IFSYSTEM) mode |= S_IFSOCK;
 	else mode |= S_IFREG;
@@ -2181,8 +2000,8 @@ u_int mode;
 
 	attr = 0;
 	if (mode & S_ISVTX) attr |= DS_IARCHIVE;
-	if (!(mode & S_IREAD)) attr |= DS_IHIDDEN;
-	if (!(mode & S_IWRITE)) attr |= DS_IRDONLY;
+	if (!(mode & S_IRUSR)) attr |= DS_IHIDDEN;
+	if (!(mode & S_IWUSR)) attr |= DS_IRDONLY;
 	if ((mode & S_IFMT) == S_IFSOCK) attr |= DS_IFSYSTEM;
 	else if ((mode & S_IFMT) == S_IFIFO) attr |= DS_IFLABEL;
 
@@ -2226,13 +2045,16 @@ time_t tim;
 #endif	/* !_NOUSELFN */
 
 int unixstatfs(path, buf)
-char *path;
+CONST char *path;
 statfs_t *buf;
 {
 #ifndef	_NOUSELFN
+# ifndef	_NODOSDRIVE
+	char tmp[3 * sizeof(long) + 1];
+# endif
 	struct fat32statfs_t fsbuf;
 	int i;
-#endif
+#endif	/* !_NOUSELFN */
 	struct SREGS sreg;
 	__dpmi_regs reg;
 	int drv, drive;
@@ -2240,26 +2062,21 @@ statfs_t *buf;
 	if (!path || !(drive = _dospath(path))) drive = getcurdrv();
 	drv = toupper2(drive) - 'A';
 
-	if (drv < 0 || drv > 'Z' - 'A') {
-		errno = ENOENT;
-		return(-1);
-	}
+	if (drv < 0 || drv > 'Z' - 'A') return(seterrno(ENOENT));
 
 #ifndef	_NOUSELFN
 	i = supportLFN(path);
 # ifndef	_NODOSDRIVE
 	if (i == -2) {
-		char tmp[sizeof(long) * 3 + 1];
-
 		if (dosstatfs(drive, tmp) < 0) return(-1);
-		buf -> f_bsize = *((u_long *)&(tmp[sizeof(long) * 0]));
-		buf -> f_blocks = *((u_long *)&(tmp[sizeof(long) * 1]));
+		buf -> f_bsize = *((u_long *)&(tmp[0 * sizeof(long)]));
+		buf -> f_blocks = *((u_long *)&(tmp[1 * sizeof(long)]));
 		buf -> f_bfree =
-		buf -> f_bavail = *((u_long *)&(tmp[sizeof(long) * 2]));
+		buf -> f_bavail = *((u_long *)&(tmp[2 * sizeof(long)]));
 	}
 	else
 # endif
-	if (i == 2) {
+	if (i >= 2) {
 		reg.x.ax = 0x7303;
 		reg.x.cx = sizeof(fsbuf);
 # ifdef	DJGPP
@@ -2286,10 +2103,7 @@ statfs_t *buf;
 		reg.x.ax = 0x3600;
 		reg.h.dl = (u_char)drv + 1;
 		int21call(&reg, &sreg);
-		if (reg.x.ax == 0xffff) {
-			errno = ENOENT;
-			return(-1);
-		}
+		if (reg.x.ax == 0xffff) return(seterrno(ENOENT));
 
 		buf -> f_bsize = (u_long)(reg.x.ax) * (u_long)(reg.x.cx);
 		buf -> f_blocks = (u_long)(reg.x.dx);
@@ -2298,67 +2112,88 @@ statfs_t *buf;
 	}
 
 	if (!(buf -> f_blocks) || !(buf -> f_bsize)
-	|| buf -> f_blocks < buf -> f_bavail) {
-		errno = EIO;
-		return(-1);
-	}
+	|| buf -> f_blocks < buf -> f_bavail)
+		return(seterrno(EIO));
 	buf -> f_files = -1L;
 
 	return(0);
 }
 
+/*ARGSUSED*/
+static int NEAR alter_findfirst(path, fdp, dbuf, lbuf)
+CONST char *path;
+u_int *fdp;
+struct dosfind_t *dbuf;
+struct lfnfind_t *lbuf;
+{
+	CONST char *cp;
+	char tmp[MAXPATHLEN];
+	int n, len;
+
+#ifndef	_NOUSELFN
+	if (fdp) n = lfn_findfirst(fdp, path, SEARCHATTRS, lbuf);
+	else
+#endif
+	n = dos_findfirst(path, SEARCHATTRS, dbuf);
+	if (n >= 0) return(n);
+	if (errno != ENOENT) return(-1);
+
+	if ((cp = strrdelim(path, 1))) cp++;
+	else cp = path;
+
+	if (isdotdir(cp) == 1) {
+		len = cp - path;
+		memcpy(tmp, path, len);
+		strcpy2(&(tmp[len]), curpath);
+#ifndef	_NOUSELFN
+		if (fdp) n = lfn_findfirst(fdp, tmp, SEARCHATTRS, lbuf);
+		else
+#endif
+		n = dos_findfirst(tmp, SEARCHATTRS, dbuf);
+	}
+#ifdef	DOUBLESLASH
+	else {
+		if (isdslash(path)) return(-1);
+		else if (_dospath(path)) /*EMPTY*/;
+		else if (getcurdrv() == '_') return(-1);
+#endif
+		n = dos_findfirst(path, DS_IFLABEL, dbuf);
+	}
+
+	return(n);
+}
+
 int unixstat(path, stp)
-char *path;
+CONST char *path;
 struct stat *stp;
 {
 #ifndef	_NOUSELFN
+# ifndef	_NODOSDRIVE
+	char buf[MAXPATHLEN];
+# endif
 	struct lfnfind_t lbuf;
 	u_int fd;
-#endif
+#endif	/* !_NOUSELFN */
 	struct dosfind_t dbuf;
 	int i;
 
+	if (strpbrk(path, "*?")) return(seterrno(ENOENT));
 #ifdef	_NOUSELFN
-	if ((i = dos_findfirst(path, SEARCHATTRS, &dbuf)) < 0) {
-		if (errno && errno != ENOENT) return(-1);
-		if (!strcmp(path, ".."))
-			i = dos_findfirst(".", SEARCHATTRS, &dbuf);
-		else i = dos_findfirst(path, DS_IFLABEL, &dbuf);
-	}
+	i = alter_findfirst(path, NULL, &dbuf, NULL);
 #else	/* !_NOUSELFN */
+	fd = (u_int)-1;
 	i = supportLFN(path);
 # ifndef	_NODOSDRIVE
 	if (i < 0 && i > -3) {
-		char buf[MAXPATHLEN];
-
 		dependdosfunc = 0;
 		if (dosstat(regpath(path, buf), stp) >= 0) return(0);
 		else if (!dependdosfunc) return(-1);
 	}
-# endif	/* !_NODOSDRIVE */
-	if (i <= 0) {
-		fd = (u_int)-1;
-		if ((i = dos_findfirst(path, SEARCHATTRS, &dbuf)) < 0) {
-			if (errno && errno != ENOENT) return(-1);
-			if (!strcmp(path, ".."))
-				i = dos_findfirst(".", SEARCHATTRS, &dbuf);
-			else i = dos_findfirst(path, DS_IFLABEL, &dbuf);
-		}
-	}
-	else {
-		if ((i = lfn_findfirst(&fd, path, SEARCHATTRS, &lbuf)) < 0) {
-			if (errno && errno != ENOENT) return(-1);
-			if (!strcmp(path, ".."))
-				i = lfn_findfirst(&fd, ".",
-					SEARCHATTRS, &lbuf);
-			else i = dos_findfirst(path, DS_IFLABEL, &dbuf);
-		}
-	}
+# endif
+	if (i <= 0) i = alter_findfirst(path, NULL, &dbuf, &lbuf);
+	else i = alter_findfirst(path, &fd, &dbuf, &lbuf);
 #endif	/* !_NOUSELFN */
-	if (i < 0) {
-		if (!errno) errno = ENOENT;
-		return(-1);
-	}
+	if (i < 0) return(-1);
 
 #ifndef	_NOUSELFN
 	if (fd != (u_int)-1) {
@@ -2383,7 +2218,7 @@ struct stat *stp;
 }
 
 int unixchmod(path, mode)
-char *path;
+CONST char *path;
 int mode;
 {
 	struct SREGS sreg;
@@ -2423,8 +2258,8 @@ int mode;
 #ifndef	_NOUSELFN
 #ifdef	USEUTIME
 int unixutime(path, times)
-char *path;
-struct utimbuf *times;
+CONST char *path;
+CONST struct utimbuf *times;
 {
 	time_t t;
 	__dpmi_regs reg;
@@ -2444,8 +2279,8 @@ struct utimbuf *times;
 	}
 #else	/* !USEUTIME */
 int unixutimes(path, tvp)
-char *path;
-struct timeval tvp[2];
+CONST char *path;
+CONST struct timeval *tvp;
 {
 	time_t t;
 	__dpmi_regs reg;
@@ -2465,13 +2300,15 @@ struct timeval tvp[2];
 	}
 #endif	/* !USEUTIME */
 	if (i <= 0) {
-		if ((fd = open(path, O_RDONLY, 0666)) < 0) return(-1);
-		reg.x.ax = 0x5701;
-		reg.x.bx = (u_short)fd;
-		putdostime(&(reg.x.dx), &(reg.x.cx), t);
-		i = int21call(&reg, &sreg);
-		close(fd);
-		return(i);
+		if ((fd = open(path, O_RDONLY, 0666)) >= 0) {
+			reg.x.ax = 0x5701;
+			reg.x.bx = (u_short)fd;
+			putdostime(&(reg.x.dx), &(reg.x.cx), t);
+			i = int21call(&reg, &sreg);
+			close(fd);
+			return(i);
+		}
+		if (i || errno != EACCES) return(-1);
 	}
 
 	reg.x.ax = 0x7143;
@@ -2487,7 +2324,7 @@ struct timeval tvp[2];
 }
 
 int unixopen(path, flags, mode)
-char *path;
+CONST char *path;
 int flags, mode;
 {
 	struct SREGS sreg;
@@ -2537,52 +2374,4 @@ int flags, mode;
 
 	return(open(path, flags, mode));
 }
-
-FILE *unixfopen(path, type)
-char *path, *type;
-{
-	FILE *fp;
-	int i, flags;
-
-	path = duplpath(path);
-	i = supportLFN(path);
-#ifndef	_NODOSDRIVE
-	if (i < 0 && i > -3) {
-		char buf[MAXPATHLEN];
-
-		dependdosfunc = 0;
-		if ((fp = dosfopen(regpath(path, buf), type))) return(fp);
-		else if (!dependdosfunc) return(NULL);
-	}
-#endif	/* !_NODOSDRIVE */
-	if (i <= 0) return(fopen(path, type));
-
-	if (!type) {
-		errno = 0;
-		return(NULL);
-	}
-	switch (*type) {
-		case 'r':
-			if (*(type + 1) != '+') flags = O_RDONLY;
-			else flags = O_RDWR;
-			break;
-		case 'w':
-			if (*(type + 1) != '+') flags = O_WRONLY;
-			else flags = O_RDWR;
-			flags |= O_CREAT | O_TRUNC;
-			break;
-		case 'a':
-			if (*(type + 1) != '+') flags = O_WRONLY;
-			else flags = O_RDWR;
-			flags |= O_APPEND | O_CREAT;
-			break;
-		default:
-			flags = 0;
-			break;
-	}
-
-	if ((i = unixopen(path, flags, 0666)) < 0) return(NULL);
-
-	return(fdopen(i, type));
-}
 #endif	/* !_NOUSELFN */
